@@ -7,24 +7,34 @@ sub acl_security_form
 {
 local $o = $_[0];
 print "<tr> <td valign=top><b>$text{'acl_users'}</b></td>\n";
-print "<td valign=top><select name=users multiple size=4 width=150>\n";
+print "<td valign=top>\n";
+printf "<input type=radio name=users_def value=1 %s> %s\n",
+	$o->{'users'} eq '*' ? 'checked' : '', $text{'acl_uall'};
+printf "<input type=radio name=users_def value=0 %s> %s<br>\n",
+	$o->{'users'} eq '*' ? '' : 'checked', $text{'acl_usel'};
+print "<select name=users multiple size=6 width=150>\n";
 map { $ucan{$_}++ } split(/\s+/, $o->{'users'});
 foreach $u (&list_users()) {
 	printf "<option %s>%s\n",
-		$ucan{$u->{'name'}} || $ucan{'*'} ? 'selected' : '',
+		$ucan{$u->{'name'}} ? 'selected' : '',
 		$u->{'name'};
 	}
+foreach $g (&list_groups()) {
+	printf "<option %s value=%s>%s\n",
+		$ucan{'_'.$g->{'name'}} ? 'selected' : '',
+		'_'.$g->{'name'}, &text('acl_gr', $g->{'name'});
+	}
 print "</select></td>\n";
 
-print "<td valign=top rowspan=3><b>$text{'acl_mods'}</b></td> ",
-      "<td valign=top rowspan=3>\n";
+print "<td valign=top><b>$text{'acl_mods'}</b></td> ",
+      "<td valign=top>\n";
 printf "<input type=radio name=mode value=0 %s> %s&nbsp;\n",
 	$o->{'mode'} == 0 ? 'checked' : '', $text{'acl_all'};
 printf "<input type=radio name=mode value=1 %s> %s<br>\n",
 	$o->{'mode'} == 1 ? 'checked' : '', $text{'acl_own'};
 printf "<input type=radio name=mode value=2 %s> %s<br>\n",
 	$o->{'mode'} == 2 ? 'checked' : '', $text{'acl_sel'};
-print "&nbsp;&nbsp;&nbsp;<select name=mods multiple size=5>\n";
+print "&nbsp;&nbsp;&nbsp;<select name=mods multiple size=6>\n";
 map { $mcan{$_}++ } split(/\s+/, $o->{'mods'});
 foreach $m (&list_modules()) {
 	%minfo = &get_module_info($m);
@@ -36,10 +46,10 @@ print "</select></td> </tr>\n";
 print "<tr> <td><b>$text{'acl_create'}</b></td> <td>\n";
 printf "<input type=radio name=create value=1 %s> $text{'yes'}\n",
 	$o->{'create'} ? 'checked' : '';
-printf "<input type=radio name=create value=0 %s> $text{'no'}</td> </tr>\n",
+printf "<input type=radio name=create value=0 %s> $text{'no'}</td>\n",
 	$o->{'create'} ? '' : 'checked';
 
-print "<tr> <td><b>$text{'acl_delete'}</b></td> <td>\n";
+print "<td><b>$text{'acl_delete'}</b></td> <td>\n";
 printf "<input type=radio name=delete value=1 %s> $text{'yes'}\n",
 	$o->{'delete'} ? 'checked' : '';
 printf "<input type=radio name=delete value=0 %s> $text{'no'}</td> </tr>\n",
@@ -80,24 +90,65 @@ printf "<input type=radio name=lang value=1 %s> $text{'yes'}\n",
 	$o->{'lang'} ? 'checked' : '';
 printf "<input type=radio name=lang value=0 %s> $text{'no'}</td> </tr>\n",
 	$o->{'lang'} ? '' : 'checked';
+
+print "<tr> <td><b>$text{'acl_perms'}</b></td> <td colspan=3>\n";
+printf "<input type=radio name=perms value=1 %s> $text{'acl_perms_1'}\n",
+	$o->{'perms'} ? 'checked' : '';
+printf "<input type=radio name=perms value=0 %s> $text{'acl_perms_0'}</td>\n",
+	$o->{'perms'} ? '' : 'checked';
+print "</tr>\n";
+
+print "<tr> <td><b>$text{'acl_sync'}</b></td> <td>\n";
+printf "<input type=radio name=sync value=1 %s> $text{'yes'}\n",
+	$o->{'sync'} ? 'checked' : '';
+printf "<input type=radio name=sync value=0 %s> $text{'no'}</td>\n",
+	$o->{'sync'} ? '' : 'checked';
+
+print "<td><b>$text{'acl_unix'}</b></td> <td>\n";
+printf "<input type=radio name=unix value=1 %s> $text{'yes'}\n",
+	$o->{'unix'} ? 'checked' : '';
+printf "<input type=radio name=unix value=0 %s> $text{'no'}</td> </tr>\n",
+	$o->{'unix'} ? '' : 'checked';
+
+print "<tr> <td colspan=4><hr></td> </tr>\n";
+
+print "<tr> <td valign=top><b>$text{'acl_groups'}</b></td> <td valign=top>\n";
+printf "<input type=radio name=groups value=1 %s> $text{'yes'}\n",
+	$o->{'groups'} == 1 ? 'checked' : '';
+printf "<input type=radio name=groups value=0 %s> $text{'no'}</td>\n",
+	$o->{'groups'} == 0 ? 'checked' : '';
+
+print "<td valign=top><b>$text{'acl_gassign'}</b></td> <td>\n";
+printf "<input type=radio name=gassign_def value=1 %s> %s\n",
+	$o->{'gassign'} eq '*' ? 'checked' : '', $text{'acl_gall'};
+printf "<input type=radio name=gassign_def value=0 %s> %s<br>\n",
+	$o->{'gassign'} eq '*' ? '' : 'checked', $text{'acl_gsel'};
+print "<select name=gassign multiple size=3 width=150>\n";
+map { $gcan{$_}++ } split(/\s+/, $o->{'gassign'});
+printf "<option value=_none %s>&lt;%s&gt;\n",
+	$gcan{'_none'} ? 'selected' : '', $text{'acl_gnone'};
+foreach $g (&list_groups()) {
+	printf "<option %s>%s\n",
+		$gcan{$g->{'name'}} ? 'selected' : '', $g->{'name'};
+	}
+print "</select></td> </tr>\n";
 }
 
 # acl_security_save(&options)
 # Parse the form for security options for the acl module
 sub acl_security_save
 {
-local @users = &list_users();
-local @sel = split(/\0/, $in{'users'});
-if (scalar(@sel) == scalar(@users)) {
+if ($in{'users_def'}) {
 	$_[0]->{'users'} = '*';
 	}
 else {
-	$_[0]->{'users'} = join(" ", @sel);
+	$_[0]->{'users'} = join(" ", split(/\0/, $in{'users'}));
 	}
 $_[0]->{'mode'} = $in{'mode'};
 $_[0]->{'mods'} = $in{'mode'} == 2 ? join(" ", split(/\0/, $in{'mods'}))
 				   : undef;
 $_[0]->{'create'} = $in{'create'};
+$_[0]->{'groups'} = $in{'groups'};
 $_[0]->{'delete'} = $in{'delete'};
 $_[0]->{'rename'} = $in{'rename'};
 $_[0]->{'acl'} = $in{'acl'};
@@ -105,5 +156,10 @@ $_[0]->{'others'} = $in{'others'};
 $_[0]->{'cert'} = $in{'cert'};
 $_[0]->{'chcert'} = $in{'chcert'};
 $_[0]->{'lang'} = $in{'lang'};
+$_[0]->{'perms'} = $in{'perms'};
+$_[0]->{'gassign'} = $in{'gassign_def'} ? '*' :
+		     join(" ", split(/\0/, $in{'gassign'}));
+$_[0]->{'sync'} = $in{'sync'};
+$_[0]->{'unix'} = $in{'unix'};
 }
 
