@@ -5,6 +5,8 @@ do '../web-lib.pl';
 &init_config();
 %access = &get_module_acl();
 
+$directive_type_count = 18;
+
 @access_types = $access{'types'} eq '*' ? (0 .. 100)
 				        : split(/\s+/, $access{'types'});
 map { $access_types{$_}++ } @access_types;
@@ -23,11 +25,13 @@ if (&read_file("$module_config_directory/site", \%site)) {
 	foreach $m (keys %httpd_modules) {
 		do "./$m.pl";
 		}
-	foreach $d (split(/\s+/, $site{'defines'}),
-		    split(/\s+/, $config{'defines'})) {
+	foreach $d (split(/\s+/, $site{'defines'})) {
 		$httpd_defines{$d}++;
 		}
 	}
+foreach $d (split(/\s+/, $config{'defines'})) {
+	$httpd_defines{$d}++;
+	}
 
 # parse_config_file(handle, lines, file)
 # Parses lines of text from some config file into a data structure. The
@@ -224,10 +228,8 @@ if (@get_config_cache) {
 $conf = $config{'httpd_conf'};
 if (!$conf) { $conf = "$config{'httpd_dir'}/conf/httpd.conf"; }
 if (!-r $conf) { $conf = "$config{'httpd_dir'}/etc/httpd.conf"; }
-open(CONF, $conf) || return undef;
-$lnum = 0;
-@get_config_cache = &parse_config_file(CONF, $lnum,$conf);
-close(CONF);
+return undef if (!-r $conf);
+@get_config_cache = &get_config_file($conf);
 
 # Read main resource and access config files
 $lnum = 0;
@@ -235,18 +237,14 @@ $res = &find_directive("ResourceConfig", \@get_config_cache);
 if (!$res) { $res = $config{'srm_conf'}; }
 if (!$res) { $res = "$config{'httpd_dir'}/conf/srm.conf"; }
 if (!-r $res) { $res = "$config{'httpd_dir'}/etc/srm.conf"; }
-open(CONF, $res);
-push(@get_config_cache, &parse_config_file(CONF, $lnum, $res));
-close(CONF);
+push(@get_config_cache, &get_config_file($res));
 
 $lnum = 0;
 $acc = &find_directive("AccessConfig", \@get_config_cache);
 if (!$acc) { $acc = $config{'access_conf'}; }
 if (!$acc) { $acc = "$config{'httpd_dir'}/conf/access.conf"; }
 if (!-r $acc) { $acc = "$config{'httpd_dir'}/etc/access.conf"; }
-open(CONF, $acc);
-push(@get_config_cache, &parse_config_file(CONF, $lnum, $acc));
-close(CONF);
+push(@get_config_cache, &get_config_file($acc));
 
 # Read extra config files in VirtualHost sections
 @virt = &find_directive_struct("VirtualHost", \@get_config_cache);
@@ -254,34 +252,49 @@ foreach $v (@virt) {
 	$mref = $v->{'members'};
 	if ($res = &find_directive("ResourceConfig", $mref)) {
 		if ($res !~ /^\//) { $res = "$config{'httpd_dir'}/$res"; }
-		$lnum = 0;
-		open(CONF, $res);
-		push(@$mref, &parse_config_file(CONF, $lnum, $res));
-		close(CONF);
+		push(@$mref, &get_config_file($res));
 		}
 	if ($acc = &find_directive("AccessConfig", $mref)) {
 		if ($acc !~ /^\//) { $acc = "$config{'httpd_dir'}/$acc"; }
-		$lnum = 0;
-		open(CONF, $acc);
-		push(@$mref, &parse_config_file(CONF, $lnum, $acc));
-		close(CONF);
+		push(@$mref, &get_config_file($acc));
 		}
 	}
 
 # Read Include'd files
 foreach $inc (&find_directive("Include", \@get_config_cache)) {
 	if ($inc !~ /^\//) { $inc = "$config{'httpd_dir'}/$inc"; }
-	$lnum = 0;
-	open(CONF, $inc);
-	push(@get_config_cache, &parse_config_file(CONF, $lnum, $inc));
-	close(CONF);
+	local @incs = glob($inc);
+	foreach $ginc (@incs) {
+		push(@get_config_cache, &get_config_file($ginc));
+		}
 	}
 
-# Move directives from active IfModule sections to section they are in
-
 return \@get_config_cache;
 }
 
+# get_config_file(filename)
+sub get_config_file
+{
+local @rv;
+if (opendir(DIR, $_[0])) {
+	# Is a directory .. parse all files!
+	local @files = readdir(DIR);
+	closedir(DIR);
+	foreach $f (@files) {
+		next if ($f =~ /^\./);
+		push(@rv, &get_config_file("$_[0]/$f"));
+		}
+	}
+else {
+	# Just a normal config file
+	local $lnum = 0;
+	open(CONF, $_[0]);
+	@rv = &parse_config_file(CONF, $lnum, $_[0]);
+	close(CONF);
+	}
+return @rv;
+}
+
 # get_virtual_config(index)
 sub get_virtual_config
 {
@@ -654,10 +667,11 @@ sub virtual_name
 if ($_[0]) {
 	local $n = &find_directive("ServerName", $_[0]->{'members'});
 	if ($n) {
-		return $_[0]->{'value'} =~ /:(\d+)$/ ? "$n:$1" : $n;
+		return &html_escape($_[0]->{'value'} =~ /:(\d+)$/ ? "$n:$1"
+								  : $n);
 		}
 	else {
-		return $_[0]->{'value'};
+		return &html_escape($_[0]->{'value'});
 		}
 	}
 else { return $_[1] ? "*" : $text{'default_serv'}; }
@@ -669,10 +683,13 @@ sub dir_name
 local($dfm, $mat);
 $_[0]->{'name'} =~ /^(Directory|Files|Location)(Match)?$/;
 $dfm = $1; $mat = $2;
-if ($mat) { return "$dfm regexp <tt>$_[0]->{'words'}->[0]</tt>"; }
-elsif ($_[0]->{'words'}->[0] eq "~")
-	{ return "$dfm regexp <tt>$_[0]->{'words'}->[1]</tt>"; }
-else { return "$dfm <tt>$_[0]->{'words'}->[0]</tt>"; }
+if ($mat) {
+	return "$dfm regexp <tt>".&html_escape($_[0]->{'words'}->[0])."</tt>";
+	}
+elsif ($_[0]->{'words'}->[0] eq "~") {
+	return "$dfm regexp <tt>".&html_escape($_[0]->{'words'}->[1])."</tt>";
+	}
+else { return "$dfm <tt>".&html_escape($_[0]->{'words'}->[0])."</tt>"; }
 }
 
 # list_user_file(file, &user,  &pass)
@@ -789,6 +806,10 @@ elsif ($verstr =~ /Red\s*Hat\s+Secure\/3\.0/i) {
 	# redhat secure server 3.0
 	$ver = 1.39;
 	}
+elsif (`rpm -q apache 2>&1` =~ /^apache-(\d+)\.([\d\.]+)/) {
+	# got version from the RPM
+	$ver = $1; $minor = $2; $minor =~ s/\.//g; $ver .= ".$minor";
+	}
 else {
 	# couldn't get version
 	return (0, undef);
@@ -822,9 +843,7 @@ if (length($txt) > $left) {
 	$txt = substr($txt, 0, $left)." ..";
 	}
 local $txtlen = length($txt);
-$txt =~ s/&/&amp;/g;
-$txt =~ s/</&lt;/g;
-$txt =~ s/>/&gt;/g;
+$txt = &html_escape($txt);
 print " " x $_[2];
 if ($_[3]) {
 	print "<a href=\"$_[3]\">",$txt,"</a>";
@@ -886,5 +905,23 @@ foreach $f (&unique(map { $_->{'file'} } @$conf)) {
 	}
 }
 
+# directive_lines(directive, ...)
+sub directive_lines
+{
+local @rv;
+foreach $d (@_) {
+	next if ($d->{'name'} eq 'dummy');
+	if ($d->{'type'}) {
+		push(@rv, "<$d->{'name'} $d->{'value'}>");
+		push(@rv, &directive_lines(@{$d->{'members'}}));
+		push(@rv, "</$d->{'name'}>");
+		}
+	else {
+		push(@rv, "$d->{'name'} $d->{'value'}");
+		}
+	}
+return @rv;
+}
+
 1;
 
