@@ -23,6 +23,7 @@
 #		14- SSL
 #		15- Perl
 #		16- PHP
+#		17- Vhost aliases
 #  multiple -	Can this directive appear multiple times
 #  global -	Can be used in the global server context
 #  virtual -	Can be used in a VirtualHost section or in the global section
@@ -94,100 +95,100 @@ return ();
 sub edit_MaxClients
 {
 return (1,
-	$text{'core_0'},
-	&opt_input($_[0]->{'value'}, "MaxClients", $text{'core_12'}, 4));
+	$text{'core_maxconc'},
+	&opt_input($_[0]->{'value'}, "MaxClients", $text{'core_default'}, 4));
 }
 sub save_MaxClients
 {
 return &parse_opt("MaxClients", '^\d+$',
-		  $text{'core_6'});
+		  $text{'core_emaxconc'});
 }
 
 sub edit_MaxKeepAliveRequests
 {
 return (1,
-	$text{'core_1'},
-	&opt_input($_[0]->{'value'}, "MaxKeepAliveRequests", $text{'core_13'}, 4));
+	$text{'core_maxkeep'},
+	&opt_input($_[0]->{'value'}, "MaxKeepAliveRequests", $text{'core_default'}, 4));
 }
 sub save_MaxKeepAliveRequests
 {
 return &parse_opt("MaxKeepAliveRequests", '^\d+$',
-		  $text{'core_7'});
+		  $text{'core_emaxkeep'});
 }
 
 sub edit_MaxRequestsPerChild
 {
 return (1,
-	$text{'core_2'},
-	&opt_input($_[0]->{'value'}, "MaxRequestsPerChild", $text{'core_14'}, 5));
+	$text{'core_maxreq'},
+	&opt_input($_[0]->{'value'}, "MaxRequestsPerChild", $text{'core_default'}, 5));
 }
 sub save_MaxRequestsPerChild
 {
 return &parse_opt("MaxRequestsPerChild", '^\d+$',
-		  $text{'core_8'});
+		  $text{'core_emaxreq'});
 }
 
 sub edit_MinSpareServers
 {
 return (1,
-	$text{'core_3'},
-	&opt_input($_[0]->{'value'},"MinSpareServers",$text{'core_15'}, 4));
+	$text{'core_minspare'},
+	&opt_input($_[0]->{'value'},"MinSpareServers",$text{'core_default'}, 4));
 }
 sub save_MinSpareServers
 {
 return &parse_opt("MinSpareServers", '^\d+$',
-		  $text{'core_9'});
+		  $text{'core_eminspare'});
 }
 
 sub edit_MaxSpareServers
 {
 return (1,
-	$text{'core_4'},
-	&opt_input($_[0]->{'value'},"MaxSpareServers",$text{'core_16'}, 4));
+	$text{'core_maxspare'},
+	&opt_input($_[0]->{'value'},"MaxSpareServers",$text{'core_default'}, 4));
 }
 sub save_MaxSpareServers
 {
 return &parse_opt("MaxSpareServers", '^\d+$',
-		  $text{'core_10'});
+		  $text{'core_emaxspare'});
 }
 
 sub edit_StartServers
 {
 return (1,
-	$text{'core_5'},
-	&opt_input($_[0]->{'value'}, "StartServers", $text{'core_17'}, 4));
+	$text{'core_initial'},
+	&opt_input($_[0]->{'value'}, "StartServers", $text{'core_default'}, 4));
 }
 sub save_StartServers
 {
 return &parse_opt("StartServers", '^\d+$',
-		  $text{'core_11'});
+		  $text{'core_einitial'});
 }
 
 sub edit_RLimitCPU
 {
-return &rlimit_input("RLimitCPU", $text{'core_18'}, $_[0]);
+return &rlimit_input("RLimitCPU", $text{'core_cpulimit'}, $_[0]);
 }
 sub save_RLimitCPU
 {
-return &parse_rlimit("RLimitCPU", $text{'core_21'});
+return &parse_rlimit("RLimitCPU", $text{'core_cpulimit2'});
 }
 
 sub edit_RLimitMEM
 {
-return &rlimit_input("RLimitMEM", $text{'core_19'}, $_[0]);
+return &rlimit_input("RLimitMEM", $text{'core_memlimit'}, $_[0]);
 }
 sub save_RLimitMEM
 {
-return &parse_rlimit("RLimitMEM", $text{'core_22'});
+return &parse_rlimit("RLimitMEM", $text{'core_memlimit2'});
 }
 
 sub edit_RLimitNPROC
 {
-return &rlimit_input("RLimitNPROC", $text{'core_20'}, $_[0]);
+return &rlimit_input("RLimitNPROC", $text{'core_proclimit'}, $_[0]);
 }
 sub save_RLimitNPROC
 {
-return &parse_rlimit("RLimitNPROC", $text{'core_23'});
+return &parse_rlimit("RLimitNPROC", $text{'core_proclimit2'});
 }
 
 # rlimit_input(name, desc, value)
@@ -195,17 +196,17 @@ sub rlimit_input
 {
 local(@v, $rv);
 @v = split(/\s+/, $_[2]->{'value'});
-$rv = sprintf "<input type=radio name=$_[0]_mode value=0 %s> $text{'Default'}<br>\n",
+$rv = sprintf "<input type=radio name=$_[0]_mode value=0 %s> $text{'core_default'}<br>\n",
 	@v ? "" : "checked";
 $rv .= sprintf "<input type=radio name=$_[0]_mode value=1 %s>\n",
 	@v == 1 ? "checked" : "";
-$rv .= sprintf "$text{'core_24'}<input name=$_[0]_soft1 size=5 value=\"%s\"><br>\n",
+$rv .= sprintf "$text{'core_slimit'}<input name=$_[0]_soft1 size=5 value=\"%s\"><br>\n",
 	@v == 1 ? $v[0] : "";
 $rv .= sprintf "<input type=radio name=$_[0]_mode value=2 %s>\n",
 	@v == 2 ? "checked" : "";
-$rv .= sprintf "$text{'core_25'}<input name=$_[0]_soft2 size=5 value=\"%s\">\n",
+$rv .= sprintf "$text{'core_slimit'}<input name=$_[0]_soft2 size=5 value=\"%s\">\n",
 	@v == 2 ? $v[0] : "";
-$rv .= sprintf "$text{'core_26'}<input name=$_[0]_hard2 size=5 value=\"%s\"><br>\n",
+$rv .= sprintf "$text{'core_hlimit'}<input name=$_[0]_hard2 size=5 value=\"%s\"><br>\n",
 	@v == 2 ? $v[1] : "";
 return (1, $_[1], $rv);
 }
@@ -216,14 +217,14 @@ sub parse_rlimit
 if ($in{"$_[0]_mode"} == 0) { return ( [ ] ); }
 elsif ($in{"$_[0]_mode"} == 1) {
 	$in{"$_[0]_soft1"} =~ /^(\d+|max)$/ ||
-		&error($in{"$_[0]_soft1"}.&text('core_27', $_[1]));
+		&error(&text('core_eslimit', $in{"$_[0]_soft1"}, $_[1]));
 	return ( [ $in{"$_[0]_soft1"} ] );
 	}
 elsif ($in{"$_[0]_mode"} == 2) {
 	$in{"$_[0]_soft2"} =~ /^(\d+|max)$/ ||
-		&error($in{"$_[0]_soft2"}.&text('core_28', $_[1]));
+		&error(&text('core_eslimit', $in{"$_[0]_soft2"}, $_[1]));
 	$in{"$_[0]_hard2"} =~ /^(\d+|max)$/ ||
-		&error($in{"$_[0]_hard2"}.&text('core_29', $_[1]));
+		&error(&text('core_ehlimit', $in{"$_[0]_hard2"}, $_[1]));
 	return ( [ $in{"$_[0]_soft2"}." ".$in{"$_[0]_hard2"} ] );
 	}
 }
@@ -256,23 +257,23 @@ else {
 	push(@plist, undef);
 	}
 $port = @$pref ? $pref->[@$pref-1]->{'value'} : 80;
-$inp = "<b>$text{'core_30'}</b> <input name=Port size=6 value=\"$port\"><br>\n";
+$inp = "<b>$text{'core_dport'}</b> <input name=Port size=6 value=\"$port\"><br>\n";
 $inp .=	"<table border>\n".
-	"<tr $tb> <td><b>$text{'core_31'}</b></td> <td><b>$text{'core_32'}</b></td> </tr>\n";
+	"<tr $tb> <td><b>$text{'core_address'}</b></td> <td><b>$text{'core_port'}</b></td> </tr>\n";
 for($i=0; $i<@blist+1; $i++) {
 	$inp .= sprintf
 		"<tr><td><input type=radio name=BindAddress_def_$i value=2 %s>".
-		" $text{'core_201'} <input type=radio name=BindAddress_def_$i value=1 %s>".
-		" $text{'core_202'} <input type=radio name=BindAddress_def_$i value=0 %s> ".
+		" $text{'core_none'} <input type=radio name=BindAddress_def_$i value=1 %s>".
+		" $text{'core_all'} <input type=radio name=BindAddress_def_$i value=0 %s> ".
 		"<input name=BindAddress_$i size=20 value=\"%s\"></td>",
 		$blist[$i] ? "" : "checked",
 		$blist[$i] eq "*" ? "checked" : "",
 		$blist[$i] && $blist[$i] ne "*" ? "checked" : "",
 		$blist[$i] eq "*" ? "" : $blist[$i];
-	$inp .= "<td>".&opt_input($plist[$i], "Port_$i", "$text{'core_34'}", 5)."</td>";
+	$inp .= "<td>".&opt_input($plist[$i], "Port_$i", "$text{'core_default'}", 5)."</td>";
 	}
 $inp .= "</table>\n";
-return (2, $text{'core_35'}, $inp);
+return (2, $text{'core_listen'}, $inp);
 }
 sub save_BindAddress_Listen_Port
 {
@@ -286,16 +287,16 @@ for($i=0; defined($in{"Port_$i"}); $i++) {
 
 	if ($bdef) { push(@blist, "*"); }
 	elsif ($b =~ /^\S+$/ && gethostbyname($b)) { push(@blist, $b); }
-	else { &error(&text('core_36', $b)); }
+	else { &error(&text('core_eaddress', $b)); }
 
 	if ($pdef) { push(@plist, undef); }
 	elsif ($p =~ /^\d+$/) { push(@plist, $p); }
-	else { &error(&text('core_37', $p)); }
+	else { &error(&text('core_eport', $p)); }
 	}
-if (!@blist) { &error($text{'core_38'}); }
+if (!@blist) { &error($text{'core_eoneaddr'}); }
 
 # Return directives
-$in{'Port'} =~ /^\d+$/ || &error($text{'core_39'});
+$in{'Port'} =~ /^\d+$/ || &error($text{'core_edefport'});
 if (@blist == 1 && !$plist[0]) {
 	# Only one address, and the default port
 	return ( $blist[0] eq "*" ? [ ] : [ $blist[0] ], [ ], [ $in{'Port'} ] );
@@ -320,8 +321,8 @@ $kref = $_[0]; $mref = $_[1];
 if ($_[2]->{'version'} >= 1.2) {
 	# two separate directives for keep-alives
 	$inp = sprintf
-		"<input type=radio name=KeepAlive_def value=0 %s> $text{'core_203'}\n".
-		"<input type=radio name=KeepAlive_def value=1 %s> $text{'core_204'}\n".
+		"<input type=radio name=KeepAlive_def value=0 %s> $text{'core_none'}\n".
+		"<input type=radio name=KeepAlive_def value=1 %s> $text{'core_default'}\n".
 		"<input type=radio name=KeepAlive_def value=2 %s> ".
 		"<input name=KeepAlive size=5 value=\"%s\">",
 		$kref->{'value'} =~ /off/i ? "checked" : "",
@@ -332,8 +333,8 @@ if ($_[2]->{'version'} >= 1.2) {
 else {
 	# only one directive
 	$inp = sprintf
-		"<input type=radio name=KeepAlive_def value=0 %s> $text{'core_205'}\n".
-		"<input type=radio name=KeepAlive_def value=1 %s> $text{'core_206'}\n".
+		"<input type=radio name=KeepAlive_def value=0 %s> $text{'core_none'}\n".
+		"<input type=radio name=KeepAlive_def value=1 %s> $text{'core_default'}\n".
 		"<input type=radio name=KeepAlive_def value=2 %s> ".
 		"<input name=KeepAlive size=5 value=\"%s\">",
 		$kref->{'value'} eq "0" ? "checked" : "",
@@ -341,7 +342,7 @@ else {
 		$kref->{'value'} ? "checked" : "",
 		$kref->{'value'} ? $kref->{'value'} : "";
 	}
-return (1, $text{'core_40'}, $inp);
+return (1, $text{'core_multi'}, $inp);
 }
 sub save_KeepAlive_MaxKeepAliveRequests
 {
@@ -350,7 +351,7 @@ if ($_[0]->{'version'} >= 1.2) {
 	if ($in{'KeepAlive_def'} == 0) { return ( [ "off" ], [ ] ); }
 	elsif ($in{'KeepAlive_def'} == 1) { return ( [ "on" ], [ ] ); }
 	elsif ($in{'KeepAlive'} !~ /^\d+$/) {
-		&error(&text('core_41', $in{'KeepAlive'}));
+		&error(&text('core_ekeep', $in{'KeepAlive'}));
 		}
 	else { return ( [ "on" ], [ $in{'KeepAlive'} ] ); }
 	}
@@ -359,7 +360,7 @@ else {
 	if ($in{'KeepAlive_def'} == 0) { return ( [ 0 ], [ ] ); }
 	elsif ($in{'KeepAlive_def'} == 1) { return ( [ ], [ ] ); }
 	elsif ($in{'KeepAlive'} !~ /^\d+$/) {
-		&error(&text('core_78', $in{'KeepAlive'}));
+		&error(&text('core_ekeep', $in{'KeepAlive'}));
 		}
 	else { return ( [ $in{'KeepAlive'} ], [ ] ); }
 	}
@@ -368,43 +369,43 @@ else {
 sub edit_KeepAliveTimeout
 {
 return (1,
-	$text{'core_55'},
-	&opt_input($_[0]->{'value'}, "KeepAliveTimeout", $text{'core_72'}, 5));
+	$text{'core_keeptout'},
+	&opt_input($_[0]->{'value'}, "KeepAliveTimeout", $text{'core_default'}, 5));
 }
 sub save_KeepAliveTimeout
 {
 return &parse_opt("KeepAliveTimeout", '^\d+$',
-		  $text{'core_67'});
+		  $text{'core_ekeeptout'});
 }
 
 sub edit_ListenBacklog
 {
 return (1,
-	$text{'core_56'},
-	&opt_input($_[0]->{'value'}, "ListenBacklog", $text{'core_73'}, 4));
+	$text{'core_lqueue'},
+	&opt_input($_[0]->{'value'}, "ListenBacklog", $text{'core_default'}, 4));
 }
 sub save_ListenBacklog
 {
 return &parse_opt("ListenBacklog", '^\d+$',
-		  $text{'core_68'});
+		  $text{'core_elqueue'});
 }
 
 sub edit_SendBufferSize
 {
 return (1,
-	$text{'core_57'},
-	&opt_input($_[0]->{'value'}, "SendBufferSize", $text{'core_74'}, 4));
+	$text{'core_bufsize'},
+	&opt_input($_[0]->{'value'}, "SendBufferSize", $text{'core_osdefault'}, 4));
 }
 sub save_SendBufferSize
 {
 return &parse_opt("SendBufferSize", '^\d+$',
-		  $text{'core_69'});
+		  $text{'core_ebufsize'});
 }
 
 sub edit_ServerAdmin
 {
-return (1, $text{'core_58'},
-        &opt_input($_[0]->{'value'}, "ServerAdmin", $text{'core_75'}, 25));
+return (1, $text{'core_admin'},
+        &opt_input($_[0]->{'value'}, "ServerAdmin", $text{'core_noadmin'}, 25));
 }
 sub save_ServerAdmin
 {
@@ -414,20 +415,20 @@ return &parse_opt("ServerAdmin");
 sub edit_TimeOut
 {
 return (1,
-	$text{'core_59'},
-	&opt_input($_[0]->{'value'}, "TimeOut", $text{'core_76'}, 4));
+	$text{'core_rtout'},
+	&opt_input($_[0]->{'value'}, "TimeOut", $text{'core_default'}, 4));
 }
 sub save_TimeOut
 {
 return &parse_opt("TimeOut", '^\d+$',
-		  $text{'core_70'});
+		  $text{'core_ertout'});
 }
 
 sub edit_UseCanonicalName
 {
-return (1, $text{'core_60'},
+return (1, $text{'core_bhostname'},
         &choice_input($_[0]->{'value'}, "UseCanonicalName",
-	               "", "$text{'core_42'},off", "$text{'core_43'},on", "$text{'core_44'},"));
+	               "", "$text{'yes'},off", "$text{'no'},on", "$text{'core_default'},"));
 }
 sub save_UseCanonicalName
 {
@@ -437,14 +438,14 @@ return &parse_choice("UseCanonicalName", "");
 sub edit_HostNameLookups
 {
 if ($_[1]->{'version'} >= 1.3) {
-	return (1, $text{'core_61'},
+	return (1, $text{'core_lookup'},
 		&choice_input($_[0]->{'value'}, "HostNameLookups", "",
-		       "$text{'core_45'},off", "$text{'core_46'},on", "$text{'core_47'},double", "$text{'core_48'},"));
+		       "$text{'no'},off", "$text{'yes'},on", "$text{'core_ltwice'},double", "$text{'core_default'},"));
 	}
 else {
-	return (1, $text{'core_62'},
+	return (1, $text{'core_lookup'},
 		&choice_input($_[0]->{'value'}, "HostNameLookups", "",
-			      "$text{'core_49'},on", "$text{'core_50'},off", "$text{'core_51'},"));
+			      "$text{'yes'},on", "$text{'no'},off", "$text{'core_default'},"));
 	}
 }
 sub save_HostNameLookups
@@ -454,9 +455,9 @@ return &parse_choice("HostNameLookups", "");
 
 sub edit_IdentityCheck
 {
-return (1, $text{'core_63'},
+return (1, $text{'core_useauth'},
 	&choice_input($_[0]->{'value'}, "IdentityCheck", "",
-		      "$text{'core_52'},on", "$text{'core_53'},off", "$text{'core_54'},"));
+		      "$text{'yes'},on", "$text{'no'},off", "$text{'core_default'},"));
 }
 sub save_IdentityCheck
 {
@@ -467,7 +468,7 @@ sub edit_ServerAlias
 {
 local($a, @al);
 foreach $a (@{$_[0]}) { push(@al, split(/\s+/, $a->{'value'})); }
-return (1, $text{'core_64'},
+return (1, $text{'core_altnames'},
 	sprintf "<textarea name=ServerAlias rows=3 cols=25>%s</textarea>\n",
 		join("\n", @al) );
 }
@@ -481,19 +482,19 @@ else { return ( [ ] ); }
 
 sub edit_ServerName
 {
-return (1, $text{'core_65'},
-	&opt_input($_[0]->{'value'}, "ServerName", $text{'core_77'}, 25));
+return (1, $text{'core_hostname'},
+	&opt_input($_[0]->{'value'}, "ServerName", $text{'core_auto'}, 25));
 }
 sub save_ServerName
 {
-return &parse_opt("ServerName", '^\S+$', $text{'core_71'});
+return &parse_opt("ServerName", '^\S+$', $text{'core_ehostname'});
 }
 
 sub edit_NameVirtualHost
 {
 local(@nv, $nv);
 foreach $nv (@{$_[0]}) { push(@nv, $nv->{'value'}); }
-return (1, $text{'core_66'},
+return (1, $text{'core_virtaddr'},
 	"<textarea name=NameVirtualHost rows=3 cols=30>".
 	join("\n", @nv)."</textarea>");
 }
@@ -502,10 +503,10 @@ sub save_NameVirtualHost
 local(@nv, $nv, $addr);
 @nv = split(/\s+/, $in{'NameVirtualHost'});
 foreach $nv (@nv) {
-	if ($nv =~ /^(\S+):(\d+)$/) { $addr = $1; }
+	if ($nv =~ /^(\S+):(\d+|\*)$/) { $addr = $1; }
 	else { $addr = $nv; }
-	if (!gethostbyname($addr)) {
-		&error(&text('core_79', $addr));
+	if (!gethostbyname($addr) && $addr ne '*') {
+		&error(&text('core_evirtaddr', $addr));
 		}
 	}
 if (@nv) { return ( \@nv ); }
@@ -517,30 +518,30 @@ else { return ( [ ] ); }
 sub edit_AccessFileName
 {
 return (1,
-        $text{'core_103'},
-        &opt_input($_[0]->{'value'}, "AccessFileName", $text{'core_110'}, 20));
+        $text{'core_optfile'},
+        &opt_input($_[0]->{'value'}, "AccessFileName", $text{'core_default'}, 20));
 }
 sub save_AccessFileName
 {
 if ($_[0]->{'version'} < 1.3) {
 	return &parse_opt("AccessFileName", '^(\S+)$',
-	                  $text{'core_108'});
+	                  $text{'core_eoptfile'});
 	}
 else {
 	return &parse_opt("AccessFileName", '\S',
-			     $text{'core_109'});
+			     $text{'core_enoopt'});
 	}
 }
 
 @AllowOverride_v = ("AuthConfig", "FileInfo", "Indexes", "Limit", "Options");
-@AllowOverride_d = ("$text{'core_80'}", "$text{'core_81'}",
-                    "$text{'core_82'}", "$text{'core_83'}",
-                    "$text{'core_84'}");
+@AllowOverride_d = ("$text{'core_auth'}", "$text{'core_mime'}",
+                    "$text{'core_indexing'}", "$text{'core_hostacc'}",
+                    "$text{'core_diropts'}");
 sub edit_AllowOverride
 {
 local($rv, @ov, %over, $rv);
 $rv = &choice_input($_[0] ? 0 : 1, "AllowOverride_def", 1,
-		 "$text{'core_85'},1", "$text{'core_86'},0");
+		 "$text{'core_default'},1", "$text{'core_filesel'},0");
 $rv .= "<table border><tr><td>\n";
 if ($_[0]) { @ov = split(/\s+/, $_[0]->{'value'}); }
 else { @ov = ("All"); }
@@ -555,7 +556,7 @@ for($i=0; $i<@AllowOverride_v; $i++) {
 	        $AllowOverride_d[$i];
 	}
 $rv .= "</td></tr></table>\n";
-return (1, $text{'core_104'}, $rv);
+return (1, $text{'core_overr'}, $rv);
 }
 sub save_AllowOverride
 {
@@ -572,9 +573,9 @@ else { return ( [ join(' ', @ov) ] ); }
 sub edit_ContentDigest
 {
 return (1,
-	$text{'core_105'},
+	$text{'core_genmd5'},
 	&choice_input($_[0]->{'value'}, "ContentDigest", "",
-		      "$text{'core_87'},on", "$text{'core_88'},off", "$text{'core_89'},"));
+		      "$text{'yes'},on", "$text{'no'},off", "$text{'core_default'},"));
 }
 sub save_ContentDigest
 {
@@ -583,14 +584,14 @@ return &parse_choice("ContentDigest", "");
 
 sub edit_DocumentRoot
 {
-return (2, $text{'core_106'},
-	&opt_input($_[0]->{'words'}->[0], "DocumentRoot", $text{'core_111'}, 40).
+return (2, $text{'core_docroot'},
+	&opt_input($_[0]->{'words'}->[0], "DocumentRoot", $text{'core_default'}, 40).
 	&file_chooser_button("DocumentRoot", 1));
 }
 sub save_DocumentRoot
 {
 $in{'DocumentRoot_def'} || -d $in{'DocumentRoot'} ||
-	&error(&text('core_112', $in{'DocumentRoot'}));
+	&error(&text('core_enodoc', $in{'DocumentRoot'}));
 return &parse_opt("DocumentRoot");
 }
 
@@ -599,12 +600,12 @@ sub edit_Options
 local(@po, @o, $o, %opts, $opts, $po, @pon, $i);
 @po = ("ExecCGI", "FollowSymLinks", "Includes", "IncludesNOEXEC",
        "Indexes", "MultiViews", "SymLinksIfOwnerMatch");
-@pon = ("$text{'core_90'}", "$text{'core_91'}",
-	"$text{'core_92'}", "$text{'core_93'}",
-	"$text{'core_94'}", "$text{'core_95'}",
-	"$text{'core_96'}");
-$opts = &choice_input($_[0] ? 0 : 1, "Options_def", 1, "$text{'core_97'},1",
-		      "$text{'core_98'},0")."<br>\n";
+@pon = ("$text{'core_execcgi'}", "$text{'core_flink'}",
+	"$text{'core_inclexe'}", "$text{'core_incl'}",
+	"$text{'core_genind'}", "$text{'core_genmview'}",
+	"$text{'core_flinkmatch'}");
+$opts = &choice_input($_[0] ? 0 : 1, "Options_def", 1, "$text{'core_default'},1",
+		      "$text{'core_optsel'},0")."<br>\n";
 @o = split(/\s+/, $_[0]->{'value'});
 foreach $o (split(/\s+/, $_[0]->{'value'})) {
 	if ($o =~ /^\+(.*)$/) { $opts{$1} = 2; }
@@ -619,24 +620,24 @@ if ($opts{'All'}) {
 		}
 	}
 $opts .= "<table border>\n";
-$opts .= "<tr $tb> <td><b>$text{'core_195'}</b></td> <td><b>$text{'core_196'}</b></td>\n";
-$opts .= "<td><b>$text{'core_197'}</b></td> </tr>\n";
+$opts .= "<tr $tb> <td><b>$text{'core_option'}</b></td> <td><b>$text{'core_setdir'}</b></td>\n";
+$opts .= "<td><b>$text{'core_merge'}</b></td> </tr>\n";
 for($i=0; $i<@po; $i++) {
 	$po = $po[$i];
 	$opts .= "<tr $cb> <td><b>$pon[$i]</b></td> <td>\n";
-	$opts .= sprintf "<input type=radio name=$po value=1 %s> $text{'core_99'}\n",
+	$opts .= sprintf "<input type=radio name=$po value=1 %s> $text{'yes'}\n",
 			$opts{$po}==1 ? "checked" : "";
-	$opts .= sprintf "<input type=radio name=$po value=0 %s> $text{'core_100'}\n",
+	$opts .= sprintf "<input type=radio name=$po value=0 %s> $text{'no'}\n",
 			$opts{$po}==0 ? "checked" : "";
 	$opts .= "</td> <td>\n";
-	$opts .= sprintf "<input type=radio name=$po value=2 %s> $text{'core_101'}\n",
+	$opts .= sprintf "<input type=radio name=$po value=2 %s> $text{'core_enable'}\n",
 			$opts{$po}==2 ? "checked" : "";
-	$opts .= sprintf "<input type=radio name=$po value=3 %s> $text{'core_102'}\n",
+	$opts .= sprintf "<input type=radio name=$po value=3 %s> $text{'core_disable'}\n",
 			$opts{$po}==3 ? "checked" : "";
 	$opts .= "</td> </tr>\n";
 	}
 $opts .= "</table>\n";
-return (2, $text{'core_107'}, $opts);
+return (2, $text{'core_diropts'}, $opts);
 }
 sub save_Options
 {
@@ -655,22 +656,22 @@ return @rv ? ( [ join(' ', @rv) ] ) : ( [ "None" ] );
 sub edit_ServerPath
 {
 return (2,
-	$text{'core_117'},
-	&opt_input($_[0]->{'value'}, "ServerPath", $text{'core_120'}, 40).
+	$text{'core_virtpath'},
+	&opt_input($_[0]->{'value'}, "ServerPath", $text{'core_default'}, 40).
 	&file_chooser_button("ServerPath", 1));
 }
 sub save_ServerPath
 {
 return &parse_opt("ServerPath", '^\/\S*$',
-		  $text{'core_119'});
+		  $text{'core_evirtpath'});
 }
 
 sub edit_ServerSignature
 {
-return (1, $text{'core_118'},
+return (1, $text{'core_footer'},
 	&choice_input($_[0]->{'value'}, "ServerSignature", undef,
-		      "$text{'core_113'},Email", "$text{'core_114'},On",
-		      "$text{'core_115'},Off", "$text{'core_116'},"));
+		      "$text{'core_sigemail'},Email", "$text{'core_signame'},On",
+		      "$text{'core_signone'},Off", "$text{'core_default'},"));
 }
 sub save_ServerSignature
 {
@@ -682,13 +683,13 @@ return &parse_choice("ServerSignature", undef);
 sub edit_DefaultType
 {
 return (1,
-	$text{'core_121'},
-	&opt_input($_[0]->{'value'}, "DefaultType", $text{'core_123'}, 20));
+	$text{'core_defmime'},
+	&opt_input($_[0]->{'value'}, "DefaultType", $text{'core_default'}, 20));
 }
 sub save_DefaultType
 {
 return &parse_opt("DefaultType", '^(\S+)\/(\S+)$',
-		  $text{'core_122'});
+		  $text{'core_edefmime'});
 }
 
 #########################################################################
@@ -697,8 +698,8 @@ sub edit_AuthName
 {
 my $val = $_[1]->{'version'} >= 1.3 ? $_[0]->{'words'}->[0]
 				    : $_[0]->{'value'};
-return (1, $text{'core_124'},
-        &opt_input($val, "AuthName", $text{'core_126'}, 25));
+return (1, $text{'core_realm'},
+        &opt_input($val, "AuthName", $text{'core_default'}, 25));
 }
 sub save_AuthName
 {
@@ -716,7 +717,7 @@ foreach $a ("", "Basic", "Digest") {
 	        lc($_[0]->{'value'}) eq lc($a) ? "selected" : "";
 	}
 $rv .= "</select>";
-return (1, $text{'core_125'}, $rv);
+return (1, $text{'core_authtype'}, $rv);
 }
 sub save_AuthType
 {
@@ -732,21 +733,21 @@ $rv = sprintf
       "<input type=radio name=require_mode value=0 %s> $text{'default'}<br>\n",
       $mode ? "" : "checked";
 $rv .= sprintf
-      "<input type=radio name=require_mode value=1 %s> $text{'core_198'}:\n",
+      "<input type=radio name=require_mode value=1 %s> $text{'core_users'}:\n",
       $mode eq "user" ? "checked" : "";
 $rv .= sprintf
       "<input name=require_user size=20 value=\"%s\"><br>\n",
       $mode eq "user" ? $list : "";
 $rv .= sprintf
-      "<input type=radio name=require_mode value=2 %s> $text{'core_199'}:\n",
+      "<input type=radio name=require_mode value=2 %s> $text{'core_groups'}:\n",
       $mode eq "group" ? "checked" : "";
 $rv .= sprintf
       "<input name=require_group size=20 value=\"%s\"><br>\n",
       $mode eq "group" ? $list : "";
 $rv .= sprintf
-      "<input type=radio name=require_mode value=3 %s> $text{'core_200'}:<br>\n",
+      "<input type=radio name=require_mode value=3 %s> $text{'core_allusers'}:<br>\n",
       $mode eq "valid-user" ? "checked" : "";
-return (1, $text{'core_130'}, $rv);
+return (1, $text{'core_authlog'}, $rv);
 }
 sub save_require
 {
@@ -758,9 +759,9 @@ else { return ( [ "valid-user" ] ); }
 
 sub edit_Satisfy
 {
-return (1, $text{'core_131'},
-	&choice_input_vert($_[0]->{'value'}, "Satisfy", "", "$text{'core_127'},",
-			   "$text{'core_128'},all","$text{'core_129'},any"));
+return (1, $text{'core_satisfy'},
+	&choice_input_vert($_[0]->{'value'}, "Satisfy", "", "$text{'core_default'},",
+			   "$text{'core_authall'},all","$text{'core_authany'},any"));
 }
 sub save_Satisfy
 {
@@ -771,93 +772,94 @@ return &parse_choice("Satisfy", "");
 # Misc. directives
 sub edit_CoreDumpDirectory
 {
-return (1, $text{'core_137'},
-	 &opt_input($_[0]->{'value'}, "CoreDumpDirectory", $text{'core_155'}, 20).
+return (1, $text{'core_coredir'},
+	 &opt_input($_[0]->{'value'}, "CoreDumpDirectory", $text{'core_sroot'}, 20).
 	 &file_chooser_button("CoreDumpDirectory", 1));
 }
 sub save_CoreDumpDirectory
 {
-return &parse_opt("CoreDumpDirectory", '^\S+$', $text{'core_147'});
+return &parse_opt("CoreDumpDirectory", '^\S+$', $text{'core_ecore'});
 }
 
 sub edit_LockFile
 {
-return (1, $text{'core_138'},
-        &opt_input($_[0]->{'value'}, "LockFile", $text{'core_156'}, 20).
+return (1, $text{'core_lockfile'},
+        &opt_input($_[0]->{'value'}, "LockFile", $text{'core_default'}, 20).
         &file_chooser_button("LockFile", 0));
 }
 sub save_LockFile
 {
-return &parse_opt("LockFile", '^\S+', $text{'core_148'});
+return &parse_opt("LockFile", '^\S+', $text{'core_elock'});
 }
 
 sub edit_LimitRequestBody
 {
-return (1, $text{'core_139'},
-	&opt_input($_[0]->{'value'}, "LimitRequestBody", $text{'core_157'}, 8)."bytes");
+return (1, $text{'core_maxbody'},
+	&opt_input($_[0]->{'value'}, "LimitRequestBody", $text{'core_default'}, 8)
+		.$text{'bytes'});
 }
 sub save_LimitRequestBody
 {
-return &parse_opt("LimitRequestBody", '^\d+$', $text{'core_149'});
+return &parse_opt("LimitRequestBody", '^\d+$', $text{'core_ebody'});
 }
 
 sub edit_LimitRequestFields
 {
-return (1, $text{'core_140'},
-	&opt_input($_[0]->{'value'}, "LimitRequestFields", $text{'core_158'}, 6));
+return (1, $text{'core_maxhead'},
+	&opt_input($_[0]->{'value'}, "LimitRequestFields", $text{'core_default'}, 6));
 }
 sub save_LimitRequestFields
 {
-return &parse_opt("LimitRequestFields", '^\d+$', $text{'core_150'});
+return &parse_opt("LimitRequestFields", '^\d+$', $text{'core_ehead'});
 }
 
 sub edit_LimitRequestFieldsize
 {
-return (1, $text{'core_141'},
-	&opt_input($_[0]->{'value'}, "LimitRequestFieldsize", $text{'core_159'}, 6));
+return (1, $text{'core_maxshead'},
+	&opt_input($_[0]->{'value'}, "LimitRequestFieldsize", $text{'core_default'}, 6));
 }
 sub save_LimitRequestFieldsize
 {
-return &parse_opt("LimitRequestFieldsize", '^\d+$', $text{'core_151'});
+return &parse_opt("LimitRequestFieldsize", '^\d+$', $text{'core_eshead'});
 }
 
 sub edit_LimitRequestLine
 {
-return (1, $text{'core_142'},
-	&opt_input($_[0]->{'value'}, "LimitRequestLine", $text{'core_160'}, 6));
+return (1, $text{'core_maxline'},
+	&opt_input($_[0]->{'value'}, "LimitRequestLine", $text{'core_default'}, 6));
 }
 sub save_LimitRequestLine
 {
-return &parse_opt("LimitRequestLine", '^\d+$', $text{'core_152'});
+return &parse_opt("LimitRequestLine", '^\d+$', $text{'core_eline'});
 }
 
 sub edit_PidFile
 {
-return (1, $text{'core_143'},
-        &opt_input($_[0]->{'value'}, "PidFile", $text{'core_161'}, 20).
+return (1, $text{'core_pid'},
+        &opt_input($_[0]->{'value'}, "PidFile", $text{'core_default'}, 20).
         &file_chooser_button("PidFile", 0));
 }
 sub save_PidFile
 {
-return &parse_opt("PidFile", '^\S+$', $text{'core_153'});
+return &parse_opt("PidFile", '^\S+$', $text{'core_epid'});
 }
 
 sub edit_ScoreBoardFile
 {
-return (1, $text{'core_144'},
-        &opt_input($_[0]->{'value'}, "ScoreBoardFile", $text{'core_162'}, 20).
+return (1, $text{'core_memsco'},
+        &opt_input($_[0]->{'value'}, "ScoreBoardFile", $text{'core_default'}, 20).
         &file_chooser_button("ScoreBoardFile", 0));
 }
 sub save_ScoreBoardFile
 {
-return &parse_opt("ScoreBoardFile", '^\S+$', $text{'core_154'});
+return &parse_opt("ScoreBoardFile", '^\S+$', $text{'core_escore'});
 }
 
 sub edit_ServerType
 {
-return (1, $text{'core_145'},
+return (1, $text{'core_exec'},
         &choice_input($_[0]->{'value'}, "ServerType", "standalone",
-                      "$text{'core_132'},standalone", "$text{'core_133'},inetd"));
+                      "$text{'core_salone'},standalone", "$text{'core_inetd'},inetd"));
 }
 sub save_ServerType
 {
@@ -866,10 +868,14 @@ return &parse_choice("ServerType", "standalone");
 
 sub edit_ServerTokens
 {
-return (1, $text{'core_146'},
-	&select_input($_[0]->{'value'}, "ServerTokens", "Full",
-		      "$text{'core_134'},Full",
-		      "$text{'core_135'},OS", "$text{'core_136'},Min"));
+local $v = $_[0]->{'value'};
+$v = "ProductOnly" if ($v eq "Prod");
+return (1, $text{'core_header'},
+	&select_input($v, "ServerTokens", "Full",
+		      "$text{'core_verosmod'},Full",
+		      "$text{'core_veros'},OS", "$text{'core_ver'},Min",
+		      $_[1]->{'version'} >= 1.313 ?
+			("$text{'core_product'},ProductOnly") : ()));
 }
 sub save_ServerTokens
 {
@@ -881,25 +887,25 @@ return &parse_select("ServerTokens", "Full");
 sub edit_Group
 {
 local($rv, @ginfo);
-$rv = sprintf "<input type=radio name=Group value=0 %s> Default&nbsp;\n",
+$rv = sprintf "<input type=radio name=Group value=0 %s>$text{'core_default'}&nbsp;\n",
        $_[0] ? "" : "checked";
-$rv .= sprintf "<input type=radio name=Group value=1 %s>$text{'core_163'}\n",
+$rv .= sprintf "<input type=radio name=Group value=1 %s>$text{'core_group'}\n",
         $_[0] && $_[0]->{'value'} !~ /^#/ ? "checked" : "";
 $rv .= sprintf "<input name=Group_name size=8 value=\"%s\"> %s&nbsp;\n",
 	$_[0]->{'value'} !~ /^#/ ? $_[0]->{'value'} : "",
 	&group_chooser_button("Group_name", 0);
-$rv .= sprintf "<input type=radio name=Group value=2 %s>$text{'core_164'}\n",
+$rv .= sprintf "<input type=radio name=Group value=2 %s>$text{'core_gid'}\n",
         $_[0]->{'value'} =~ /^#/ ? "checked" : "";
 $rv .= sprintf "<input name=Group_id size=6 value=\"%s\">\n",
 	 $_[0]->{'value'} =~ /^#(.*)$/ ? $1 : "";
-return (2, $text{'core_167'}, $rv);
+return (2, $text{'core_asgroup'}, $rv);
 }
 sub save_Group
 {
 if ($in{'Group'} == 0) { return ( [ ] ); }
 elsif ($in{'Group'} == 1) { return ( [ $in{'Group_name'} ] ); }
 elsif ($in{'Group_id'} !~ /^\-?\d+$/) {
-	&error(&text('core_169', $in{'Group_id'}));
+	&error(&text('core_euid', $in{'Group_id'}));
 	}
 else { return ( [ "#$in{'Group_id'}" ] ); }
 }
@@ -907,25 +913,25 @@ else { return ( [ "#$in{'Group_id'}" ] ); }
 sub edit_User
 {
 local($rv, @uinfo);
-$rv = sprintf "<input type=radio name=User value=0 %s> Default&nbsp;\n",
+$rv = sprintf "<input type=radio name=User value=0 %s>$text{'core_default'}&nbsp;\n",
        $_[0] ? "" : "checked";
-$rv .= sprintf "<input type=radio name=User value=1 %s>$text{'core_165'}\n",
+$rv .= sprintf "<input type=radio name=User value=1 %s>$text{'core_user'}\n",
         $_[0] && $_[0]->{'value'} !~ /^#/ ? "checked" : "";
 $rv .= sprintf "<input name=User_name size=8 value=\"%s\"> %s&nbsp;\n",
 	$_[0]->{'value'} !~ /^#/ ? $_[0]->{'value'} : "",
 	&user_chooser_button("User_name", 0);
-$rv .= sprintf "<input type=radio name=User value=2 %s>$text{'core_166'}\n",
+$rv .= sprintf "<input type=radio name=User value=2 %s>$text{'core_uid'}\n",
         $_[0]->{'value'} =~ /^#/ ? "checked" : "";
 $rv .= sprintf "<input name=User_id size=6 value=\"%s\">\n",
 	 $_[0]->{'value'} =~ /^#(.*)$/ ? $1 : "";
-return (2, $text{'core_168'}, $rv);
+return (2, $text{'core_asuser'}, $rv);
 }
 sub save_User
 {
 if ($in{'User'} == 0) { return ( [ ] ); }
 elsif ($in{'User'} == 1) { return ( [ $in{'User_name'} ] ); }
 elsif ($in{'User_id'} !~ /^\-?\d+$/) {
-	&error(&text('core_170', $in{'User_id'}));
+	&error(&text('core_egid', $in{'User_id'}));
 	}
 else { return ( [ "#$in{'User_id'}" ] ); }
 }
@@ -936,8 +942,8 @@ sub edit_ErrorDocument
 {
 local($rv, $len, $i);
 $rv = "<table border width=100%>\n";
-$rv .= "<tr $tb> <td><b>$text{'core_173'}</b></td> <td><b>$text{'core_174'}</b></td> ".
-       "<td><b>$text{'core_175'}</b></td> </tr>\n";
+$rv .= "<tr $tb> <td><b>$text{'core_error'}</b></td> <td><b>$text{'core_resp'}</b></td> ".
+       "<td><b>$text{'core_urlmsg'}</b></td> </tr>\n";
 $len = @{$_[0]} + 1;
 for($i=0; $i<$len; $i++) {
 	$v = $_[0]->[$i]->{'value'};
@@ -949,14 +955,14 @@ for($i=0; $i<$len; $i++) {
 	$rv .= "<tr $cb>\n";
 	$rv .= "<td><input name=ErrorDocument_code_$i size=3 value=$code></td>\n";
 	$rv .= "<td><input type=radio name=ErrorDocument_type_$i value=0 ".
-	       ($type==0 ? "checked" : "").">$text{'core_171'}\n";
+	       ($type==0 ? "checked" : "").">$text{'core_tourl'}\n";
 	$rv .= "<input type=radio name=ErrorDocument_type_$i value=1 ".
-	       ($type==1 ? "checked" : "").">$text{'core_172'}</td>\n";
+	       ($type==1 ? "checked" : "").">$text{'core_mesg'}</td>\n";
 	$rv .= "<td><input name=ErrorDocument_url_$i size=40 value=\"$url\"></td>\n";
 	$rv .= "</tr>\n";
 	}
 $rv .= "</table>\n";
-return (2, $text{'core_176'}, $rv);
+return (2, $text{'core_custom'}, $rv);
 }
 sub save_ErrorDocument
 {
@@ -964,9 +970,9 @@ local($i, $code, $url, @rv);
 for($i=0; defined($in{"ErrorDocument_code_$i"}); $i++) {
 	$code = $in{"ErrorDocument_code_$i"}; $url = $in{"ErrorDocument_url_$i"};
 	if ($code !~ /\S/ && $url !~ /\S/) { next; }
-	$code =~ /^\d\d\d$/ || &error(&text('core_177', $code));
+	$code =~ /^\d\d\d$/ || &error(&text('core_eerror', $code));
 	if ($in{"ErrorDocument_type_$i"} == 0) {
-		$url =~ /^\S+$/ || &error(&text('core_178', $url));
+		$url =~ /^\S+$/ || &error(&text('core_eurl', $url));
 		push(@rv, "$code $url");
 		}
 	elsif ($_[0]->{'version'} >= 1.2) { push(@rv, "$code \"$url"); }
@@ -980,8 +986,8 @@ return ( \@rv );
 sub edit_ErrorLog
 {
 if ($_[1]->{'version'} < 1.3) {
-	return (1, $text{'core_183'},
-		&opt_input($_[0]->{'value'}, "ErrorLog", $text{'core_186'}, 20).
+	return (1, $text{'core_errfile'},
+		&opt_input($_[0]->{'value'}, "ErrorLog", $text{'core_default'}, 20).
 		&file_chooser_button("ErrorLog", 0));
 	}
 else {
@@ -991,27 +997,27 @@ else {
 		   $v =~ /^\|/ ? 1 : 0;
 	$rv = sprintf "<input type=radio name=ErrorLog_type value=3 %s>\n",
 		$t == 3 ? "checked" : "";
-	$rv .= "$text{'core_179'}";
+	$rv .= "$text{'core_default'}";
 	$rv .= sprintf "<input type=radio name=ErrorLog_type value=2 %s>\n",
 		$t == 2 ? "checked" : "";
-	$rv .= "$text{'core_180'}<br>\n";
+	$rv .= "$text{'core_syslog'}<br>\n";
 	$rv .= sprintf "<input type=radio name=ErrorLog_type value=0 %s>\n",
 		$t == 0 ? "checked" : "";
-	$rv .= sprintf "$text{'core_181'}<input name=ErrorLog_file size=25 value='%s'>\n",
+	$rv .= sprintf "$text{'core_filelog'}<input name=ErrorLog_file size=25 value='%s'>\n",
 		$t == 0 ? $v : "";
 	$rv .= sprintf "<input type=radio name=ErrorLog_type value=1 %s>\n",
 		$t == 1 ? "checked" : "";
-	$rv .= sprintf "$text{'core_182'}<input name=ErrorLog_prog size=25 value='%s'><br>\n",
+	$rv .= sprintf "$text{'core_proglog'}<input name=ErrorLog_prog size=25 value='%s'><br>\n",
 		$t == 1 ? substr($v, 1) : "";
-	return (2, $text{'core_184'}, $rv);
+	return (2, $text{'core_logto'}, $rv);
 	}
 }
 sub save_ErrorLog
 {
 if ($_[0]->{'version'} < 1.3) {
 	$in{'ErrorLog_def'} || &allowed_auth_file($in{'ErrorLog'}) ||
-		&error($text{'core_187'});
-	return &parse_opt("ErrorLog", '^\S+$', $text{'core_185'});
+		&error($text{'core_edirlog'});
+	return &parse_opt("ErrorLog", '^\S+$', $text{'core_efilelog'});
 	}
 else {
 	if ($in{'ErrorLog_type'} == 3) {
@@ -1019,16 +1025,16 @@ else {
 		}
 	elsif ($in{'ErrorLog_type'} == 0) {
 		$in{'ErrorLog_file'} =~ /\S/ ||
-		    &error($text{'core_188'});
+		    &error($text{'core_efilemiss'});
 		&allowed_auth_file($in{'ErrorLog_file'}) ||
-		    &error($text{'core_189'});
+		    &error($text{'core_edirlog'});
 		return ( [ $in{'ErrorLog_file'} ] );
 		}
 	elsif ($in{'ErrorLog_type'} == 1) {
 		$in{'ErrorLog_prog'} =~ /\S/ ||
-		    &error($text{'core_190'});
+		    &error($text{'core_eprogmiss'});
 		$access{'pipe'} ||
-		    &error($text{'core_191'});
+		    &error($text{'core_eperm'});
 		return ( [ "\"|$in{'ErrorLog_prog'}\"" ] );
 		}
 	else {
@@ -1046,10 +1052,10 @@ local($mods, @allmods, $d, %mods, $m, $i, $rv);
 @allmods = grep { !/^core$/ } @$mods;
 local $conf = &get_config();
 foreach $d (&find_directive_struct("LoadModule", $conf)) {
-	if ($d->{'words'}->[1] =~ /(mod_\S+)\.so/) {
+	if ($d->{'words'}->[1] =~ /(mod_\S+)\.(so|dll)/) {
 		push(@allmods, $1);
 		}
-	elsif ($d->{'words'}->[1] =~ /lib([^\/\s]+)\.so/) {
+	elsif ($d->{'words'}->[1] =~ /lib([^\/\s]+)\.(so|dll)/) {
 		push(@allmods, "mod_$1");
 		}
 	}
@@ -1064,7 +1070,7 @@ if (@{$_[0]}) {
 	}
 else { foreach $m (@allmods) { $mods{$m} = "checked"; } }
 $rv = &choice_input(@{$_[0]} ? 1 : 0, "ClearModuleList", 1,
-		    "$text{'core_192'},0", "$text{'core_193'},1")."<br>\n";
+		    "$text{'core_allmod'},0", "$text{'core_selmod'},1")."<br>\n";
 $rv .= "<table>\n";
 foreach $m (@allmods) {
 	if ($i%4 == 0) { $rv .= "<tr>\n"; }
@@ -1072,7 +1078,7 @@ foreach $m (@allmods) {
 	if ($i++%4 == 3) { $rv .= "</tr>\n"; }
 	}
 $rv .= "</table>\n";
-return (2, $text{'core_194'}, $rv);
+return (2, $text{'core_actmod'}, $rv);
 }
 sub save_ClearModuleList_AddModule
 {
