@@ -14,7 +14,7 @@ local($d, @w, $i, @type, @mode, @what, $rv);
 foreach $d (@{$_[0]}, @{$_[1]}) {
 	@w = split(/\s+/, $d->{'value'});
 	for($i=1; $i<@w; $i++) {
-		push(@type, $d->{'name'} eq "allow" ? 1 : 2);
+		push(@type, lc($d->{'name'}) eq "allow" ? 1 : 2);
 		push(@what, $w[$i]);
 		if ($w[$i] =~ /^env=(\S+)$/) {
 			$what[$#what] = $1;
@@ -29,31 +29,31 @@ foreach $d (@{$_[0]}, @{$_[1]}) {
 		}
 	}
 push(@type, ""); push(@what, ""); push(@mode, 0);
-$rv = "<i>$text{'mod_access_0'}</i>\n".
+$rv = "<i>$text{'mod_access_order'}</i>\n".
       &choice_input($_[2]->[0]->{'value'}, "order", "",
-       "$text{'mod_access_1'},deny,allow", "$text{'mod_access_2'},allow,deny",
-       "$text{'mod_access_3'},mutual-failure", "$text{'mod_access_4'},")."<br>\n";
+       "$text{'mod_access_denyallow'},deny,allow", "$text{'mod_access_allowdeny'},allow,deny",
+       "$text{'mod_access_mutual'},mutual-failure", "$text{'mod_access_default'},")."<br>\n";
 $rv .= "<table border>\n".
-       "<tr $tb> <td><b>$text{'mod_access_5'}</b></td> <td><b>$text{'mod_access_6'}</b></td> </tr>\n";
-@sels = ("$text{'mod_access_7'},0", "$text{'mod_access_8'},1",
-	 "$text{'mod_access_9'},2", "$text{'mod_access_10'},3");
+       "<tr $tb> <td><b>$text{'mod_access_action'}</b></td> <td><b>$text{'mod_access_cond'}</b></td> </tr>\n";
+@sels = ("$text{'mod_access_all'},0", "$text{'mod_access_host'},1",
+	 "$text{'mod_access_ip'},2", "$text{'mod_access_pip'},3");
 if ($_[3]->{'version'} >= 1.3) {
-	push(@sels, "$text{'mod_access_11'},4",
-		    "$text{'mod_access_12'},5");
+	push(@sels, "$text{'mod_access_mask'},4",
+		    "$text{'mod_access_cidr'},5");
 	}
 if ($_[3]->{'version'} >= 1.2) {
-	push(@sels, "$text{'mod_access_13'},6");
+	push(@sels, "$text{'mod_access_var'},6");
 	}
 for($i=0; $i<@type; $i++) {
 	$rv .= "<tr $cb> <td>".&select_input($type[$i], "allow_type_$i", "",
-	       ",0", "Allow,1", "Deny,2")."</td>\n";
+	       ",0", "$text{'mod_access_allow'},1", "$text{'mod_access_deny'},2")."</td>\n";
 	$rv .= "<td>".&select_input($mode[$i], "allow_mode_$i", "0", @sels);
 	$rv .= sprintf "<input name=allow_what_$i size=20 value=\"%s\"></td>\n",
 	        $mode[$i] ? $what[$i] : "";
 	$rv .= "</tr>\n";
 	}
 $rv .= "</table>\n";
-return (2, "$text{'mod_access_14'}", $rv);
+return (2, "$text{'mod_access_restr'}", $rv);
 }
 sub save_allow_deny_order
 {
@@ -63,22 +63,22 @@ for($i=0; defined($type = $in{"allow_type_$i"}); $i++) {
 	if (!$type) { next; }
 	if ($mode == 0) { $what = "all"; }
 	elsif ($mode == 2 && !&check_ipaddress($what)) {
-		&error(&text('mod_access_15', $what));
+		&error(&text('mod_access_eip', $what));
 		}
 	elsif ($mode == 3 && $what !~ /^[0-9\.]+$/) {
-		&error(&text('mod_access_16', $what));
+		&error(&text('mod_access_epip', $what));
 		}
 	elsif ($mode == 4 && ($what !~ /^([0-9\.]+)\/([0-9\.]+)$/ ||
 	       !&check_ipaddress($1) || !&check_ipaddress($2))) {
-		&error(&text('mod_access_17', $what));
+		&error(&text('mod_access_emask', $what));
 		}
 	elsif ($mode == 5 && ($what !~ /^([0-9\.]+)\/([0-9]+)$/ ||
 	       !&check_ipaddress($1) || $2 > 32)) {
-		&error(&text('mod_access_18', $what));
+		&error(&text('mod_access_ecidr', $what));
 		}
 	elsif ($mode == 6) {
 		$what =~ /^\S+$/ ||
-			&error(&text('mod_access_19', $what));
+			&error(&text('mod_access_evar', $what));
 		$what = "env=$what";
 		}
 	if ($type == 1) { push(@allow, "from $what"); }
