@@ -20,7 +20,7 @@ sub alias_input
 {
 local($rv, $len, $i, $from, $to);
 $rv = "<table border>\n".
-      "<tr $tb> <td><b>From</b></td> <td><b>To</b></td> </tr>\n";
+      "<tr $tb> <td><b>$text{'mod_alias_from'}</b></td> <td><b>$text{'mod_alias_to'}</b></td> </tr>\n";
 $len = @{$_[0]} + 1;
 for($i=0; $i<$len; $i++) {
 	$from = $_[0]->[$i]->{'words'}->[0];
@@ -36,14 +36,15 @@ return (2, $_[2], $rv);
 
 %alias_statmap = ("permanent", 301,  "temp", 302,
 		  "seeother", 303,   "gone", 410);
-$url_regexp = '^(http|ftp|gopher|https|mailto|telnet):(\S+)$';
+#$url_regexp = '^(http|ftp|gopher|https|mailto|telnet):(\S+)$';
+$url_regexp = '^\S+$';
 
 # alias_status_input(array, name, title)
 sub alias_status_input
 {
 local($rv, $len, $i, $from, $to, $stat);
 $rv = "<table border>\n".
-      "<tr $tb> <td><b>From</b></td> <td><b>Status</b></td> <td><b>To</b></td> </tr>\n";
+      "<tr $tb> <td><b>$text{'mod_alias_from'}</b></td> <td><b>$text{'mod_alias_status'}</b></td> <td><b>$text{'mod_alias_to'}</b></td> </tr>\n";
 $len = @{$_[0]} + 1;
 for($i=0; $i<$len; $i++) {
 	if ($_[0]->[$i]->{'words'}->[0] =~ /^(permanent|temp|seeother|gone|\d+)$/) {
@@ -75,8 +76,8 @@ $re = $_[2];
 for($i=0; defined($in{"$_[0]_from_$i"}); $i++) {
 	$from = $in{"$_[0]_from_$i"}; $to = $in{"$_[0]_to_$i"};
 	if ($from !~ /\S/ && $to !~ /\S/) { next; }
-	if ($from !~ /^\S+$/) { &error(&text('mod_alias_0', $from, $_[1])); }
-	if ($to !~ /$re/) { &error(&text('mod_alias_1', $to, $_[1])); }
+	if ($from !~ /^\S+$/) { &error(&text('mod_alias_efrom', $from, $_[1])); }
+	if ($to !~ /$re/) { &error(&text('mod_alias_edest', $to, $_[1])); }
 	push(@rv, "$from $to");
 	}
 return ( \@rv );
@@ -92,11 +93,11 @@ for($i=0; defined($in{"$_[0]_from_$i"}); $i++) {
 	$to = $in{"$_[0]_to_$i"};
 	$stat = $in{"$_[0]_stat_$i"};
 	if ($from !~ /\S/ && $to !~ /\S/) { next; }
-	if ($from !~ /^\S+$/) { &error(&text('mod_alias_11', $from, $_[1])); }
-	if ($stat !~ /^(\d*)$/) { &error(&text('mod_alias_12', $stat)); }
+	if ($from !~ /^\S+$/) { &error(&text('mod_alias_efrom', $from, $_[1])); }
+	if ($stat !~ /^(\d*)$/) { &error(&text('mod_alias_estatus', $stat)); }
 	if (!$stat || $stat >= 300 && $stat <= 399) {
 		if ($to !~ /$re/) {
-			&error(&text('mod_alias_13', $to, $_[1]));
+			&error(&text('mod_alias_edest', $to, $_[1]));
 			}
 		}
 	else { $to = ""; }
@@ -108,80 +109,80 @@ return ( \@rv );
 
 sub edit_Alias
 {
-return &alias_input($_[0], "Alias", "$text{'mod_alias_2'}");
+return &alias_input($_[0], "Alias", $text{'mod_alias_alias'});
 }
 sub save_Alias
 {
-return &parse_alias("Alias", "document alias", '^\S+$');
+return &parse_alias("Alias", $text{'mod_alias_alias2'}, '^\S+$');
 }
 
 sub edit_AliasMatch
 {
-return &alias_input($_[0], "AliasMatch", "$text{'mod_alias_3'}");
+return &alias_input($_[0], "AliasMatch", $text{'mod_alias_regexp'});
 }
 sub save_AliasMatch
 {
-return &parse_alias("AliasMatch", "regexp document alias", '^\S+$');
+return &parse_alias("AliasMatch", $text{'mod_alias_regexp2'}, '^\S+$');
 }
 
 sub edit_Redirect
 {
 if ($_[1]->{'version'} >= 1.2) {
-	return &alias_status_input($_[0], "Redirect", "$text{'mod_alias_4'}");
+	return &alias_status_input($_[0], "Redirect", $text{'mod_alias_redir'});
 	}
-else { return &alias_input($_[0], "Redirect", "$text{'mod_alias_5'}"); }
+else { return &alias_input($_[0], "Redirect", $text{'mod_alias_redir'}); }
 }
 sub save_Redirect
 {
 if ($_[0]->{'version'} >= 1.2) {
-	return &parse_alias_status("Redirect", "URL redirect", $url_regexp);
+	return &parse_alias_status("Redirect", $text{'mod_alias_redir2'}, $url_regexp);
 	}
-else { return &parse_alias("Redirect", "URL redirect", $url_regexp); }
+else { return &parse_alias("Redirect", $text{'mod_alias_redir2'}, $url_regexp); }
 }
 
 sub edit_RedirectMatch
 {
-return &alias_status_input($_[0], "RedirectMatch", "$text{'mod_alias_6'}");
+return &alias_status_input($_[0], "RedirectMatch", $text{'mod_alias_rredir'});
 }
 sub save_RedirectMatch
 {
-return &parse_alias_status("RedirectMatch", "regexp URL redirect", $url_regexp);
+return &parse_alias_status("RedirectMatch", $text{'mod_alias_rredir2'}, $url_regexp);
 }
 
 sub edit_RedirectTemp
 {
-return &alias_input($_[0], "RedirectTemp", "$text{'mod_alias_7'}");
+return &alias_input($_[0], "RedirectTemp", $text{'mod_alias_tredir'});
 }
 sub save_RedirectTemp
 {
-return &parse_alias("RedirectTemp", "temporary URL redirect", $url_regexp);
+return &parse_alias("RedirectTemp", $text{'mod_alias_tredir2'}, $url_regexp);
 }
 
 sub edit_RedirectPermanent
 {
-return &alias_input($_[0], "RedirectPermanent", "$text{'mod_alias_8'}");
+return &alias_input($_[0], "RedirectPermanent", $text{'mod_alias_predir'});
 }
 sub save_RedirectPermanent
 {
-return &parse_alias("RedirectPermanent", "permantent URL redirect",$url_regexp);
+return &parse_alias("RedirectPermanent", $text{'mod_alias_predir2'}, $url_regexp);
 }
 
 sub edit_ScriptAlias
 {
-return &alias_input($_[0], "ScriptAlias", "$text{'mod_alias_9'}");
+return &alias_input($_[0], "ScriptAlias", $text{'mod_alias_cgi'});
 }
 sub save_ScriptAlias
 {
-return &parse_alias("ScriptAlias", "CGI directory alias", '^\S+$');
+return &parse_alias("ScriptAlias", $text{'mod_alias_cgi2'}, '^\S+$');
 }
 
 sub edit_ScriptAliasMatch
 {
-return &alias_input($_[0], "ScriptAliasMatch", "$text{'mod_alias_10'}");
+return &alias_input($_[0], "ScriptAliasMatch", $text{'mod_alias_rcgi'});
 }
 sub save_ScriptAliasMatch
 {
-return &parse_alias("ScriptAliasMatch", "regexp CGI directory alias", '^\S+$');
+return &parse_alias("ScriptAliasMatch", $text{'mod_alias_rcgi2'}, '^\S+$');
 }
 
 1;
