@@ -19,27 +19,27 @@ return &make_directives($rv, $_[0], "mod_autoindex");
 sub edit_IndexOrderDefault
 {
 local $rv = sprintf
-	"<input type=radio name=IndexOrderDefault_def value=1 %s> $text{'mod_autoindex_0'}\n",
+	"<input type=radio name=IndexOrderDefault_def value=1 %s> $text{'mod_autoindex_default'}\n",
 	$_[0] ? "" : "checked";
 $rv .= sprintf "<input type=radio name=IndexOrderDefault_def value=0 %s>\n",
 	$_[0] ? "checked" : "";
 $rv .= "<select name=IndexOrderDefault_asc>\n";
-$rv .= sprintf "<option %s>$text{'mod_autoindex_1'}\n",
+$rv .= sprintf "<option %s>$text{'mod_autoindex_asc'}\n",
 		$_[0]->{'words'}->[0] eq "Ascending" ? "selected" : "";
-$rv .= sprintf "<option %s>$text{'mod_autoindex_2'}\n",
+$rv .= sprintf "<option %s>$text{'mod_autoindex_descend'}\n",
 		$_[0]->{'words'}->[0] eq "Descending" ? "selected" : "";
 $rv .= "</select>\n";
 $rv .= "<select name=IndexOrderDefault_what>\n";
-$rv .= sprintf "<option %s>$text{'mod_autoindex_3'}\n",
+$rv .= sprintf "<option %s>$text{'mod_autoindex_name'}\n",
 		$_[0]->{'words'}->[1] eq "Name" ? "selected" : "";
-$rv .= sprintf "<option %s>$text{'mod_autoindex_4'}\n",
+$rv .= sprintf "<option %s>$text{'mod_autoindex_date'}\n",
 		$_[0]->{'words'}->[1] eq "Date" ? "selected" : "";
-$rv .= sprintf "<option %s>$text{'mod_autoindex_5'}\n",
+$rv .= sprintf "<option %s>$text{'mod_autoindex_size'}\n",
 		$_[0]->{'words'}->[1] eq "Size" ? "selected" : "";
-$rv .= sprintf "<option %s>$text{'mod_autoindex_6'}\n",
+$rv .= sprintf "<option %s>$text{'mod_autoindex_desc'}\n",
 		$_[0]->{'words'}->[1] eq "Description" ? "selected" : "";
 $rv .= "</select>\n";
-return (2, "$text{'mod_autoindex_7'}", $rv);
+return (2, "$text{'mod_autoindex_sort'}", $rv);
 }
 sub save_IndexOrderDefault
 {
