@@ -16,11 +16,11 @@ return ("imap-file");
 
 sub edit_ImapMenu
 {
-return (2, "$text{'mod_imap_0'}",
-        &select_input($_[0]->{'value'}, "ImapMenu", "", "$text{'mod_imap_1'},",
-         "$text{'mod_imap_2'},none", "$text{'mod_imap_3'},formatted",
-         "$text{'mod_imap_4'},semiformatted",
-         "$text{'mod_imap_5'},unformatted"));
+return (2, "$text{'mod_imap_action'}",
+        &select_input($_[0]->{'value'}, "ImapMenu", "", "$text{'mod_imap_default'},",
+         "$text{'mod_imap_dogefurl'},none", "$text{'mod_imap_form'},formatted",
+         "$text{'mod_imap_semiform'},semiformatted",
+         "$text{'mod_imap_unform'},unformatted"));
 }
 sub save_ImapMenu
 {
@@ -32,20 +32,20 @@ sub edit_ImapDefault
 local($rv, $v);
 if ($_[0]->{'value'} =~ /^\S+:/) { $v = "url"; }
 else { $v = $_[0]->{'value'}; }
-$rv = &select_input($v, "ImapDefault", "", "$text{'mod_imap_6'},",
-       "$text{'mod_imap_7'},error",  "$text{'mod_imap_8'},nocontent",
-       "$text{'mod_imap_9'},map", "$text{'mod_imap_10'},referer",
-       "$text{'mod_imap_11'},url");
+$rv = &select_input($v, "ImapDefault", "", "$text{'mod_imap_default'},",
+       "$text{'mod_imap_disperr'},error",  "$text{'mod_imap_donoth'},nocontent",
+       "$text{'mod_imap_goimap'},map", "$text{'mod_imap_goref'},referer",
+       "$text{'mod_imap_gourl'},url");
 $rv .= sprintf "&nbsp;<input name=ImapDefault_url size=30 value=\"%s\">\n",
         $v eq "url" ? $_[0]->{'value'} : "";
-return (2, "$text{'mod_imap_12'}", $rv);
+return (2, "$text{'mod_imap_defact'}", $rv);
 }
 sub save_ImapDefault
 {
 if ($in{'ImapDefault'} eq "") { return ( [ ] ); }
 elsif ($in{'ImapDefault'} ne "url") { return ( [ $in{'ImapDefault'} ] ); }
 elsif ($in{'ImapDefault_url'} !~ /^\S+$/) {
-	&error(&text('mod_imap_19', $in{'ImapDefault_url'}));
+	&error(&text('mod_imap_eurl', $in{'ImapDefault_url'}));
 	}
 else { return ( [ $in{'ImapDefault_url'} ] ); }
 }
@@ -55,19 +55,19 @@ sub edit_ImapBase
 local($rv, $v);
 if ($_[0]->{'value'} =~ /^\S+:/) { $v = "url"; }
 else { $v = $_[0]->{'value'}; }
-$rv = &select_input($v, "ImapBase", "", "$text{'mod_imap_13'},",
-       "$text{'mod_imap_14'},root", "$text{'mod_imap_15'},map",
-       "$text{'mod_imap_16'},referer", "$text{'mod_imap_17'},url");
+$rv = &select_input($v, "ImapBase", "", "$text{'mod_imap_default2'},",
+       "$text{'mod_imap_root'},root", "$text{'mod_imap_imapurl'},map",
+       "$text{'mod_imap_refurl'},referer", "$text{'mod_imap_url'},url");
 $rv .= sprintf "&nbsp;<input name=ImapBase_url size=30 value=\"%s\">\n",
         $v eq "url" ? $_[0]->{'value'} : "";
-return (2, "$text{'mod_imap_18'}", $rv);
+return (2, "$text{'mod_imap_defbase'}", $rv);
 }
 sub save_ImapBase
 {
 if ($in{'ImapBase'} eq "") { return ( [ ] ); }
 elsif ($in{'ImapBase'} ne "url") { return ( [ $in{'ImapBase'} ] ); }
 elsif ($in{'ImapBase_url'} !~ /^\S+$/) {
-	&error(&text('mod_imap_20', $in{'ImapBase_url'}));
+	&error(&text('mod_imap_eurl', $in{'ImapBase_url'}));
 	}
 else { return ( [ $in{'ImapBase_url'} ] ); }
 }
