@@ -14,9 +14,9 @@ return ("server-parsed");
 
 sub edit_XBitHack
 {
-return (2, "$text{'mod_include_0'}",
+return (2, "$text{'mod_include_incl'}",
         &choice_input($_[0]->{'value'}, "XBitHack", "",
-        "$text{'mod_include_1'},off", "$text{'mod_include_2'},on", "$text{'mod_include_3'},full", "$text{'mod_include_4'},"));
+        "$text{'no'},off", "$text{'yes'},on", "$text{'mod_include_set'},full", "$text{'mod_include_default'},"));
 }
 sub save_XBitHack
 {
