@@ -6,6 +6,8 @@ sub mod_mime_directives
 $rv = [ [ 'AddType', 1, 6, 'virtual directory htaccess', undef, 10 ],
 	[ 'AddHandler', 1, 6, 'virtual directory htaccess', undef, 6 ],
 	[ 'RemoveHandler', 1, 6, 'directory htaccess', 1.304, 5 ],
+	[ 'RemoveType', 1, 6, 'directory htaccess', 1.313, 5 ],
+	[ 'RemoveEncoding', 1, 6, 'directory htaccess', 1.313, 5 ],
 	[ 'AddLanguage', 1, 6, 'virtual directory htaccess', undef, 4 ],
 	[ 'AddEncoding', 1, 6, 'virtual directory htaccess', undef, 8 ],
 	[ 'AddCharset', 1, 6, 'virtual directory htaccess', 1.310, 2 ],
@@ -20,8 +22,8 @@ sub extmap_input
 {
 local($rv, $i, $type, $exts);
 $rv = "<table border>\n".
-      "<tr $tb> <td><b>$_[1]</b></td> <td><b>$text{'mod_mime_0'}</b></td>\n".
-      "<td><b>$_[1]</b></td> <td><b>$text{'mod_mime_1'}</b></td> </tr>\n";
+      "<tr $tb> <td><b>$_[1]</b></td> <td><b>$text{'mod_mime_ext'}</b></td>\n".
+      "<td><b>$_[1]</b></td> <td><b>$text{'mod_mime_ext'}</b></td> </tr>\n";
 $len = int(@{$_[4]}/2)*2 + 2;
 for($i=0; $i<$len; $i++) {
 	if ($i%2 == 0) { $rv .= "<tr $cb>\n"; }
@@ -48,8 +50,8 @@ local($i, $type, $exts, @rv, $re); $re = $_[1];
 for($i=0; defined($in{"$_[0]_type_$i"}); $i++) {
 	$type = $in{"$_[0]_type_$i"}; $exts = $in{"$_[0]_exts_$i"};
 	if ($type !~ /\S/ && $exts !~ /\S/) { next; }
-	if ($type !~ /$re/) { &error(&text('mod_mime_18', $type, $_[2])); }
-	if ($exts !~ /\S/) { &error(&text('mod_mime_19', $_[2], $type)); }
+	if ($type !~ /$re/) { &error(&text('mod_mime_einvalid', $type, $_[2])); }
+	if ($exts !~ /\S/) { &error(&text('mod_mime_eext', $_[2], $type)); }
 	push(@rv, "$type $exts");
 	}
 return ( \@rv );
@@ -57,17 +59,17 @@ return ( \@rv );
 
 sub edit_AddType
 {
-&extmap_input("AddType", "$text{'mod_mime_20'}", 20, "$text{'mod_mime_2'}", $_[0]);
+&extmap_input("AddType", $text{'mod_mime_type'}, 20, $text{'mod_mime_xtype'}, $_[0]);
 }
 sub save_AddType
 {
 return &parse_extmap("AddType", '^(\S+)\/(\S+)$',
-		     "$text{'mod_mime_3'}");
+		     $text{'mod_mime_mtype'});
 }
 
 sub edit_AddHandler
 {
-&extmap_input("AddHandler", "$text{'mod_mime_21'}", "h", "$text{'mod_mime_4'}", $_[0]);
+&extmap_input("AddHandler", $text{'mod_mime_handler'}, "h", $text{'mod_mime_chandl'}, $_[0]);
 }
 sub save_AddHandler
 {
@@ -76,36 +78,36 @@ return &parse_extmap("AddHandler");
 
 sub edit_AddEncoding
 {
-&extmap_input("AddEncoding", "$text{'mod_mime_22'}", 20, "$text{'mod_mime_5'}", $_[0]);
+&extmap_input("AddEncoding", $text{'mod_mime_cenc'}, 20, $text{'mod_mime_cencs'}, $_[0]);
 }
 sub save_AddEncoding
 {
 return &parse_extmap("AddEncoding", '^(\S+)$',
-		     "$text{'mod_mime_6'}");
+		     $text{'mod_mime_cenc'});
 }
 
 sub edit_AddLanguage
 {
-&extmap_input("AddLanguage", "$text{'mod_mime_23'}", 5, "$text{'mod_mime_7'}", $_[0]);
+&extmap_input("AddLanguage", $text{'mod_mime_lang'}, 5, $text{'mod_mime_clangs'}, $_[0]);
 }
 sub save_AddLanguage
 {
-return &parse_extmap("AddLanguage", '^\S+$', "$text{'mod_mime_8'}");
+return &parse_extmap("AddLanguage", '^\S+$', $text{'mod_mime_clang'});
 }
 
 sub edit_ForceType
 {
-return (1, "$text{'mod_mime_9'}",
-	&opt_input($_[0]->{'value'}, "ForceType", "$text{'mod_mime_10'}", 15));
+return (1, $text{'mod_mime_defmime'},
+	&opt_input($_[0]->{'value'}, "ForceType", $text{'mod_mime_real'}, 15));
 }
 sub save_ForceType
 {
-return &parse_opt("ForceType", '^\S+\/\S+$', "$text{'mod_mime_11'}");
+return &parse_opt("ForceType", '^\S+\/\S+$', $text{'mod_mime_etype'});
 }
 
 sub edit_SetHandler
 {
-return (1, "$text{'mod_mime_12'}",
+return (1, $text{'mod_mime_pass'},
 	&handler_input($_[0]->{'value'}, "SetHandler"));
 }
 sub save_SetHandler
@@ -116,8 +118,8 @@ return &parse_handler("SetHandler");
 sub edit_TypesConfig
 {
 return (2,
-	"$text{'mod_mime_13'}",
-	&opt_input($_[0]->{'value'}, "TypesConfig", "$text{'mod_mime_14'}", 40).
+	$text{'mod_mime_file'},
+	&opt_input($_[0]->{'value'}, "TypesConfig", $text{'mod_mime_default'}, 40).
 	&file_chooser_button("TypesConfig", 0));
 }
 sub save_TypesConfig
@@ -127,21 +129,52 @@ return &parse_opt("TypesConfig");
 
 sub edit_RemoveHandler
 {
+local @list;
 foreach $v (@{$_[0]}) {
 	push(@list, @{$v->{'words'}});
 	}
-return (2, "$text{'mod_mime_15'}",
+return (2, "$text{'mod_mime_ignhand'}",
 	&opt_input(@list ? join(" ", @list) : undef,
-		   "RemoveHandler", "$text{'mod_mime_16'}", 40));
+		   "RemoveHandler", "$text{'mod_mime_none'}", 40));
 }
 sub save_RemoveHandler
 {
 return &parse_opt("RemoveHandler");
 }
 
+sub edit_RemoveType
+{
+local @list;
+foreach $v (@{$_[0]}) {
+	push(@list, @{$v->{'words'}});
+	}
+return (2, "$text{'mod_mime_igntype'}",
+	&opt_input(@list ? join(" ", @list) : undef,
+		   "RemoveType", "$text{'mod_mime_none'}", 40));
+}
+sub save_RemoveType
+{
+return &parse_opt("RemoveType");
+}
+
+sub edit_RemoveEncoding
+{
+local @list;
+foreach $v (@{$_[0]}) {
+	push(@list, @{$v->{'words'}});
+	}
+return (2, "$text{'mod_mime_ignenc'}",
+	&opt_input(@list ? join(" ", @list) : undef,
+		   "RemoveEncoding", "$text{'mod_mime_none'}", 40));
+}
+sub save_RemoveEncoding
+{
+return &parse_opt("RemoveEncoding");
+}
+
 sub edit_AddCharset
 {
-&extmap_input("AddCharset", "$text{'mod_mime_24'}", 5, "$text{'mod_mime_17'}", $_[0]);
+&extmap_input("AddCharset", "$text{'mod_mime_chars'}", 5, "$text{'mod_mime_xchars'}", $_[0]);
 }
 sub save_AddCharset
 {
