@@ -16,9 +16,9 @@ return ("type-map");
 sub edit_CacheNegotiatedDocs
 {
 local($v); $v = $_[0] ? "1" : "0";
-return (1, "$text{'mod_negotiation_0'}",
+return (1, "$text{'mod_negotiation_cache'}",
         &choice_input($v, "CacheNegotiatedDocs", "0",
-        "$text{'mod_negotiation_1'},1", "$text{'mod_negotiation_2'},0"));
+        "$text{'yes'},1", "$text{'no'},0"));
 }
 sub save_CacheNegotiatedDocs
 {
@@ -27,12 +27,12 @@ return $in{'CacheNegotiatedDocs'} ? ( [ "" ] ) : ( [ ] );
 
 sub edit_LanguagePriority
 {
-return (1, "$text{'mod_negotiation_3'}",
-        &opt_input($_[0]->{'value'}, "LanguagePriority", "$text{'mod_negotiation_4'}", 15));
+return (1, "$text{'mod_negotiation_pri'}",
+        &opt_input($_[0]->{'value'}, "LanguagePriority", "$text{'mod_negotiation_def'}", 15));
 }
 sub save_LanguagePriority
 {
-return &parse_opt("LanguagePriority", '\S', "$text{'mod_negotiation_5'}");
+return &parse_opt("LanguagePriority", '\S', "$text{'mod_negotiation_epri'}");
 }
 
 1;
