@@ -26,9 +26,9 @@ return &make_directives($rv, $_[0], "mod_proxy");
 
 sub edit_ProxyRequests
 {
-return (1, "$text{'mod_proxy_0'}", 
+return (1, $text{'mod_proxy_proxy'},
         &choice_input($_[0]->{'value'}, "ProxyRequests", "off",
-                      "$text{'mod_proxy_1'},on", "$text{'mod_proxy_2'},off"));
+                      "$text{'yes'},on", "$text{'no'},off"));
 }
 sub save_ProxyRequests
 {
@@ -39,7 +39,7 @@ sub edit_ProxyRemote
 {
 local($rv, $i, $match, $proxy, $max);
 $rv = "<table border>\n".
-      "<tr $tb> <td><b>$text{'mod_proxy_3'}</b></td> <td><b>$text{'mod_proxy_4'}</b></td> </tr>\n";
+      "<tr $tb> <td><b>$text{'mod_proxy_req'}</b></td> <td><b>$text{'mod_proxy_forw'}</b></td> </tr>\n";
 $max = @{$_[0]}+1;
 for($i=0; $i<$max; $i++) {
 	if ($_[0]->[$i]->{'value'} =~ /^(\S+)\s+(\S+)$/) {
@@ -48,10 +48,10 @@ for($i=0; $i<$max; $i++) {
 	else { $match = "*"; $proxy = ""; }
 	$rv .= "<tr $cb>\n";
 	$rv .= sprintf
-	        "<td><input type=radio name=ProxyRemote_match_all_$i %s> $text{'mod_proxy_5'}\n",
+	        "<td><input type=radio name=ProxyRemote_match_all_$i %s> $text{'mod_proxy_all'}\n",
 	        $match eq "*" ? "checked" : "";
 	$rv .= sprintf
-	        "<input type=radio name=ProxyRemote_match_all_$i %s> $text{'mod_proxy_6'}\n",
+	        "<input type=radio name=ProxyRemote_match_all_$i %s> $text{'mod_proxy_match'}\n",
 	        $match eq "*" ? "" : "checked";
 	$rv .= sprintf
 	        "<input name=ProxyRemote_match_$i size=30 value=\"%s\"></td>\n",
@@ -61,7 +61,7 @@ for($i=0; $i<$max; $i++) {
 	$rv .= "</tr>\n";
 	}
 $rv .= "</table>\n";
-return (2, "$text{'mod_proxy_7'}", $rv);
+return (2, $text{'mod_proxy_pass'}, $rv);
 }
 sub save_ProxyRemote
 {
@@ -71,8 +71,8 @@ for($i=0; defined($proxy = $in{"ProxyRemote_proxy_$i"}); $i++) {
 	$match_all = $in{"ProxyRemote_match_all_$i"};
 	if ($match !~ /\S/ && $proxy !~ /\S/) { next; }
 	if ($match_all) { $match = "*"; }
-	elsif ($match !~ /^\S+$/) { &error(&text('mod_proxy_54', $match)); }
-	$proxy =~ /^http:\/\/\S+$/ || &error(&text('mod_proxy_55', $proxy));
+	elsif ($match !~ /^\S+$/) { &error(&text('mod_proxy_erequest', $match)); }
+	$proxy =~ /^http:\/\/\S+$/ || &error(&text('mod_proxy_epurl', $proxy));
 	push(@rv, "$match $proxy");
 	}
 return ( \@rv );
@@ -82,7 +82,7 @@ sub edit_ProxyPass
 {
 local($rv, $i, $path, $url, $max);
 $rv = "<table border>\n".
-      "<tr $tb> <td><b>$text{'mod_proxy_8'}</b></td> <td><b>$text{'mod_proxy_9'}</b></td> </tr>\n";
+      "<tr $tb> <td><b>$text{'mod_proxy_local'}</b></td> <td><b>$text{'mod_proxy_remote'}</b></td> </tr>\n";
 $max = @{$_[0]} + 1;
 for($i=0; $i<$max; $i++) {
 	if ($_[0]->[$i]->{'value'} =~ /^(\S+)\s+(\S+)$/) {
@@ -95,7 +95,7 @@ for($i=0; $i<$max; $i++) {
 	$rv .= "</tr>\n";
 	}
 $rv .= "</table>\n";
-return (2, "$text{'mod_proxy_10'}", $rv);
+return (2, $text{'mod_proxy_map'}, $rv);
 }
 sub save_ProxyPass
 {
@@ -103,8 +103,8 @@ local($i, $url, $path, @rv);
 for($i=0; defined($path = $in{"ProxyPass_path_$i"}); $i++) {
 	$url = $in{"ProxyPass_url_$i"};
 	if ($path !~ /\S/ && $url !~ /\S/) { next; }
-	$path =~ /^\/\S*$/ || &error(&text('mod_proxy_56', $path));
-	$url =~ /^http:\/\/(\S+)$/ || &error(&text('mod_proxy_57', $url));
+	$path =~ /^\/\S*$/ || &error(&text('mod_proxy_elurl', $path));
+	$url =~ /^http:\/\/(\S+)$/ || &error(&text('mod_proxy_erurl', $url));
 	push(@rv, "$path $url");
 	}
 return ( \@rv );
@@ -114,7 +114,7 @@ sub edit_ProxyPassReverse
 {
 local($rv, $i, $path, $url, $max);
 $rv = "<table border>\n".
-      "<tr $tb> <td><b>$text{'mod_proxy_64'}</b></td> <td><b>$text{'mod_proxy_65'}</b></td> </tr>\n";
+      "<tr $tb> <td><b>$text{'mod_proxy_rurl'}</b></td> <td><b>$text{'mod_proxy_lurl'}</b></td> </tr>\n";
 $max = @{$_[0]} + 1;
 for($i=0; $i<$max; $i++) {
 	if ($_[0]->[$i]->{'value'} =~ /^(\S+)\s+(\S+)$/) {
@@ -127,7 +127,7 @@ for($i=0; $i<$max; $i++) {
 	$rv .= "</tr>\n";
 	}
 $rv .= "</table>\n";
-return (2, "$text{'mod_proxy_66'}", $rv);
+return (2, $text{'mod_proxy_headers'}, $rv);
 }
 sub save_ProxyPassReverse
 {
@@ -135,8 +135,8 @@ local($i, $url, $path, @rv);
 for($i=0; defined($path = $in{"ProxyPassReverse_path_$i"}); $i++) {
 	$url = $in{"ProxyPassReverse_url_$i"};
 	if ($path !~ /\S/ && $url !~ /\S/) { next; }
-	$path =~ /^\/\S*$/ || &error(&text('mod_proxy_56', $path));
-	$url =~ /^http:\/\/(\S+)$/ || &error(&text('mod_proxy_57', $url));
+	$path =~ /^\/\S*$/ || &error(&text('mod_proxy_elurl', $path));
+	$url =~ /^http:\/\/(\S+)$/ || &error(&text('mod_proxy_erurlerurl', $url));
 	push(@rv, "$path $url");
 	}
 return ( \@rv );
@@ -146,12 +146,12 @@ sub edit_ProxyBlock
 {
 local($b, @b);
 foreach $b (@{$_[0]}) { push(@b, split(/\s+/, $b->{'value'})); }
-return (2, "$text{'mod_proxy_11'}",
-        &opt_input(@b ? join(' ', @b) : undef, "ProxyBlock", "None", 50));
+return (2, $text{'mod_proxy_block'},
+        &opt_input(@b ? join(' ', @b) : undef, "ProxyBlock", $text{'mod_proxy_none3'}, 50));
 }
 sub save_ProxyBlock
 {
-return &parse_opt("ProxyBlock", '\S', "$text{'mod_proxy_12'}");
+return &parse_opt("ProxyBlock", '\S', $text{'mod_proxy_eblock'});
 }
 
 sub edit_NoProxy
@@ -159,7 +159,7 @@ sub edit_NoProxy
 local($n, @n, $i, $rv);
 foreach $n (@{$_[0]}) { push(@n, split(/\s+/, $n->{'value'})); }
 $rv = "<table border>\n".
-      "<tr $tb> <td><b>$text{'mod_proxy_13'}</b></td> <td><b>$text{'mod_proxy_14'}</b></td> </tr>\n";
+      "<tr $tb> <td><b>$text{'mod_proxy_type'}</b></td> <td><b>$text{'mod_proxy_noproxy'}</b></td> </tr>\n";
 for($i=0; $i<=@n; $i++) {
 	$rv .= "<tr $cb>\n";
 	if ($i>=@n) { $type = 0; }
@@ -171,13 +171,15 @@ for($i=0; $i<=@n; $i++) {
 	elsif ($n[$i] =~ /^([0-9\.]+)$/) { $type = 3; }
 	elsif ($n[$i] =~ /^\.(\S+)$/) { $type = 2; }
 	else { $type = 1; }
-	$rv .= "<td>".&select_input($type, "NoProxy_type_$i", 0, "IP address,0",
-	        "Hostname,1", "Domain,2", "IP network,3", "Network/bits,4")."</td>\n";
+	$rv .= "<td>".&select_input($type, "NoProxy_type_$i", 0,
+		"$text{'mod_proxy_ip'},0", "$text{'mod_proxy_host'},1",
+		"$text{'mod_proxy_domain'},2", "$text{'mod_proxy_net'},3",
+		"$text{'mod_proxy_netbit'},4")."</td>\n";
 	$rv .= "<td><input name=NoProxy_for_$i size=30 value=\"$n[$i]\"></td>\n";
 	$rv .= "</tr>\n";
 	}
 $rv .= "</table>\n";
-return (2, "$text{'mod_proxy_15'}", $rv);
+return (2, $text{'mod_proxy_nopass'}, $rv);
 }
 sub save_NoProxy
 {
@@ -187,25 +189,25 @@ for($i=0; defined($type = $in{"NoProxy_type_$i"}); $i++) {
 	if ($for !~ /\S/) { next; }
 	if ($type == 0) {
 		&check_ipaddress($for) || 
-			&error(&text('mod_proxy_58', $for));
+			&error(&text('mod_proxy_eip', $for));
 		}
 	elsif ($type == 1) {
 		$for =~ /^[A-z0-9\-][A-z0-9\-\.]+[A-z0-9\-]$/ ||
-			&error(&text('mod_proxy_59', $for));
+			&error(&text('mod_proxy_ehost', $for));
 		}
 	elsif ($type == 2) {
 		$for =~ /^\.[A-z0-9\-\.]+[A-z0-9\-]$/ ||
-			&error(&text('mod_proxy_60', $for));
+			&error(&text('mod_proxy_edomain', $for));
 		}
 	elsif ($type == 3) {
 		if ($for =~ /^(\d+)$/) { $for .= ".0.0.0"; }
 		elsif ($for =~ /^(\d+)\.(\d+)$/) { $for .= ".0.0"; }
 		elsif ($for =~ /^(\d+)\.(\d+)\.(\d+)$/) { $for .= ".0"; }
-		&check_ipaddress($for) || &error(&text('mod_proxy_61', $for));
+		&check_ipaddress($for) || &error(&text('mod_proxy_enet', $for));
 		}
 	elsif ($type == 4) {
 		($for =~ /^(\S+)\/(\d+)$/ && &check_ipaddress($1) && $2 < 32) ||
-			&error(&text('mod_proxy_62', $for));
+			&error(&text('mod_proxy_enetbit', $for));
 		}
 	push(@rv, $for);
 	}
@@ -214,134 +216,136 @@ return @rv ? ( [ join(' ', @rv) ] ) : ( [ ] );
 
 sub edit_ProxyDomain
 {
-return (1, "$text{'mod_proxy_16'}",
-        &opt_input($_[0]->{'value'}, "ProxyDomain", "$text{'mod_proxy_17'}", 20));
+return (1, $text{'mod_proxy_nodomain'},
+        &opt_input($_[0]->{'value'}, "ProxyDomain", $text{'mod_proxy_none'}, 20));
 }
 sub save_ProxyDomain
 {
-return &parse_opt("ProxyDomain", '^[A-z0-9\-]+$', "$text{'mod_proxy_18'}");
+return &parse_opt("ProxyDomain", '^[A-z0-9\-]+$', $text{'mod_proxy_enodomain'});
 }
 
 sub edit_AllowCONNECT
 {
-return (1, "$text{'mod_proxy_19'}",
-        &opt_input($_[0]->{'value'}, "AllowCONNECT", "$text{'mod_proxy_20'}", 10));
+return (1, $text{'mod_proxy_connect'},
+        &opt_input($_[0]->{'value'}, "AllowCONNECT", $text{'mod_proxy_default'}, 10));
 }
 sub save_AllowCONNECT
 {
-return &parse_opt("AllowCONNECT", '^[\d ]+$', "$text{'mod_proxy_21'}");
+return &parse_opt("AllowCONNECT", '^[\d ]+$', $text{'mod_proxy_econnect'});
 }
 
 sub edit_CacheRoot
 {
-return (1, "$text{'mod_proxy_22'}",
-        &opt_input($_[0]->{'value'}, "CacheRoot", "$text{'mod_proxy_23'}", 20).
+return (1, $text{'mod_proxy_dir'},
+        &opt_input($_[0]->{'value'}, "CacheRoot", $text{'mod_proxy_none'}, 20).
         &file_chooser_button("CacheRoot", 0));
 }
 sub save_CacheRoot
 {
 $in{'CacheRoot_def'} || &allowed_auth_file($in{'CacheRoot'}) ||
-	&error($text{'mod_proxy_63'});
-return &parse_opt("CacheRoot", '^\S+$', "$text{'mod_proxy_24'}");
+	&error($text{'mod_proxy_eunder'});
+return &parse_opt("CacheRoot", '^\S+$', $text{'mod_proxy_edir'});
 }
 
 sub edit_CacheSize
 {
-return (1, "$text{'mod_proxy_25'}",
-        &opt_input($_[0]->{'value'}, "CacheSize", "$text{'mod_proxy_26'}", 8)." Kb");
+return (1, $text{'mod_proxy_size'},
+        &opt_input($_[0]->{'value'}, "CacheSize", $text{'mod_proxy_default'}, 8)." kB");
 }
 sub save_CacheSize
 {
-return &parse_opt("CacheSize", '^\d+$', "$text{'mod_proxy_27'}");
+return &parse_opt("CacheSize", '^\d+$', $text{'mod_proxy_esize'});
 }
 
 sub edit_CacheGcInterval
 {
-return (1, "$text{'mod_proxy_28'}",
-        &opt_input($_[0]->{'value'}, "CacheGcInterval", "$text{'mod_proxy_29'}", 6)." $text{'mod_proxy_30'}");
+return (1, $text{'mod_proxy_garbage'},
+        &opt_input($_[0]->{'value'}, "CacheGcInterval", $text{'mod_proxy_nogc'}, 6).
+	$text{'mod_proxy_hours'});
 }
 sub save_CacheGcInterval
 {
 return &parse_opt("CacheGcInterval", '^\d+$',
-                  "$text{'mod_proxy_31'}");
+                  $text{'mod_proxy_egarbage'});
 }
 
 sub edit_CacheMaxExpire
 {
-return (1, "$text{'mod_proxy_32'}",
-        &opt_input($_[0]->{'value'}, "CacheMaxExpire", "$text{'mod_proxy_33'}", 6)." $text{'mod_proxy_34'}");
+return (1, $text{'mod_proxy_maxexp'},
+        &opt_input($_[0]->{'value'}, "CacheMaxExpire", $text{'mod_proxy_default'}, 6).
+	$text{'mod_proxy_hours'});
 }
 sub save_CacheMaxExpire
 {
 return &parse_opt("CacheMaxExpire", '^\d+$',
-                  "$text{'mod_proxy_35'}");
+                  $text{'mod_proxy_emaxexp'});
 }
 
 sub edit_CacheLastModifiedFactor
 {
-return (1, "$text{'mod_proxy_36'}",
-        &opt_input($_[0]->{'value'}, "CacheLastModifiedFactor", "$text{'mod_proxy_37'}", 6));
+return (1, $text{'mod_proxy_expfac'},
+        &opt_input($_[0]->{'value'}, "CacheLastModifiedFactor", $text{'mod_proxy_default'}, 6));
 }
 sub save_CacheLastModifiedFactor
 {
 return &parse_opt("CacheLastModifiedFactor", '^\d+$',
-                  "$text{'mod_proxy_38'}");
+                  $text{'mod_proxy_eexpfac'});
 }
 
 sub edit_CacheDirLevels
 {
-return (1, "$text{'mod_proxy_39'}",
-        &opt_input($_[0]->{'value'}, "CacheDirLevels", "$text{'mod_proxy_40'}", 6));
+return (1, $text{'mod_proxy_levels'},
+        &opt_input($_[0]->{'value'}, "CacheDirLevels", $text{'mod_proxy_default'}, 6));
 }
 sub save_CacheDirLevels
 {
 return &parse_opt("CacheDirLevels", '^\d+$',
-                  "$text{'mod_proxy_41'}");
+                  $text{'mod_proxy_elevels'});
 }
 
 sub edit_CacheDirLength
 {
-return (1, "$text{'mod_proxy_42'}",
-        &opt_input($_[0]->{'value'}, "CacheDirLength", "$text{'mod_proxy_43'}", 4));
+return (1, $text{'mod_proxy_length'},
+        &opt_input($_[0]->{'value'}, "CacheDirLength", $text{'mod_proxy_default'}, 4));
 }
 sub save_CacheDirLength
 {
 return &parse_opt("CacheDirLength", '^\d+$',
-                  "$text{'mod_proxy_44'}");
+                  $text{'mod_proxy_elength'});
 }
 
 sub edit_CacheDefaultExpire
 {
-return (1, "$text{'mod_proxy_45'}",
-        &opt_input($_[0]->{'value'}, "CacheDefaultExpire", "$text{'mod_proxy_46'}", 6).
-	" hours");
+return (1, $text{'mod_proxy_defexp'},
+        &opt_input($_[0]->{'value'}, "CacheDefaultExpire", $text{'mod_proxy_default'}, 6).
+	$text{'mod_proxy_hours'});
 }
 sub save_CacheDefaultExpire
 {
-return &parse_opt("CacheDefaultExpire", '^\d+$', "$text{'mod_proxy_47'}");
+return &parse_opt("CacheDefaultExpire", '^\d+$', $text{'mod_proxy_edefexp'});
 }
 
 sub edit_CacheForceCompletion
 {
-return (1, "$text{'mod_proxy_48'}",
-        &opt_input($_[0]->{'value'}, "CacheForceCompletion", "$text{'mod_proxy_49'}", 6)."%");
+return (1, $text{'mod_proxy_finish'},
+        &opt_input($_[0]->{'value'}, "CacheForceCompletion", $text{'mod_proxy_default'}, 6)."%");
 }
 sub save_CacheForceCompletion
 {
 return &parse_opt("CacheForceCompletion", '^\d+$',
-		  "$text{'mod_proxy_50'}");
+		  $text{'mod_proxy_efinish'});
 }
 
 sub edit_NoCache
 {
 local($n, @n);
 foreach $n (@{$_[0]}) { push(@n, $n->{'value'}); }
-return (1, "$text{'mod_proxy_51'}",
-        &opt_input(@n ? join(' ', @n) : undef, "NoCache", "$text{'mod_proxy_52'}", 20));
+return (1, $text{'mod_proxy_nocache'},
+        &opt_input(@n ? join(' ', @n) : undef, "NoCache", $text{'mod_proxy_none2'}, 20));
 }
 sub save_NoCache
 {
-return &parse_opt("NoCache", '\S', "$text{'mod_proxy_53'}");
+return &parse_opt("NoCache", '\S', $text{'mod_proxy_enocache'});
 }
 
 1;
