@@ -13,8 +13,10 @@ sub useradmin_create_user
 {
 foreach $s (@sync) {
 	if ($s->[1] =~ /create/ && !&get_authuser($s->[0], $_[0]->{'user'})) {
+		&lock_file($s->[0]);
 		&create_authuser($s->[0], { 'user' => $_[0]->{'user'},
 					    'pass' => $_[0]->{'pass'} });
+		&unlock_file($s->[0]);
 		}
 	}
 }
@@ -24,7 +26,9 @@ sub useradmin_delete_user
 {
 foreach $s (@sync) {
 	if ($s->[1] =~ /delete/) {
+		&lock_file($s->[0]);
 		&delete_authuser($s->[0], $_[0]->{'user'});
+		&unlock_file($s->[0]);
 		}
 	}
 }
@@ -34,9 +38,11 @@ sub useradmin_modify_user
 {
 foreach $s (@sync) {
 	if ($s->[1] =~ /modify/) {
+		&lock_file($s->[0]);
 		&save_authuser($s->[0], $_[0]->{'olduser'},
 			       { 'user' => $_[0]->{'user'},
 				 'pass' => $_[0]->{'pass'} });
+		&unlock_file($s->[0]);
 		}
 	}
 }
