@@ -5,6 +5,11 @@ do '../web-lib.pl';
 &init_config();
 do './records-lib.pl';
 
+if (open(VERSION, "$module_config_directory/version")) {
+	chop($bind_version = <VERSION>);
+	close(VERSION);
+	}
+
 # get_config()
 # Returns an array of references to assocs, each containing the details of
 # one directive
@@ -20,11 +25,12 @@ return \@get_config_cache;
 # Returns a structure containing the top-level config as members
 sub get_config_parent
 {
+local $conf = &get_config();
 return { 'file' => $config{'named_conf'},
 	 'type' => 1,
-	 'line' => 0,
+	 'line' => -1,
 	 'eline' => $lines_count{$config{'named_conf'}},
-	 'members' => &get_config() };
+	 'members' => $conf };
 }
 
 # read_config_file(file, [expand includes])
@@ -39,7 +45,7 @@ while($line = <FILE>) {
 	# strip comments
 	$line =~ s/\r|\n//g;
 	$line =~ s/#.*$//g;
-	$line =~ s/\/\/.*$//g;
+	$line =~ s/\/\/.*$//g if ($line !~ /".*\/\/.*"/);
 	$line =~ s/\/\*.*\*\///g;
 	while(1) {
 		if (!$cmode && $line =~ /\/\*/) {
@@ -196,7 +202,7 @@ $config{'named_conf'} =~ /^(.*)\/[^\/]+$/;
 return $1;
 }
 
-# save_directive(&parent, name, &values, indent, [structonly])
+# save_directive(&parent, name|&old, &values, indent, [structonly])
 # Given a structure containing a directive name, type, values and members
 # add, update or remove that directive in config structure and data files.
 # Updating of files assumes that there is no overlap between directives -
@@ -205,10 +211,10 @@ sub save_directive
 {
 local(@oldv, @newv, $pm, $i, $o, $n, $lref, @nl);
 $pm = $_[0]->{'members'};
-@oldv = &find($_[1], $pm);
+@oldv = ref($_[1]) ? ( $_[1] ) : &find($_[1], $pm);
 @newv = @{$_[2]};
 for($i=0; $i<@oldv || $i<@newv; $i++) {
-	if ($i >= @oldv) {
+	if ($i >= @oldv && !$_[5]) {
 		# a new directive is being added.. put it at the end of
 		# the parent
 		if (!$_[4]) {
@@ -224,6 +230,22 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 			}
 		push(@$pm, $newv[$i]);
 		}
+	elsif ($i >= @oldv && $_[5]) {
+		# a new directive is being added.. put it at the start of
+		# the parent
+		if (!$_[4]) {
+			$lref = &read_file_lines($config{'chroot'}.$_[0]->{'file'});
+			@nl = &directive_lines($newv[$i], $_[3]);
+			splice(@$lref, $_[0]->{'line'}+1, 0, @nl);
+			$newv[$i]->{'file'} = $_[0]->{'file'};
+			$newv[$i]->{'line'} = $_[0]->{'line'}+1;
+			$newv[$i]->{'eline'} =
+				$_[0]->{'line'} + scalar(@nl);
+			&renumber(&get_config(), $_[0]->{'line'},
+				  $_[0]->{'file'}, scalar(@nl));
+			}
+		splice(@$pm, 0, 0, $newv[$i]);
+		}
 	elsif ($i >= @newv) {
 		# a directive was deleted
 		if (!$_[4]) {
@@ -299,7 +321,7 @@ foreach $d (@{$_[0]}) {
 sub choice_input
 {
 local($rv, $v, $i, @ops);
-$rv = "<td><b>$_[0]</b></td> <td valign=top>";
+$rv = "<td valign=top><b>$_[0]</b></td> <td valign=top>";
 $v = &find_value($_[1], $_[2]);
 for($i=3; $i<@_; $i+=2) {
 	$rv .= sprintf "<input type=radio name=%s value='%s' %s> %s\n",
@@ -332,6 +354,7 @@ $rv .= "<textarea name=$_[1] rows=3 cols=15>".
 	join("\n", @av)."</textarea></td>\n";
 }
 
+# save_addr_match(name, &parent, indent)
 sub save_addr_match
 {
 local($addr, @vals, $dir);
@@ -345,6 +368,33 @@ else {
 	}
 }
 
+# address_port_input(addresstext, portlabeltext, portnametext, defaulttext,
+#                    addressname, portname, &config, size, type)
+sub address_port_input
+  {
+    my $rv = &address_input($_[0], $_[4], $_[6], $_[8]);
+    my $v = &find($_[4], $_[6]);
+
+    my $port;
+    for ($i = 0; $i < @{$v->{'values'}}; $i++) {
+      if ($v->{'values'}->[$i] eq $_[5]) {
+	$port = $v->{'values'}->[$i+1];
+	last;
+      }
+    }
+    my $n;
+    ($n = $_[5]) =~ s/[^A-Za-z0-9_]/_/g;
+    $rv .= "<td valign=top><b>$_[1]</b></td> <td nowrap valign=top>\n";
+    $rv .= sprintf "<input type=radio name=${n}_def value=1 %s> $_[3]\n",
+      defined($port) ? "" : "checked";
+    $rv .= sprintf "<input type=radio name=${n}_def value = 0 %s> ",
+      defined($port) ? "checked" : "";
+    $rv .= sprintf "<input name=$n size=$_[7] value=\"%s\"> $_[2]</td>\n",
+      defined($port) ? $port : "";
+
+    return $rv;
+  }
+
 # address_input(text, name, &config, type)
 sub address_input
 {
@@ -364,11 +414,26 @@ else {
 return $rv;
 }
 
+sub save_port_address {
+  local($addr, $port, @vals, $dir);
+  foreach $addr (split(/\s+/, $in{$_[0]})) {
+    &check_ipaddress($addr) || &error(&text('eip', $addr));
+    push(@vals, { 'name' => $addr });
+  }
+  $dir = { 'name' => $_[0], 'type' => 1, 'members' => \@vals };
+  ($n = $_[1]) =~ s/[^A-Za-z0-9_]/_/g;
+  $dir->{'values'} = [ $_[1], $in{$_[1]} ] if (!$in{"${n}_def"});
+  &save_directive($_[2], $_[0], @vals ? [ $dir ] : [ ], $_[3]);
+}
+
+# save_address(name, &parent, indent, ips-only)
 sub save_address
 {
 local($addr, @vals, $dir);
 foreach $addr (split(/\s+/, $in{$_[0]})) {
-	&check_ipaddress($addr) || &error(&text('eip', $addr));
+	if ($_[3]) {
+		&check_ipaddress($addr) || &error(&text('eip', $addr));
+		}
 	push(@vals, { 'name' => $addr });
 	}
 $dir = { 'name' => $_[0], 'type' => 1, 'members' => \@vals };
@@ -415,35 +480,72 @@ foreach $need (@need_quote) {
 
 1;
 
-# find_reverse(address)
+# find_reverse(address, [view])
 # Returns the zone and record structures for the PTR record for some address
 sub find_reverse
 {
-local($conf, @zl, $rev, $z, $revconf, $revfile, $revrec, @revrecs, $addr, $rr);
+local($conf, @zl, $rev, $z, $revconf, $revfile, $revrec, @revrecs, $addr, $rr,
+      @octs, $i, @hexs, $ipv6, @zero);
 
 # find reverse domain
 $conf = &get_config();
+if ($in{'view'} ne '') {
+	$conf = $conf->[$in{'view'}]->{'members'};
+	}
 @zl = &find("zone", $conf);
-@octs = split(/\./, $_[0]);
-DOMAIN: for($i=2; $i>=0; $i--) {
-	$rev = &ip_to_arpa(join('.', @octs[0..$i]));
-	$rev =~ s/\.$//g;
-	foreach $z (@zl) {
-		if (lc($z->{'value'}) eq $rev &&
-		    ($typed = &find("type", $z->{'members'})) &&
-		    $typed->{'value'} eq "master") {
-			# found the reverse master domain
-			$revconf = $z;
-			last DOMAIN;
+$ipv6 = $config{'support_aaaa'} && &check_ip6address($_[0]);
+if ($ipv6) {
+	@zero = (0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0);
+	$addr = &expandall_ip6($_[0]);
+	$addr =~ s/://g;
+	@hexs = split('', $addr);
+	DOMAIN: for($i=30; $i>=0; $i--) {
+		$addr = join(':',split(/(.{4})/,join('', (@hexs[0..$i],@zero[$i..30]))));
+		$addr =~ s/::/:/g;
+		$addr =~ s/(^:|:$)//g;
+		$rev = &net_to_ip6int($addr, 4*($i+1));
+		$rev =~ s/\.$//g;
+		foreach $z (@zl) {
+			if (lc($z->{'value'}) eq $rev &&
+			    ($typed = &find("type", $z->{'members'})) &&
+			    $typed->{'value'} eq "master") {
+				# found the reverse master domain
+				$revconf = $z;
+				last DOMAIN;
+				}
+			}
+		}
+	}
+else {
+	@octs = split(/\./, $_[0]);
+	DOMAIN: for($i=2; $i>=-1; $i--) {
+		$rev = $i<0 ? "in-addr.arpa"
+			    : &ip_to_arpa(join('.', @octs[0..$i]));
+		$rev =~ s/\.$//g;
+		foreach $z (@zl) {
+			if ((lc($z->{'value'}) eq $rev ||
+			     lc($z->{'value'}) eq "$rev.") &&
+			    ($typed = &find("type", $z->{'members'})) &&
+			    $typed->{'value'} eq "master") {
+				# found the reverse master domain
+				$revconf = $z;
+				last DOMAIN;
+				}
 			}
 		}
 	}
 
 # find reverse record
 if ($revconf) {
-	$revfile = &find("file", $revconf->{'members'})->{'value'};
+	$revfile = &absolute_path(
+		&find("file", $revconf->{'members'})->{'value'});
 	@revrecs = &read_zone_file($revfile, $revconf->{'value'});
-	$addr = &ip_to_arpa($_[0]);
+	if ($ipv6) {
+		$addr = &net_to_ip6int($_[0], 128);
+		}
+	else {
+		$addr = &ip_to_arpa($_[0]);
+		}
 	foreach $rr (@revrecs) {
 		if ($rr->{'type'} eq "PTR" &&
 		    lc($rr->{'name'}) eq lc($addr)) {
@@ -456,22 +558,26 @@ if ($revconf) {
 return ($revconf, $revfile, $revrec);
 }
 
-# find_forward(address)
+# find_forward(address, [view])
 # Returns the zone and record structures for the A record for some address
 sub find_forward
 {
-local ($fwdconf, $i, $fwdfile, $fwdrec, $fr);
+local ($fwdconf, $i, $fwdfile, $fwdrec, $fr, $ipv6);
 
 # find forward domain
 local $host = $_[0]; $host =~ s/\.$//;
 local $conf = &get_config();
+if ($in{'view'} ne '') {
+	$conf = $conf->[$in{'view'}]->{'members'};
+	}
 local @zl = &find("zone", $conf);
 local @parts = split(/\./, $host);
 DOMAIN: for($i=1; $i<@parts; $i++) {
 	local $fwd = join(".", @parts[$i .. @parts-1]);
 	foreach $z (@zl) {
 		local $typed;
-		if (lc($z->{'value'}) eq $fwd &&
+		if ((lc($z->{'value'}) eq $fwd ||
+		     lc($z->{'value'}) eq "$fwd.") &&
 		    ($typed = &find("type", $z->{'members'})) &&
 		    $typed->{'value'} eq "master") {
 			# Found the forward master!
@@ -483,10 +589,11 @@ DOMAIN: for($i=1; $i<@parts; $i++) {
 
 # find forward record
 if ($fwdconf) {
-	$fwdfile = &find("file", $fwdconf->{'members'})->{'value'};
+	$fwdfile = &absolute_path(
+		&find("file", $fwdconf->{'members'})->{'value'});
 	local @fwdrecs = &read_zone_file($fwdfile, $fwdconf->{'value'});
 	foreach $fr (@fwdrecs) {
-		if ($fr->{'type'} eq 'A' &&
+		if ($ipv6 ? $fr->{'type'} eq "AAAA" : $fr->{'type'} eq "A" &&
 		    $fr->{'name'} eq $_[0]) {
 			# found the forward record!
 			$fwdrec = $fr;
@@ -498,92 +605,125 @@ if ($fwdconf) {
 return ($fwdconf, $fwdfile, $fwdrec);
 }
 
-# can_edit_zone(&access, zone)
+# can_edit_zone(&access, &zone)
 sub can_edit_zone
 {
 local %zcan;
-return 1 if ($access{'zones'} eq '*');
-foreach (split(/\s+/, $access{'zones'})) {
-	return 1 if ($_ eq $_[1]);
+local $zn = $_[1]->{'value'};
+
+# Check zone name
+if ($access{'zones'} eq '*') {
 	}
-return 0;
+elsif ($access{'zones'} =~ /^\!/) {
+	foreach (split(/\s+/, $access{'zones'})) {
+		return 0 if ($_ eq $zn);
+		}
+	}
+else {
+	local $ok;
+	foreach (split(/\s+/, $access{'zones'})) {
+		$ok++ if ($_ eq $zn);
+		}
+	return 0 if (!$ok);
+	}
+
+if ($_[0]->{'dironly'}) {
+	# Check directory access control 
+	local $file = &absolute_path(
+			&find("file", $_[1]->{'members'})->{'value'});
+	return 0 if (!&allowed_zone_file($_[0], $file));
+	}
+return 1;
 }
 
-# record_input(zoneindex, type, file, origin, [num], [record])
+# record_input(zoneindex, view, type, file, origin, [num], [record])
 # Display a form for editing or creating a DNS record
 sub record_input
 {
-local(%rec, @recs);
+local(%rec, @recs, $ttl, $ttlunit);
 print "<form action=save_record.cgi>\n";
-print "<input type=hidden name=index value=$_[0]>\n";
-print "<input type=hidden name=file value=$_[2]>\n";
-print "<input type=hidden name=origin value=$_[3]>\n";
-if (@_ >= 5) {
-	print "<input type=hidden name=num value=$_[4]>\n";
-	%rec = %{$_[5]};
+print "<input type=hidden name=index value='$_[0]'>\n";
+print "<input type=hidden name=view value='$_[1]'>\n";
+print "<input type=hidden name=file value='$_[3]'>\n";
+print "<input type=hidden name=origin value='$_[4]'>\n";
+print "<input type=hidden name=sort value='$in{'sort'}'>\n";
+if (@_ >= 6) {
+	print "<input type=hidden name=num value=$_[5]>\n";
+	%rec = %{$_[6]};
 	}
 else { print "<input type=hidden name=new value=1>\n"; }
-print "<input type=hidden name=type value=$_[1]>\n";
+print "<input type=hidden name=type value=$_[2]>\n";
 print "<table border>\n";
-print "<tr $tb><td><b>",&text(@_ >= 5 ? 'edit_edit' : 'edit_add',
-			      $text{"type_$_[1]"}),"</b></td> </tr>\n";
+print "<tr $tb><td><b>",&text(@_ >= 6 ? 'edit_edit' : 'edit_add',
+			      $text{"edit_$_[2]"}),"</b></td> </tr>\n";
 print "<tr $cb><td><table>\n";
 
-if ($_[1] eq "PTR") {
+if ($_[2] eq "PTR") {
 	print "<tr> <td><b>$text{'edit_addr'}</b></td>\n";
 	printf "<td><input name=name value=\"%s\" size=30></td>\n",
-		!%rec && $_[3] =~ /^(\d+)\.(\d+)\.(\d+)\.in-addr/ ?
-			"$3.$2.$1." : &arpa_to_ip($rec{'name'});
+		!%rec && $_[4] =~ /^(\d+)\.(\d+)\.(\d+)\.in-addr/ ?
+			"$3.$2.$1." : &ip6int_to_net(&arpa_to_ip($rec{'name'}));
+	}
+elsif ($_[2] eq "NS") {
+	print "<tr> <td><b>$text{'edit_zonename'}</b></td>\n";
+	print "<td><input name=name value=\"$rec{'name'}\" size=30></td>\n";
 	}
 else {
 	print "<tr> <td><b>$text{'edit_name'}</b></td>\n";
 	print "<td><input name=name value=\"$rec{'name'}\" size=30></td>\n";
 	}
-
+if ($rec{'ttl'} =~ /^(\d+)([SMHDW]?)$/i) {
+	$ttl = $1; $ttlunit = $2;
+	}
+else {
+	$ttl = $rec{'ttl'}; $ttlunit = "";
+	}
 print "<td><b>$text{'edit_ttl'}</b></td>\n";
 printf "<td><input type=radio name=ttl_def value=1 %s> $text{'default'}\n",
 	defined($rec{'ttl'}) ? "" : "checked";
 printf "<input type=radio name=ttl_def value=0 %s>\n",
 	defined($rec{'ttl'}) ? "checked" : "";
-print "<input name=ttl size=8 value=\"$rec{'ttl'}\"></td> </tr>\n";
+print "<input name=ttl size=8 value=\"$ttl\">\n";
+&time_unit_choice("ttlunit", $ttlunit);
+print "</td> </tr>\n";
 
 @v = @{$rec{'values'}};
-if ($_[1] eq "A") {
+if ($_[2] eq "A" || $_[2] eq "AAAA") {
 	print "<tr> <td><b>$text{'value_A1'}</b></td>\n";
 	print "<td><input name=value0 size=20 value=\"$v[0]\"></td> </tr>\n";
-	if (@_ >= 5) {
+	if (@_ >= 6) {
 		print "<input type=hidden name=oldname ",
 		      "value=\"$rec{'name'}\">\n";
 		print "<input type=hidden name=oldvalue0 value=\"$v[0]\">\n";
 		}
 	}
-elsif ($_[1] eq "NS") {
+elsif ($_[2] eq "NS") {
 	print "<tr> <td><b>$text{'value_NS1'}</b></td>\n";
 	print "<td colspan=3><input name=value0 size=30 value=\"$v[0]\">\n";
 	print "($text{'edit_cnamemsg'})</td> </tr>\n";
 	}
-elsif ($_[1] eq "CNAME") {
+elsif ($_[2] eq "CNAME") {
 	print "<tr> <td><b>$text{'value_CNAME1'}</b></td>\n";
 	print "<td colspan=3><input name=value0 size=30 value=\"$v[0]\">\n";
 	print "($text{'edit_cnamemsg'})</td> </tr>\n";
 	}
-elsif ($_[1] eq "MX") {
+elsif ($_[2] eq "MX") {
 	print "<tr> <td><b>$text{'value_MX2'}</b></td>\n";
 	print "<td><input name=value1 size=30 value=\"$v[1]\"></td>\n";
 	print "<td><b>$text{'value_MX1'}</b></td>\n";
 	print "<td><input name=value0 size=8 value=\"$v[0]\"></td> </tr>\n";
 	}
-elsif ($_[1] eq "HINFO") {
+elsif ($_[2] eq "HINFO") {
 	print "<tr> <td><b>$text{'value_HINFO1'}</b></td>\n";
 	print "<td><input name=value0 size=20 value=\"$v[0]\"></td>\n";
 	print "<td><b>$text{'value_HINFO2'}</b></td>\n";
 	print "<td><input name=value1 size=20 value=\"$v[1]\"></td> </tr>\n";
 	}
-elsif ($_[1] eq "TXT") {
+elsif ($_[2] eq "TXT") {
 	print "<tr> <td><b>$text{'value_TXT1'}</b></td>\n";
 	print "<td><input name=value0 size=30 value=\"$v[0]\"></td> </tr>\n";
 	}
-elsif ($_[1] eq "WKS") {
+elsif ($_[2] eq "WKS") {
 	print "<tr> <td><b>$text{'value_WKS1'}</b></td>\n";
 	print "<td><input name=value0 size=15 value=\"$v[0]\"></td>\n";
 	print "<td><b>$text{'value_WKS2'}</b></td>\n";
@@ -595,30 +735,45 @@ elsif ($_[1] eq "WKS") {
 	print "<td><textarea name=value2 rows=3 cols=20 wrap>",
 		join(' ', @v[2..$#v]),"</textarea></td> </tr>\n";
 	}
-elsif ($_[1] eq "RP") {
+elsif ($_[2] eq "RP") {
 	print "<tr> <td><b>$text{'value_RP1'}</b></td>\n";
-	$v[0] =~ s/\./\@/;
+	$v[0] = &dotted_to_email($v[0]);
 	print "<td><input name=value0 size=20 value=\"$v[0]\"></td>\n";
 	print "<td><b>$text{'value_RP2'}</b></td>\n";
 	print "<td><input name=value1 size=30 value=\"$v[1]\"></td> </tr>\n";
 	}
-elsif ($_[1] eq "PTR") {
+elsif ($_[2] eq "PTR") {
 	print "<tr> <td><b>$text{'value_PTR1'}</b></td>\n";
 	print "<td><input name=value0 size=30 value=\"$v[0]\"></td> </tr>\n";
-	if (@_ >= 5) {
+	if (@_ >= 6) {
 		print "<input type=hidden name=oldname ",
 		      "value=\"$rec{'name'}\">\n";
 		print "<input type=hidden name=oldvalue0 value=\"$v[0]\">\n";
 		}
 	}
-if ($_[1] eq "A") {
+elsif ($_[2] eq "LOC") {
+	print "<tr> <td><b>$text{'value_LOC1'}</b></td>\n";
+	printf "<td colspan=3><input name=value0 size=45 value=\"%s\"></td> </tr>\n", join(" ", @v);
+	}
+if ($_[2] ne "WKS") {
+	printf "<tr> <td><b>%s</b></td>\n",
+		$config{'allow_comments'} ? $text{'edit_comment'} : "";
+	printf "<td><input %s name=comment size=30 value=\"%s\"></td> </tr>\n",
+		$config{'allow_comments'} ? "" : "type=hidden", $rec{'comment'};
+	}
+if ($_[2] eq "A" || $_[2] eq "AAAA") {
 	print "<tr> <td><b>$text{'edit_uprev'}</b></td>\n";
 	printf "<td><input type=radio name=rev value=1 %s> $text{'yes'}\n",
-		$config{'rev_def'} ? '' : 'checked';
+		$config{'rev_def'} == 0 ? 'checked' : "";
+	if (@_ < 6) {
+		printf "<input type=radio name=rev value=2 %s> %s\n",
+			$config{'rev_def'} == 2 ? 'checked' : '',
+			$text{'edit_over'};
+		}
 	printf "<input type=radio name=rev value=0 %s> $text{'no'}</td>\n",
-		$config{'rev_def'} ? 'checked' : '';
+		$config{'rev_def'} == 1 ? 'checked' : '';
 	}
-elsif ($_[1] eq "PTR") {
+elsif ($_[2] eq "PTR") {
 	print "<tr> <td><b>$text{'edit_upfwd'}</b></td>\n";
 	printf "<td><input type=radio name=fwd value=1 %s> $text{'yes'}\n",
 		$config{'rev_def'} ? '' : 'checked';
@@ -629,7 +784,7 @@ else { print "<tr> <td colspan=2></td>\n"; }
 print "<td colspan=2 align=right>\n";
 local %a = &get_module_acl();
 if (!$a{'ro'}) {
-	if (@_ >= 5) {
+	if (@_ >= 6) {
 		print "<input type=submit value=\"$text{'save'}\">\n";
 		print "<input type=submit name=delete ",
 		      "value=\"$text{'delete'}\">\n";
@@ -654,6 +809,136 @@ for($i=0; $i<@{$_[0]}; $i++) {
 print "</table>\n";
 }
 
+# convert_illegal(text)
+# Convert text containing special HTML characters to properly display it.
+sub convert_illegal
+{
+$_[0] =~ s/&/&amp;/g;
+$_[0] =~ s/>/&gt;/g;
+$_[0] =~ s/</&lt;/g;
+$_[0] =~ s/"/&quot;/g;
+$_[0] =~ s/ /&nbsp;/g;
+return $_[0];
+}
+
+sub check_net_ip
+{
+local($j, $arg = $_[0]);
+if ($arg !~ /^(\d{1,3}\.){0,3}([0-9\-\/]+)$/) {
+	return 0;
+	}
+foreach $j (split(/\./, $arg)) {
+	$j =~ /^(\d+)-(\d+)$/ && $1 < 255 && $2 < 255 ||
+	$j =~ /^(\d+)\/(\d+)$/ && $1 < 255 && $2 <= 32 ||
+		$j <= 255 || return 0;
+	}
+return 1;
+}
+
+# expand_ip6(ip)
+# Transform compact (with ::) IPv6 address to the unique expanded form
+# (without :: and leading zeroes in all parts) 
+sub expand_ip6
+{
+local($n);
+for($n = 6 - ($_[0] =~ s/([^:]):(?=[^:])/$1:/g); $n > 0; $n--) {
+	$_[0] =~ s/::/:0::/;
+	}
+$_[0] =~ s/::/:/;
+$_[0] =~ s/^:/0:/;
+$_[0] =~ s/:$/:0/;
+$_[0] =~ s/(:|^)0(?=\w)/$1/;
+$_[0] =~ tr/[A-Z]/[a-z]/;
+return $_[0];
+}
+
+# expandall_ip6(ip)
+# Transform IPv6 address to the expanded form containing all internal 0's 
+sub expandall_ip6
+{
+&expand_ip6($_[0]);
+$_[0] =~ s/(:|^)(\w{3})(?=:|$)/:0$2/g;
+$_[0] =~ s/(:|^)(\w{2})(?=:|$)/:00$2/g;
+$_[0] =~ s/(:|^)(\w)(?=:|$)/:000$2/g;
+return $_[0];
+}
+
+# check_ip6address(ip)
+# Check if some IPv6 address is properly formatted
+sub check_ip6address
+{
+local($ip6);
+$ip6 = $_[0];
+$ip6 = &expand_ip6($ip6);
+return ($ip6 =~ /^([\da-f]{1,4}:){7}([\da-f]{1,4})$/i);
+}
+
+sub time_unit_choice 
+{
+print "<select name=$_[0]>\n";
+printf "<option %s value=\"\">$text{'seconds'}\n",
+       $_[1] =~ /^(S?)$/i ? "selected" : "";
+printf "<option %s value=\"M\">$text{'minutes'}\n",
+       $_[1] =~ /M/i ? "selected" : "";
+printf "<option %s value=\"H\">$text{'hours'}\n",
+       $_[1] =~ /H/i ? "selected" : "";
+printf "<option %s value=\"D\">$text{'days'}\n",
+       $_[1] =~ /D/i ? "selected" : "";
+printf "<option %s value=\"W\">$text{'weeks'}\n",
+       $_[1] =~ /W/i ? "selected" : "";
+print "</select>\n";
+}
+
+sub extract_time_units
+{
+local(@ret);
+foreach $j (@_) {
+	if ($j =~ /^(\d+)([SMHDW]?)$/is) {
+		push(@ret, $2); $j = $1;
+		}
+	}
+return @ret;
+}
+
+sub email_to_dotted
+{
+local $v = $_[0];
+$v =~ s/\.$//;
+if ($v =~ /^([^.]+)\@(.*)$/) {
+	return "$1.$2.";
+	}
+elsif ($v =~ /^(.*)\@(.*)$/) {
+	local ($u, $d) = ($1, $2);
+	$u =~ s/\./\\\./g;
+	return "\"$u.$d.\"";
+	}
+else {
+	return $v;
+	}
+}
+
+sub dotted_to_email
+{
+local $v = $_[0];
+if ($v ne ".") {
+	$v =~ s/([^\\])\./$1\@/;
+	$v =~ s/\\\./\./g;
+	$v =~ s/\.$//;
+	}
+return $v;
+}
+
+# set_ownership(file)
+sub set_ownership
+{
+if ($config{'file_owner'}) {
+	&system_logged("chown \"$config{'file_owner'}\" $_[0] >/dev/null 2>&1");
+	}
+if ($config{'file_perms'}) {
+	&system_logged("chmod \"$config{'file_perms'}\" $_[0] >/dev/null 2>&1");
+	}
+}
+
 @cat_list = ( 'default', 'config', 'parser', 'queries',
 	      'lame-servers', 'statistics', 'panic', 'update',
 	      'ncache', 'xfer-in', 'xfer-out', 'db',
