@@ -8,6 +8,8 @@ sub acl_security_form
 print "<tr> <td valign=top><b>$text{'acl_users'}</b></td> <td colspan=3>\n";
 printf "<input type=radio name=mode value=0 %s> $text{'acl_all'}<br>\n",
 	$_[0]->{'mode'} == 0 ? "checked" : "";
+printf "<input type=radio name=mode value=3 %s> $text{'acl_this'}<br>\n",
+	$_[0]->{'mode'} == 3 ? "checked" : "";
 printf "<input type=radio name=mode value=1 %s> $text{'acl_only'}\n",
 	$_[0]->{'mode'} == 1 ? "checked" : "";
 printf "<input name=userscan size=40 value='%s'> %s<br>\n",
@@ -31,7 +33,7 @@ printf "<input type=radio name=allow value=0 %s> $text{'no'}</td> </tr>\n",
 sub acl_security_save
 {
 $_[0]->{'mode'} = $in{'mode'};
-$_[0]->{'users'} = $in{'mode'} == 0 ? "" :
+$_[0]->{'users'} = $in{'mode'} == 0 || $in{'mode'} == 3 ? "" :
 		   $in{'mode'} == 1 ? $in{'userscan'}
 				    : $in{'userscannot'};
 $_[0]->{'allow'} = $in{'allow'};
