@@ -3,7 +3,8 @@
 
 do '../web-lib.pl';
 &init_config();
-$cron_temp_file = "/tmp/cronadmin_temp";
+%access = &get_module_acl();
+$cron_temp_file = &tempname();
 
 # list_cron_jobs()
 # Returns a lists of structures of all cron jobs
@@ -27,6 +28,12 @@ if ($config{'vixie_cron'}) {
 				    'weekdays' => $7, 'user' => $8,
 				    'command' => $9,
 				    'index' => scalar(@rv) });
+			if ($rv[$#rv]->{'user'} =~ /^\//) {
+				# missing the user, as in redhaty 7 !
+				$rv[$#rv]->{'command'} = $rv[$#rv]->{'user'}.
+					' '.$rv[$#rv]->{'command'};
+				$rv[$#rv]->{'user'} = 'root';
+				}
 			}
 		$lnum++;
 		}
@@ -99,10 +106,12 @@ return join(" ", @c);
 # create_cron_job(&job)
 sub create_cron_job
 {
-system("cp $config{'cron_dir'}/$_[0]->{'user'} $cron_temp_file");
+unlink($cron_temp_file);
+system("cp $config{'cron_dir'}/$_[0]->{'user'} $cron_temp_file 2>/dev/null");
 open(TAB, ">>$cron_temp_file");
 print TAB &cron_job_line($_[0]),"\n";
 close(TAB);
+system("chown $_[0]->{'user'} $cron_temp_file");
 &copy_crontab($_[0]->{'user'});
 }
 
@@ -110,6 +119,7 @@ close(TAB);
 sub change_cron_job
 {
 if ($_[0]->{'type'} == 0) {
+	unlink($cron_temp_file);
 	system("cp $config{'cron_dir'}/$_[0]->{'user'} $cron_temp_file");
 	&replace_file_line($cron_temp_file, $_[0]->{'line'},
 			   &cron_job_line($_[0])."\n");
@@ -125,6 +135,7 @@ else {
 sub delete_cron_job
 {
 if ($_[0]->{'type'} == 0) {
+	unlink($cron_temp_file);
 	system("cp $config{'cron_dir'}/$_[0]->{'user'} $cron_temp_file");
 	&replace_file_line($cron_temp_file, $_[0]->{'line'});
 	&copy_crontab($_[0]->{'user'});
@@ -160,12 +171,13 @@ if (`cat $cron_temp_file` =~ /\S/) {
 		chop($pwd = `pwd`);
 		$ENV{"VISUAL"} = $ENV{"EDITOR"} = "$pwd/cron_editor.pl";
 		$ENV{"CRON_EDITOR_COPY"} = $cron_temp_file;
-		system(&user_sub($config{cron_edit_command},$_[0]).
+		system("chown $_[0] $cron_temp_file");
+		system(&user_sub($config{'cron_edit_command'},$_[0]).
 		       " >/dev/null 2>/dev/null");
 		}
 	else {
 		# use the cron copy command
-		system(&user_sub($config{cron_copy_command},$_[0]).
+		system(&user_sub($config{'cron_copy_command'},$_[0]).
 		       " <$cron_temp_file >/dev/null 2>/dev/null");
 		}
 	}
@@ -314,7 +326,127 @@ local %umap;
 map { $umap{$_}++; } split(/\s+/, $_[0]->{'users'});
 if ($_[0]->{'mode'} == 1 && !$umap{$_[1]} ||
     $_[0]->{'mode'} == 2 && $umap{$_[1]}) { return 0; }
-else { return 1; }
+elsif ($_[0]->{'mode'} == 3) {
+	return $remote_user eq $_[1];
+	}
+else {
+	return 1;
+	}
+}
+
+# show_times_input(&job)
+sub show_times_input
+{
+local $job = $_[0];
+print "<td><b>$text{'edit_mins'}</b></td> <td><b>$text{'edit_hours'}</b></td> ",
+      "<td><b>$text{'edit_days'}</b></td> <td><b>$text{'edit_months'}</b></td>",
+      "<td><b>$text{'edit_weekdays'}</b></td> </tr> <tr $cb>\n";
+
+local @mins = (0..59);
+local @hours = (0..23);
+local @days = (1..31);
+local @months = map { $text{"month_$_"}."=".$_ } (1 .. 12);
+local @weekdays = map { $text{"day_$_"}."=".$_ } (0 .. 6);
+
+foreach $arr ("mins", "hours", "days", "months", "weekdays") {
+	# Find out which ones are being used
+	local %inuse;
+	local $min = ($arr =~ /days|months/ ? 1 : 0);
+	local $max = $min+scalar(@$arr)-1;
+	foreach $w (split(/,/ , $job->{$arr})) {
+		if ($w eq "*") {
+			# all values
+			for($j=$min; $j<=$max; $j++) { $inuse{$j}++; }
+			}
+		elsif ($w =~ /^\*\/(\d+)$/) {
+			# only every Nth
+			for($j=$min; $j<=$max; $j+=$1) { $inuse{$j}++; }
+			}
+		elsif ($w =~ /^(\d+)-(\d+)\/(\d+)$/) {
+			# only every Nth of some range
+			for($j=$1; $j<=$2; $j+=$3) { $inuse{int($j)}++; }
+			}
+		elsif ($w =~ /^(\d+)-(\d+)$/) {
+			# all of some range
+			for($j=$1; $j<=$2; $j++) { $inuse{int($j)}++; }
+			}
+		else {
+			# One value
+			$inuse{int($w)}++;
+			}
+		}
+	if ($job->{$arr} eq "*") { undef(%inuse); }
+
+	# Output selection list
+	print "<td valign=top>\n";
+	printf "<input type=radio name=all_$arr value=1 %s> $text{'edit_all'}<br>\n",
+		$job->{$arr} eq "*" ? "checked" : "";
+	printf "<input type=radio name=all_$arr value=0 %s> $text{'edit_selected'}<br>\n",
+		$job->{$arr} ne "*" ? "checked" : "";
+	print "<table> <tr>\n";
+	for($j=0; $j<@$arr; $j+=12) {
+		$jj = $j+11;
+		if ($jj >= @$arr) { $jj = @$arr - 1; }
+		@sec = @$arr[$j .. $jj];
+		printf "<td valign=top><select multiple size=%d name=$arr>\n",
+			@sec > 12 ? 12 : scalar(@sec);
+		foreach $v (@sec) {
+			if ($v =~ /^(.*)=(.*)$/) { $disp = $1; $code = $2; }
+			else { $disp = $code = $v; }
+			printf "<option value=\"$code\" %s>$disp\n",
+				$inuse{$code} ? "selected" : "";
+			}
+		print "</select></td>\n";
+		}
+	print "</tr></table></td>\n";
+	}
+print "</tr>\n";
+}
+
+# parse_times_input(&job, &in)
+sub parse_times_input
+{
+local $job = $_[0];
+local %in = %{$_[1]};
+foreach $arr ("mins", "hours", "days", "months", "weekdays") {
+	if ($in{"all_$arr"}) {
+		# All mins/hrs/etc.. chosen
+		$job->{$arr} = "*";
+		}
+	elsif (defined($in{$arr})) {
+		# Need to work out and simplify ranges selected
+		local (@range, @newrange, $i);
+		@range = split(/\0/, $in{$arr});
+		@range = sort { $a <=> $b } @range;
+		local $start = -1;
+		for($i=0; $i<@range; $i++) {
+			if ($i && $range[$i]-1 == $range[$i-1]) {
+				# ok.. looks like a range
+				if ($start < 0) { $start = $i-1; }
+				}
+			elsif ($start < 0) {
+				# Not in a range at all
+				push(@newrange, $range[$i]);
+				}
+			else {
+				# End of the range.. add it
+				$newrange[@newrange - 1] =
+					"$range[$start]-".$range[$i-1];
+				push(@newrange, $range[$i]);
+				$start = -1;
+				}
+			}
+		if ($start >= 0) {
+			# Reached the end while in a range
+			$newrange[@newrange - 1] =
+				"$range[$start]-".$range[$i-1];
+			}
+		$job->{$arr} = join(',' , @newrange);
+		}
+	else {
+		&error(&text('save_enone', $text{"edit_$arr"}));
+		}
+	}
 }
 
 1;
