@@ -7,11 +7,15 @@ do '../web-lib.pl';
 # get_parent_config()
 # Returns a dummy parent structure for the DHCP config
 sub get_parent_config
-{
-return { 'file' => $config{'dhcpd_conf'},
-	 'members' => &get_config(),
-	 'line' => -1,
-	 'eline' => $get_config_lines };
+{ 
+return $get_parent_config_cache if ($get_parent_config_cache);
+return ($get_parent_config_cache =
+		{
+		 'file' => $config{'dhcpd_conf'},
+	     'members' => &get_config(),
+	     'line' => -1,
+	     'eline' => $get_config_lines 
+		  } );
 }
 
 # get_config()
@@ -40,10 +44,12 @@ open(FILE, $_[0]);
 while($line = <FILE>) {
 	# strip comments
 	$line =~ s/\r|\n//g;
-	$line =~ s/^([^"]*)#.*$/$1/g;
-	$line =~ s/\/\/.*$//g;
+	$line =~ s/^([^"#]*)#.*$/$1/g;
+	$line =~ s/^([^"]*)\/\/.*$/$1/g;
+	$line =~ s/\s+#.*$//g;
+	$line =~ s/\\"/\\176/g;
 	while(1) {
-		if (!$cmode && $line =~ /\/\*/) {
+		if (!$cmode && $line =~ /\/\*/ && $line !~ /\".*\/\*.*\"/) {
 			# start of a C-style comment
 			$cmode = 1;
 			$line =~ s/\/\*.*$//g;
@@ -83,7 +89,7 @@ return $lines;
 # Pos will end up at the start of the next structure
 sub parse_struct
 {
-local(%str, $i, $t, @vals, @quotes, $str);
+local(%str, $i, $t, @vals, @quotes, $str, @text);
 $i = ${$_[1]};
 $str{'name'} = $_[0]->[$i]->[0];
 $str{'line'} = $_[0]->[$i]->[2];
@@ -92,10 +98,12 @@ while(1) {
 	if ($t->[0] eq "{" || $t->[0] eq ";") { last; }
 	elsif (!defined($t->[0])) { ${$_[1]} = $i; return undef; }
 	else { push(@vals, $t->[0]); push(@quotes, $t->[1]); }
+	push(@text, $t->[1] ? "\"$t->[0]\"" : $t->[0]);
 	}
 $str{'values'} = \@vals;
 $str{'quotes'} = \@quotes;
 $str{'value'} = $vals[0];
+$str{'text'} = join(" ", @text);
 $str{'index'} = $_[2];
 $str{'file'} = $_[3];
 if ($t->[0] eq "{") {
@@ -405,5 +413,250 @@ if ($d1->{'type'}) {
 return 0;
 }
 
-1;
+# group_name($members)
+sub group_name
+{
+return ($_[0] == 0) ? $text{'index_nomemb'} :
+	($_[0] == 1) ? $text{'index_1memb'} :
+	($_[0] >= 2 && $_[0] <= 4) ? &text('index_234memb', $_[0]) :
+	&text('index_memb', $_[0]);
 
+}
+
+# get_subnets_and_hosts() 
+# returns the references to sorted lists of hosts and subnets
+sub get_subnets_and_hosts
+{
+return (\@get_subnets_cache, \@get_hosts_cache) 
+	if (@get_subnets_cache && @get_hosts_cache);
+
+local(@subn,@host,@group,@shan, $s,$h,$g,$sn, $conf);
+$conf = &get_config();
+
+# get top level hosts and groups
+@host = &find("host", $conf);
+foreach $h (&find("host", $conf)) {
+	$h->{'order'} = $h->{'index'};
+	}
+@group = &find("group", $conf);
+foreach $g (@group) {
+	foreach $h (&find("host", $g->{'members'})) {
+		push(@host, $h);
+		}
+	}
+@subn = &find("subnet", $conf);
+foreach $u (@subn) {
+	foreach $h (&find("host", $u->{'members'})) {
+		push(@host, $h);
+		}
+	foreach $g (&find("group", $u->{'members'})) {
+		push(@group, $g);
+		foreach $h (&find("host", $g->{'members'})) {
+			push(@host, $h);
+			}
+		}
+	}
+@shan = &find("shared-network", $conf);
+foreach $s (@shan) {
+	foreach $h (&find("host", $s->{'members'})) {
+		push(@host, $h);
+		}
+	foreach $g (&find("group", $s->{'members'})) {
+		push(@group, $g);
+		foreach $h (&find("host", $g->{'members'})) {
+			push(@host, $h);
+			}
+		}
+	foreach $u (&find("subnet", $s->{'members'})) {
+		push(@subn, $u);
+		foreach $h (&find("host", $u->{'members'})) {
+			push(@host, $h);
+			}
+		foreach $g (&find("group", $sn->{'members'})) {
+			push(@group, $g);
+			foreach $h (&find("host", $g->{'members'})) {
+				push(@host, $h);
+				}
+			}
+		}
+	}
+@get_subnets_cache = sort { $a->{'order'} <=> $b->{'order'} } @subn;
+@get_hosts_cache = sort { $a->{'order'} <=> $b->{'order'} } @host;
+
+return (\@get_subnets_cache, \@get_hosts_cache);
+}
+
+sub get_subnets
+{
+local ($sr, $hr) = &get_subnets_and_hosts();
+return @{$sr};
+}
+
+sub get_hosts
+{
+local ($sr, $hr) = &get_subnets_and_hosts();
+return @{$hr};
+}
+
+# hash that links objtypes shortcuts with object names
+%obj_names2types = qw(host hst group grp subnet sub shared-network sha);
+
+# get_branch($objtype) 
+# usefull for edit_*.cgi and save_*.cgi scripts
+# $objtype = one of 'hst' 'grp' 'sub' 'sha'
+sub get_branch
+{
+local %obj_types2names = reverse %obj_names2types;
+local $name = $obj_types2names{$_[0]};
+local ($parnode, $nparnode, $node, $indent, $nindent);
+$parnode = $nparnode = &get_parent_config();
+$indent = $nindent = 0;
+foreach ($in{'sidx'}, $in{'uidx'}, $in{'gidx'}) {
+    if ($_ ne '') {
+		$parnode = $parnode->{'members'}->[$_];
+		$indent++;
+		}
+    }
+
+if (!($in{'delete'} && $in{'options'})) {
+	if ($in{'assign'} > 0 && !defined($in{'parent'})) {
+		# A quirk for not javascript-capable browser
+		# New parent is undefined yet; we need 2nd step
+		undef $nparnode;
+		}
+	else {
+		foreach (split(/\,/, $in{'parent'})) {
+			$nindent++;
+			if ($_ < @{$nparnode->{'members'}}) {
+				$nparnode = $nparnode->{'members'}->[$_];
+				}
+			}
+		}
+	}
+
+if (!$in{'new'}) {
+	$node = $parnode->{'members'}->[$in{'idx'}];
+	}
+else {
+	die "Wrong call to get_nodes: pass objtype for new object" unless $name;
+	# Construct new node structure
+	$node->{'name'} = $name;
+	$node->{'type'} = 1;
+	$node->{'members'} = [ ];
+	}
+return ($parnode, $node, $indent, $nparnode, $nindent);
+}
+
+# can(permissions_string, \%access, \%config_node, smode)
+# this is a cached wrapper of can_noncached(...)
+sub can
+{
+local ($perm, $acc, $node, $smode) = @_;
+if (defined($can_cache) &&
+	($can_perm_cache eq $perm) &&
+	($can_node_cache eq $node) &&
+	($can_smode_cache eq $smode)) {
+	return $can_cache;
+	}
+else {
+	$can_perm_cache = $perm;
+	$can_node_cache = $node;
+	$can_smode_cache = $smode;
+	return ($can_cache = &can_noncached(@_));
+	}
+}
+
+# can_noncached(permissions_string, \%access, \%config_node, smode)
+# check global and per-object permissions:
+#
+# permissions_string= 'c' 'r' 'w' or any combination.
+# smode= 0 or undef - check only current, 1 - recursive childs check, 
+#	2 - check parents, 3 - check parents and all childs
+#	note: while deleting an object you must allways enforce smode=1 or 3
+#		because all child objects are deletes recursevly. 
+#	this maybe an optional parameter 
+sub can_noncached
+{
+local $acl;
+local ($perm, $acc, $node, $smode) = @_;
+local @perm = split(//, $perm);
+
+if ($node ne get_parent_config()) {
+	foreach (@perm) { 
+		next if ($_ ne 'c') &&  ($_ ne 'r') && ($_ ne 'w');
+		return 0 unless $acc->{$_ . '_' . $obj_names2types{$node->{'name'}} };
+		}
+
+	# per-object permissions
+	return 0 unless &can_node(\@perm, $acc, $node);
+
+	if (($acc->{'smode'} == 2) || ($smode == 2) ||
+	    ($acc->{'smode'} == 3) || ($smode == 3)) {
+		# check parents
+		$parnode=&get_parent_config();								
+		foreach ($in{'sidx'}, $in{'uidx'}, $in{'gidx'}) {
+			if ($_ ne '') {
+				$parnode = $parnode->{'members'}->[$_];
+				return 0 unless &can_node(\@perm, $acc, $parnode);
+				}
+			}
+		}
+		
+	if (($acc->{'smode'} == 1) || ($smode == 1) ||
+		($acc->{'smode'} == 3) || ($smode == 3)) {
+		# check childs
+		return 0 unless &can_subtree(\@perm, $acc, $node);
+		}
+	}
+return 1;
+}
+
+# can_node(\@perm, $acc, $node)
+# checks object permissions for current node
+sub can_node
+{
+local ($rperm, $acc, $node)=@_;
+# per-object permissions
+local $otype=$obj_names2types{$node->{'name'}};
+if ($acc->{'per_' . $otype . '_acls'}) {  
+	local $acl = $acc->{'ACL' . $otype . '_' . $node->{'values'}->[0]};
+	foreach (@{$rperm}) {
+		next if $_ eq 'c'; # skip creation perms for per-obj acls
+		return 0 if index($acl, $_) == -1;
+		}
+	}
+return 1;
+}
+
+# can_subtree(\@perm, $acc, $node)
+# checks object permissions for subtree
+sub can_subtree
+{
+local ($rperm, $acc, $node)=@_;
+return 0 unless &can_node($rperm, $acc, $node); 
+if($node->{'members'}) {
+	# recursevly process this subtree
+	foreach (@{$node->{'members'}}) {
+		return 0 unless &can_subtree($rperm, $acc, $_);
+		}
+	}
+return 1;	
+}
+
+# save_dhcpd_acl(permissions_string, obj_type, \%access, obj_name)
+sub save_dhcpd_acl
+{
+$_[2]->{'ACL'.$_[1].'_'.$_[3]} = $_[0];
+undef($can_cache);
+return &save_module_acl($_[2]);
+}
+
+# drop_dhcpd_acl(obj_type, \%access, obj_name)
+sub drop_dhcpd_acl
+{
+delete($_[1]->{'ACL'.$_[0].'_'.$_[2]});
+undef($can_cache);
+return &save_module_acl($_[1]);
+}
+
+1;
