@@ -4,7 +4,7 @@
 do '../web-lib.pl';
 &init_config();
 &foreign_require("mount", "mount-lib.pl");
-&foreign_require("raid", "raid-lib.pl");
+&foreign_require("raid", "raid-lib.pl") if (&foreign_check("raid"));
 &foreign_require("proc", "proc-lib.pl");
 %access = &get_module_acl();
 $| = 1;
@@ -13,6 +13,7 @@ $| = 1;
 # Returns a list of structures, one per disk
 sub list_disks
 {
+return @list_disks_cache if (defined(@list_disks_cache));
 local (@rv, $f, $i);
 local $idx = 0;
 if ((-d "/proc/ide" || -d "/proc/scsi") && -r "/proc/partitions") {
@@ -47,14 +48,20 @@ if ((-d "/proc/ide" || -d "/proc/scsi") && -r "/proc/partitions") {
 	closedir(IDE);
 
 	# Get the list of SCSI disks from the kernel and fdisk
-	local @scsi;
+	local (@scsi, %dac, %raid, %ida);
 	open(PARTS, "/proc/partitions");
 	while(<PARTS>) {
-		s/\r|\n//g;
-		if (/\d+\s+\d+\s+\d+\s+sd(\S)$/) {
+		if (/\d+\s+\d+\s+\d+\s+sd(\S)\s/) {
+			# Old-style SCSI disk
 			push(@scsi, "/dev/sd$1");
 			}
-		elsif (/\d+\s+\d+\s+\d+\s+(rd\/c(\d+)d\d+)$/) {
+		elsif (/\d+\s+\d+\s+\d+\s+(scsi\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/disc)\s+/) {
+			# New complete SCSI disk specification
+			push(@scsi, "/dev/$1");
+			$scsidev{"/dev/$1"} = [ $2, $3, $4, $5 ];
+			}
+		elsif (/\d+\s+\d+\s+\d+\s+(rd\/c(\d+)d\d+)\s/) {
+			# Mylex raid device
 			local ($d, $c) = ("/dev/$1", $2);
 			push(@scsi, $d);
 			open(RD, "/proc/rd/c$c/current_status");
@@ -68,6 +75,18 @@ if ((-d "/proc/ide" || -d "/proc/scsi") && -r "/proc/partitions") {
 				}
 			close(RD);
 			}
+		elsif (/\d+\s+\d+\s+\d+\s+(ida\/c(\d+)d\d+)\s/) {
+			# Compaq raid device
+			local ($d, $c) = ($1, "/dev/$1", $2);
+			push(@scsi, $d);
+			open(IDA, "/proc/driver/array/ida$c");
+			while(<IDA>) {
+				if (/^(\S+):\s+(.*)/ && $1 eq "ida$c") {
+					$ida{$d} = $2;
+					}
+				}
+			close(IDA);
+			}
 		}
 	close(PARTS);
 	local @pscsi;
@@ -109,6 +128,17 @@ if ((-d "/proc/ide" || -d "/proc/scsi") && -r "/proc/partitions") {
 					$disk->{'desc'} = &text('select_mylex', "$1", "$2");
 					$disk->{'prefix'} = $disk->{'device'}.'p';
 					}
+				elsif ($ida{$disk->{'device'}}) {
+					$disk->{'model'} = $ida{$disk->{'device'}};
+					$disk->{'device'} =~ /c(\d+)d(\d+)$/;
+					$disk->{'desc'} = &text('select_cpq', "$1", "$2");
+					$disk->{'prefix'} = $disk->{'device'}.'p';
+					}
+				elsif ($scsidev{$disk->{'device'}}) {
+					local @sd = @{$scsidev{$disk->{'device'}}};
+					$disk->{'desc'} = &text('select_scsi', @sd);
+					($disk->{'prefix'} = $disk->{'device'}) =~ s/disc$/part/g;
+					}
 				else {
 					$disk->{'desc'} = &text('select_device', 'SCSI',
 								uc(substr($disk->{'device'}, -1)));
@@ -120,10 +150,15 @@ if ((-d "/proc/ide" || -d "/proc/scsi") && -r "/proc/partitions") {
 		&close_fdisk();
 		}
 	}
-else {
-	# Get the list from fdisk -l
+
+if (!@rv) {
+	# Get the list from fdisk -l or sfdisk -l
 	local @cdstat = stat("/dev/cdrom");
-	&open_fdisk("-l");
+	if (&has_command("sfdisk")) {
+		&open_sfdisk("-l");
+	} else {
+		&open_fdisk("-l");
+	}
 	while(<$fh>) {
 		if (/Disk\s+([^ :]+):\s+(\d+)\s+\S+\s+(\d+)\s+\S+\s+(\d+)/) {
 			local @st = stat($1);
@@ -143,7 +178,9 @@ else {
 				}
 			}
 		}
+	&close_fdisk();
 	}
+@list_disks_cache = @rv;
 return @rv;
 }
 
@@ -151,6 +188,8 @@ return @rv;
 # Returns a list of structures, one per partition
 sub list_partitions
 {
+return @{$list_partitions_cache{$_[0]}}
+	if (defined($list_partitions_cache{$_[0]}));
 local(@rv, $wf);
 local $idx = 0;
 &open_fdisk($_[0]);
@@ -171,11 +210,16 @@ while(1) {
 		    'extended' => $matches[6] eq '5' || $matches[6] eq 'f' ? 1 : 0,
 		    'desc' => $matches[1] =~ /(.)d(\S)(\d+)$/ ?
 			      &text('select_part', $1 eq 's' ? 'SCSI' : 'IDE', uc($2), "$3") :
+			      $matches[1] =~ /scsi\/host(\d+)\/bus(\d+)\/target(\d+)\/lun(\d+)\/part(\d+)/ ?
+			      &text('select_spart', "$1", "$2", "$3", "$4", "$5") :
 			      $matches[1] =~ /rd\/c(\d+)d(\d+)p(\d+)$/ ? 
-			      &text('select_mpart', "$1", "$2", "$3") : "???",
+			      &text('select_mpart', "$1", "$2", "$3") :
+			      $matches[1] =~ /ida\/c(\d+)d(\d+)p(\d+)$/ ? 
+			      &text('select_cpart', "$1", "$2", "$3") : "???",
 		    'index' => $idx++ });
 	}
 &close_fdisk();
+$list_partitions_cache{$_[0]} = \@rv;
 return @rv;
 }
 
@@ -299,8 +343,8 @@ return $fstypes{$_[0]};
 sub mkfs_options
 {
 if ($_[0] eq "ext2") {
-	&opt_input("ext2_b", "bytes", 1);
-	&opt_input("ext2_f", "bytes", 0);
+	&opt_input("ext2_b", $text{'bytes'}, 1);
+	&opt_input("ext2_f", $text{'bytes'}, 0);
 	&opt_input("ext2_i", "", 1);
 	&opt_input("ext2_m", "%", 0);
 	&opt_input("ext2_g", "", 1);
@@ -310,7 +354,7 @@ if ($_[0] eq "ext2") {
 	      "</td> </tr>\n";
 	}
 elsif ($_[0] eq "msdos" || $_[0] eq "vfat") {
-	&opt_input("msdos_f", "", 1);
+	&opt_input("msdos_ff", "", 1);
 	&opt_input("msdos_F", "bits", 0);
 	&opt_input("msdos_i", "", 1);
 	&opt_input("msdos_n", "", 0);
@@ -330,6 +374,38 @@ elsif ($_[0] eq "minix") {
 	print "<input type=radio name=msdos_c value=0 checked> $text{'no'}",
 	      "</td> </tr>\n";
 	}
+elsif ($_[0] eq "reiserfs") {
+	print "<tr> <td><b>$text{'reiserfs_force'}</b></td>\n";
+	print "<td><input type=radio name=reiserfs_f value=1> $text{'yes'}\n";
+	print "<input type=radio name=reiserfs_f value=0 checked> ",
+	      "$text{'no'}</td>\n";
+
+	print "<td><b>$text{'reiserfs_hash'}</b></td>\n";
+	print "<td><select name=reiserfs_h>\n";
+	print "<option value='' checked>$text{'default'}\n";
+	print "<option>rupasov\n";
+	print "<option>tea\n";
+	print "</select></td> </tr>\n";
+	}
+elsif ($_[0] eq "ext3") {
+	&opt_input("ext2_b", $text{'bytes'}, 1);
+	&opt_input("ext2_f", $text{'bytes'}, 0);
+	&opt_input("ext2_i", "", 1);
+	&opt_input("ext2_m", "%", 0);
+	&opt_input("ext3_j", "MB", 1);
+	print "<td align=right><b>$text{'ext2_c'}</b></td>\n";
+	print "<td><input type=radio name=ext2_c value=1> $text{'yes'}\n";
+	print "<input type=radio name=ext2_c value=0 checked> $text{'no'}",
+	      "</td> </tr>\n";
+	}
+elsif ($_[0] eq "xfs") {
+	print "<tr> <td><b>$text{'xfs_force'}</b></td>\n";
+	print "<td><input type=radio name=xfs_f value=1> $text{'yes'}\n";
+	print "<input type=radio name=xfs_f value=0 checked> ",
+	      "$text{'no'}</td>\n";
+
+	&opt_input("xfs_b", $text{'bytes'}, 0);
+	}
 }
 
 # mkfs_parse(type, device)
@@ -350,7 +426,7 @@ if ($_[0] eq "ext2") {
 	}
 elsif ($_[0] eq "msdos" || $_[0] eq "vfat") {
 	$cmd = "mkfs -t $_[0]";
-	$cmd .= &opt_check("msdos_f", '[1-2]', "-f");
+	$cmd .= &opt_check("msdos_ff", '[1-2]', "-f");
 	$cmd .= &opt_check("msdos_F", '\d+', "-F");
 	$cmd .= &opt_check("msdos_i", '[0-9a-f]{8}', "-i");
 	$cmd .= &opt_check("msdos_n", '\S{1,11}', "-n");
@@ -365,8 +441,35 @@ elsif ($_[0] eq "minix") {
 	$cmd .= &opt_check("minix_n", '14|30', "-n ");
 	$cmd .= &opt_check("minix_i", '\d+', "-i ");
 	$cmd .= $in{'minix_c'} ? " -c" : "";
-	$cmd .= " $_[1]";
 	$cmd .= &opt_check("minix_b", '\d+', " ");
+	$cmd .= " $_[1]";
+	}
+elsif ($_[0] eq "reiserfs") {
+	$cmd = "yes | mkreiserfs";
+	$cmd .= " -f" if ($in{'reiserfs_f'});
+	$cmd .= " -h $in{'reiserfs_h'}" if ($in{'reiserfs_h'});
+	$cmd .= " $_[1]";
+	}
+elsif ($_[0] eq "ext3") {
+	if (&has_command("mkfs.ext3")) {
+		$cmd = "mkfs -t ext3";
+		}
+	else {
+		$cmd = "mke3fs";
+		}
+	$cmd .= &opt_check("ext2_b", '\d+', "-b");
+	$cmd .= &opt_check("ext2_f", '\d+', "-f");
+	$cmd .= &opt_check("ext2_i", '\d{4,}', "-i");
+	$cmd .= &opt_check("ext2_m", '\d+', "-m");
+	$cmd .= &opt_check("ext3_j", '\d+', "-j");
+	$cmd .= $in{'ext2_c'} ? " -c" : "";
+	$cmd .= " $_[1]";
+	}
+elsif ($_[0] eq "xfs") {
+	$cmd = "mkfs -t $_[0]";
+	$cmd .= " -f" if ($in{'xfs_f'});
+	$cmd .= " -b size=$in{'xfs_b'}" if (!$in{'xfs_b_def'});
+	$cmd .= " $_[1]";
 	}
 return $cmd;
 }
@@ -451,23 +554,27 @@ sub device_status
 {
 @mounted = &foreign_call("mount", "list_mounted") if (!@mounted);
 @mounts = &foreign_call("mount", "list_mounts") if (!@mounts);
-$raidconf = &foreign_call("raid", "get_raidtab") if (!$raidconf);
+local $label = `e2label $_[0] 2>/dev/null`;
+chop($label);
 
-local ($mounted) = grep { $_->[1] eq $_[0] } @mounted;
-local ($mount) = grep { $_->[1] eq $_[0] } @mounts;
+local ($mounted) = grep { $_->[1] eq $_[0] ||
+			  $_->[1] eq "LABEL=$label" } @mounted;
+local ($mount) = grep { $_->[1] eq $_[0] ||
+			$_->[1] eq "LABEL=$label" } @mounts;
 if ($mounted) { return ($mounted->[0], $mounted->[2], 1,
 			&indexof($mount, @mounts),
 			&indexof($mounted, @mounted)); }
 elsif ($mount) { return ($mount->[0], $mount->[2], 0,
 			 &indexof($mount, @mounts)); }
-else {
+elsif (&foreign_check("raid")) {
+	$raidconf = &foreign_call("raid", "get_raidtab") if (!$raidconf);
 	foreach $c (@$raidconf) {
-		if (&indexof($_[0], @{$c->{'device'}})) {
+		if (&indexof($_[0], @{$c->{'devices'}}) >= 0) {
 			return ( $c->{'value'}, "raid", 1 );
 			}
 		}
-	return ();
 	}
+return ();
 }
 
 # can_fsck(type)
@@ -498,7 +605,7 @@ return $text{"fsck_err$_[0]"} ? $text{"fsck_err$_[0]"}
 			      : &text("fsck_unknown", $_[0]);
 }
 
-# partition_select(name, value, mode, &found)
+# partition_select(name, value, mode, [&found], [disk_regexp])
 # Returns HTML for selecting a disk or partition
 # mode 0 = floppies and disk partitions
 #      1 = disks
@@ -511,15 +618,16 @@ local ($found, $d, $p);
 if ($_[2] == 0 || $_[2] == 2) {
 	$rv .= sprintf "<option %s value=/dev/fd0>%s\n",
 		$_[1] eq "/dev/fd0" ? "selected" : "",
-		&text('select_fd', 0);
+		&text('select_fd', 0) if (!$_[4] || "/dev/fd0" =~ /$_[4]/);
 	$rv .= sprintf "<option %s value=/dev/fd1>%s\n",
 		$_[1] eq "/dev/fd1" ? "selected" : "",
-		&text('select_fd', 1);
+		&text('select_fd', 1) if (!$_[4] || "/dev/fd1" =~ /$_[4]/);
 	$found++ if ($_[1] =~ /^\/dev\/fd[01]$/);
 	}
 local @dlist = &list_disks();
 foreach $d (@dlist) {
 	local $dev = $d->{'device'};
+	next if ($_[4] && $dev !~ /$_[4]/);
 	if ($_[2] == 1 || $_[2] == 2) {
 		local $name = $d->{'desc'};
 		$name .= " ($d->{'model'})" if ($d->{'model'});
@@ -551,16 +659,56 @@ $rv .= "</select>\n";
 return $rv;
 }
 
+# label_select(name, value, &found)
+sub label_select
+{
+local $rv = "<select name=$_[0]>\n";
+local @dlist = &list_disks();
+local $any;
+foreach $d (@dlist) {
+	local $dev = $d->{'device'};
+	local @parts = &list_partitions($dev);
+	foreach $p (@parts) {
+		next if ($p->{'type'} ne '83');
+		local $label = `e2label $p->{'device'} 2>&1`;
+		next if ($? || $label !~ /\S/);
+		chop($label);
+		$rv .= sprintf "<option %s value=%s>%s (%s)\n",
+			  $_[1] eq $label ? "selected" : "",
+			  $label, $label, $p->{'desc'};
+		${$_[2]}++ if ($_[1] eq $label);
+		$any++;
+		}
+	}
+if (!$found && $_[1] && !$_[2]) {
+	$rv .= "<option selected>$_[1]\n";
+	}
+$rv .= "</select>\n";
+return $any ? $rv : undef;
+}
+
 #############################################################################
 # Internal functions
 #############################################################################
 sub open_fdisk
 {
-local $fpath = &has_command("fdisk");
-&error("<tt>fdisk</tt> command not found") if (!$fpath);
+local $fpath = &check_fdisk();
 ($fh, $fpid) = &foreign_call("proc", "pty_process_exec", $fpath, @_);
 }
 
+sub open_sfdisk
+{
+local $sfpath = &has_command("sfdisk");
+($fh, $fpid) = &foreign_call("proc", "pty_process_exec", $sfpath, @_);
+}
+
+sub check_fdisk
+{
+local $fpath = &has_command("fdisk");
+&error(&text('open_error', "<tt>fdisk</tt>")) if (!$fpath);
+return $fpath;
+}
+
 sub close_fdisk
 {
 close($fh); kill('TERM', $fpid);
@@ -644,6 +792,7 @@ else { return " $_[2] $in{$_[0]}"; }
 	'85', 'Linux extended',
 	'86', 'NTFS volume set (86)',
 	'87', 'NTFS volume set (87)',
+	'8e', 'Linux LVM',
 	'93', 'Amoeba',
 	'94', 'Amoeba BBT',
 	'a0', 'IBM Thinkpad hibernation',
@@ -677,9 +826,12 @@ else { return " $_[2] $in{$_[0]}"; }
 %fstypes = ("ext2","Linux Native",
 	    "minix","Minix",
 	    "msdos","MS-DOS",
-	    "vfat","Windows 95");
+	    "vfat","Windows 95",
+	    "reiserfs","ReiserFS",
+	    "ext3","New Linux Native",
+	    "xfs","SGI");
 
-@space_type = ( '1', '4', '5', 'b', 'c', 'e', '83' );
+@space_type = ( '1', '4', '5', '6', 'b', 'c', 'e', '83' );
 
 # can_edit_disk(device)
 sub can_edit_disk
