@@ -1,7 +1,7 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-require './fdisk-lib.pl';
+do './fdisk-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params)
 # Converts logged information from this module into human-readable form
@@ -29,6 +29,8 @@ return $_[0] =~ /^\/dev\/(s|h)d([a-z])$/ ?
 	&text('select_part', $1 eq 's' ? 'SCSI' : 'IDE', uc($2), $_[1]) :
        $_[0] =~ /rd\/c(\d+)d(\d+)$/ ?
 	&text('select_mpart', "$1", "$2", $_[1]) :
+       $_[0] =~ /ida\/c(\d+)d(\d+)$/ ?
+	&text('select_cpart', "$1", "$2", $_[1]) :
 	$_[0];
 }
 
@@ -38,6 +40,8 @@ return $_[0] =~ /^\/dev\/(s|h)d([a-z])(\d+)$/ ?
 	&text('select_part', $1 eq 's' ? 'SCSI' : 'IDE', uc($2), "$3") :
        $_[0] =~ /rd\/c(\d+)d(\d+)p(\d+)$/ ?
 	&text('select_mpart', "$1", "$2", "$3") :
+       $_[0] =~ /ida\/c(\d+)d(\d+)p(\d+)$/ ?
+	&text('select_cpart', "$1", "$2", "$3") :
 	$_[0];
 }
 
@@ -47,5 +51,7 @@ return $_[0] =~ /^\/dev\/(s|h)d([a-z])$/ ?
 	&text('select_device', $1 eq 's' ? 'SCSI' : 'IDE', uc($2)) :
        $_[0] =~ /rd\/c(\d+)d(\d+)$/ ?
 	&text('select_mylex', "$1", "$2") :
+       $_[0] =~ /ida\/c(\d+)d(\d+)$/ ?
+	&text('select_cpq', "$1", "$2") :
 	$_[0];
 }
