@@ -14,7 +14,7 @@ public class FileManager extends Applet
 	// top buttons
 	CbButton open_b, view_b, edit_b, refresh_b, props_b,
 		 copy_b, cut_b, paste_b, delete_b, new_b, upload_b, mkdir_b,
-		 makelink_b, rename_b, share_b;
+		 makelink_b, rename_b, share_b, search_b, acl_b, attr_b;
 
 	// Directory tree
 	Hierarchy dirs;
@@ -39,6 +39,10 @@ public class FileManager extends Applet
 		  ntab = new Hashtable();
 	boolean sambamode;
 	int nfsmode;
+	String trust;
+
+	boolean got_filesystems;
+	Vector fslist = new Vector();
 
 	public void init()
 	{
@@ -47,12 +51,14 @@ public class FileManager extends Applet
 	accroot = new String[tok.countTokens()];
 	for(int i=0; tok.hasMoreTokens(); i++)
 		accroot[i] = tok.nextToken();
+	trust = getParameter("trust");
 
 	// download language strings
 	String l[] = get_text("lang.cgi");
 	for(int i=0; i<l.length; i++) {
 		int eq = l[i].indexOf('=');
-		lang.put(l[i].substring(0, eq), l[i].substring(eq+1));
+		if (eq >= 0)
+			lang.put(l[i].substring(0, eq), l[i].substring(eq+1));
 		}
 
 	// list samba file shares
@@ -83,6 +89,20 @@ public class FileManager extends Applet
 			}
 		}
 
+	// list filesystems
+	String f[] = get_text("filesystems.cgi");
+	got_filesystems = f[0].equals("1");
+	boolean acl_support = false,
+		attr_support = false;
+	if (got_filesystems) {
+		for(int i=1; i<f.length; i++) {
+			FileSystem fs = new FileSystem(f[i]);
+			fslist.addElement(fs);
+			if (fs.acls) acl_support = true;
+			if (fs.attrs) attr_support = true;
+			}
+		}
+
 	// create button panel
 	BorderPanel top = new BorderPanel(2);
 	top.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
@@ -94,6 +114,11 @@ public class FileManager extends Applet
 	top1.add(edit_b = make_button("edit.gif", text("top_edit")));
 	top1.add(refresh_b = make_button("refresh.gif", text("top_refresh")));
 	top1.add(props_b = make_button("props.gif", text("top_info")));
+	if (acl_support)
+		top1.add(acl_b = make_button("acl.gif", text("top_eacl")));
+	if (attr_support)
+		top1.add(attr_b = make_button("attr.gif", text("top_attr")));
+	top1.add(search_b = make_button("search.gif", text("top_search")));
 	top.add(top1);
 	top.add(new Label(""));
 	
@@ -107,7 +132,7 @@ public class FileManager extends Applet
 		top2.add(makelink_b = make_button("makelink.gif",
 					text("top_new")));
 	top2.add(rename_b = make_button("rename.gif", text("top_rename")));
-	if (sambamode || nfsmode != 0)
+	if ((sambamode || nfsmode != 0) && getParameter("sharing").equals("1"))
 		top2.add(share_b = make_button("share.gif", text("top_share")));
 	top.add(top2);
 	top.add(new Label(""));
@@ -120,17 +145,12 @@ public class FileManager extends Applet
 	top.add(top3);
 	add("North", top);
 
-	Panel mid = new Panel();
-	add("Center", mid);
-	mid.setLayout(new GridLayout(1, 2));
-
 	// create directory tree
 	BorderPanel left = new BorderPanel(2);
 	left.setLayout(new BorderLayout());
 	root = new FileNode(new RemoteFile(this, get_text("root.cgi")[0],null));
 	left.add("Center", dirs = new Hierarchy(root, this));
 	root.open = true; root.fill();
-	mid.add(left);
 
 	// create file window
 	BorderPanel right = new BorderPanel(2);
@@ -145,16 +165,24 @@ public class FileManager extends Applet
 	files.setWidths(widths);
 	files.setDrawLines(false);
 	show_files(root.file);
-	mid.add(right);
+
+	ResizePanel mid = new ResizePanel(left, right, .3);
+	add("Center", mid);
 
 	// Go to the restricted directory
-	if (!accroot[0].equals("/"))
+	String home = getParameter("home");
+	if (home != null)
+		find_directory(home, true);
+	else if (!accroot[0].equals("/"))
 		find_directory(accroot[0], true);
 	}
 
 	CbButton make_button(String f, String t)
 	{
-	return new CbButton(get_image(f), t, CbButton.ABOVE, this);
+	if (size().width < 700 && size().width > 1)
+		return new CbButton(get_image(f), this);
+	else
+		return new CbButton(get_image(f), t, CbButton.ABOVE, this);
 	}
 
 	// Gets an image from the images directory
@@ -169,6 +197,7 @@ public class FileManager extends Applet
 		long now = System.currentTimeMillis();
 		if (url.indexOf('?') > 0) url += "&rand="+now;
 		else url += "?rand="+now;
+		url += "&trust="+trust;
 		URL u = new URL(getDocumentBase(), url);
 		Vector lv = new Vector();
 		LineInputStream is = new LineInputStream(u.openStream());
@@ -182,14 +211,17 @@ public class FileManager extends Applet
 		}
 	catch(Exception e) {
 		e.printStackTrace();
-		return null;
+		//return null;
+		String err[] = { e.getMessage() };
+		return err;
 		}
 	}
 
 	// Fill the multicolumn list with files from some directory
-	void show_files(RemoteFile f)
+	boolean show_files(RemoteFile f)
 	{
 	RemoteFile fl[] = f.list();
+	if (fl == null) return false;
 	files.clear();
 	Object rows[][] = new Object[fl.length+1][];
 	long now = System.currentTimeMillis();
@@ -201,6 +233,7 @@ public class FileManager extends Applet
 	rows[0][2] = rows[0][3] = rows[0][4] = rows[0][5] = "";
 
 	// Create file rows
+	Date n = new Date(now);
 	for(int i=0; i<fl.length; i++) {
 		Object row[] = rows[i+1] = new Object[6];
 		if (fl[i].shared())
@@ -209,20 +242,24 @@ public class FileManager extends Applet
 			row[0] = get_image(RemoteFile.tmap[fl[i].type]);
 		row[1] = fl[i].name;
 		if (fl[i].size < 1000)
-			row[2] = spad(fl[i].size, 5)+" b";
+			row[2] = spad(fl[i].size, 5)+" B";
 		else if (fl[i].size < 1000000)
-			row[2] = spad(fl[i].size/1000, 5)+" Kb";
+			row[2] = spad(fl[i].size/1000, 5)+" kB";
 		else
-			row[2] = spad(fl[i].size/1000000, 5)+" Mb";
+			row[2] = spad(fl[i].size/1000000, 5)+" MB";
 		row[3] = fl[i].user;
 		row[4] = fl[i].group;
 		Date d = new Date(fl[i].modified);
-		if (now - fl[i].modified < 24*60*60*1000) {
+		//if (now - fl[i].modified < 24*60*60*1000) {
+		if (n.getDate() == d.getDate() &&
+		    n.getMonth() == d.getMonth() &&
+		    n.getYear() == d.getYear()) {
 			// show as hour:min
 			row[5] = pad(d.getHours(),2)+":"+
 				 pad(d.getMinutes(),2);
 			}
-		else if (now - fl[i].modified < 24*60*60*365*1000) {
+		//else if (now - fl[i].modified < 24*60*60*365*1000) {
+		else if (n.getYear() == d.getYear()) {
 			// show as day/mon
 			row[5] = pad(d.getDate(),2)+"/"+
 				 monmap[d.getMonth()];
@@ -236,6 +273,7 @@ public class FileManager extends Applet
 	files.addItems(rows);
 	showing_files = f;
 	pathname.setText(f.path);
+	return true;
 	}
 
 	String pad(int n, int s)
@@ -330,6 +368,26 @@ public class FileManager extends Applet
 		if (f == null) return;
 		new PropertiesWindow(f, this);
 		}
+	else if (b == acl_b) {
+		// Display the ACL window (if filesystem supports them)
+		if (f == null) return;
+		FileSystem filefs = find_filesys(f);
+		if (filefs == null) return;
+		if (filefs.acls)
+			new ACLWindow(this, f);
+		else
+			new ErrorWindow(text("eacl_efs", filefs.mount));
+		}
+	else if (b == attr_b) {
+		// Display the attributes window (if filesystem supports them)
+		if (f == null) return;
+		FileSystem filefs = find_filesys(f);
+		if (filefs == null) return;
+		if (filefs.acls)
+			new AttributesWindow(this, f);
+		else
+			new ErrorWindow(text("attr_efs", filefs.mount));
+		}
 	else if (b == copy_b) {
 		// Copy the selected file
 		if (f == null) return;
@@ -410,6 +468,10 @@ public class FileManager extends Applet
 		if (f == null || f.type != RemoteFile.DIR) return;
 		new SharingWindow(f, this);
 		}
+	else if (b == search_b) {
+		// Open window for finding a file
+		new SearchWindow(showing_files.path, this);
+		}
 	}
 
 	// Returns the object for some directory, or null if not found.
@@ -437,6 +499,7 @@ public class FileManager extends Applet
 		String fn = tok.nextToken();
 		if (fn.equals("")) continue;
 		RemoteFile fl[] = pos.list();
+		if (fl == null) return null;
 		if (fill) {
 			posnode.open = true;
 			posnode.fill();
@@ -459,15 +522,32 @@ public class FileManager extends Applet
 			posnode = (FileNode)nodemap.get(pos);
 		}
 	if (fill) {
-		show_files(pos);
-		posnode.fill();
-		posnode.open = true;
-		dirs.select(posnode);
-		dirs.redraw();
+		if (show_files(pos)) {
+			posnode.fill();
+			posnode.open = true;
+			dirs.select(posnode);
+			dirs.redraw();
+			}
 		}
 	return pos;
 	}
 
+	FileSystem find_filesys(RemoteFile f)
+	{
+	FileSystem filefs = null;
+	for(int i=0; i<fslist.size(); i++) {
+		FileSystem fs = (FileSystem)fslist.elementAt(i);
+		int l = fs.mount.length();
+		if (fs.mount.equals(f.path) ||
+		    (f.path.length() >= l+1 &&
+		     f.path.substring(0, l+1).equals(fs.mount+"/")) ||
+		    fs.mount.equals("/")) {
+			filefs = fs;
+			}
+		}
+	return filefs;
+	}
+
 	public boolean action(Event e, Object o)
 	{
 	if (e.target == pathname) {
@@ -507,11 +587,12 @@ public class FileManager extends Applet
 		pn.fill();
 		pn.open = true;
 		FileNode fn = (FileNode)nodemap.get(d);
-		fn.fill();
-		fn.open = true;
-		show_files(d);
-		dirs.select(fn);
-		dirs.redraw();
+		if (show_files(d)) {
+			fn.fill();
+			fn.open = true;
+			dirs.select(fn);
+			dirs.redraw();
+			}
 		}
 	else if (d.type <= 4) {
 		// Direct the browser to this file
@@ -526,13 +607,15 @@ public class FileManager extends Applet
 			URL u = new URL(getDocumentBase(),
 					"show.cgi"+urlize(f.path)+
 					"?rand="+System.currentTimeMillis()+
-					"&type=application%2Foctet%2Dstream");
+					"&type=application%2Foctet%2Dstream"+
+					"&trust="+trust);
 			getAppletContext().showDocument(u);
 			}
 		else {
 			URL u = new URL(getDocumentBase(),
 					"show.cgi"+urlize(f.path)+
-					"?rand="+System.currentTimeMillis());
+					"?rand="+System.currentTimeMillis()+
+					"&trust="+trust);
 			getAppletContext().showDocument(u, "show");
 			}
 		}
@@ -544,8 +627,10 @@ public class FileManager extends Applet
 	StringBuffer rv = new StringBuffer();
 	for(int i=0; i<s.length(); i++) {
 		char c = s.charAt(i);
-		if (c == '%' || c == '+' || c == '&' || c == '?' || c == ' ' ||
-		    c == '#' || c == '<' || c == '>')
+		if (c < 16)
+			rv.append("%0"+Integer.toString(c, 16));
+		else if (!Character.isLetterOrDigit(c) && c != '/' &&
+		    c != '.' && c != '_' && c != '-')
 			rv.append("%"+Integer.toString(c, 16));
 		else
 			rv.append(c);
@@ -553,6 +638,22 @@ public class FileManager extends Applet
 	return rv.toString();
 	}
 
+	static String un_urlize(String s)
+	{
+	StringBuffer rv = new StringBuffer();
+	for(int i=0; i<s.length(); i++) {
+		char c = s.charAt(i);
+		if (c == '%') {
+			rv.append((char)Integer.parseInt(
+				s.substring(i+1, i+3), 16));
+			i += 2;
+			}
+		else
+			rv.append(c);
+		}
+	return rv.toString();
+	}
+
 	public void upload_notify(String path_str, String info)
 	{
 	int sl = path_str.lastIndexOf('/');
@@ -678,6 +779,7 @@ class FileNode extends HierarchyNode
 	{
 	if (!known) {
 		RemoteFile l[] = file.list();
+		if (l == null) return;
 		ch.removeAllElements();
 		for(int i=0; i<l.length; i++)
 			if (l[i].type == 0)
@@ -753,8 +855,12 @@ class RemoteFile
 	if (list == null) {
 		String l[] = parent.get_text("list.cgi?dir="+
 					     parent.urlize(path));
-		if (l[0].length() > 0)
-			list = new RemoteFile[0];
+		if (l[0].length() > 0) {
+			//list = new RemoteFile[0];
+			// Error reading the remote directory!
+			new ErrorWindow(parent.text("list_edir", path, l[0]));
+			list = null;
+			}
 		else {
 			list = new RemoteFile[l.length-3];
 			for(int i=3; i<l.length; i++)
@@ -767,9 +873,11 @@ class RemoteFile
 	RemoteFile find(String n)
 	{
 	RemoteFile l[] = list();
-	for(int i=0; i<l.length; i++)
-		if (l[i].name.equals(n))
-			return l[i];
+	if (l != null) {
+		for(int i=0; i<l.length; i++)
+			if (l[i].name.equals(n))
+				return l[i];
+		}
 	return null;
 	}
 
@@ -825,13 +933,35 @@ class EditorWindow extends FixedFrame implements CbButtonCallback
 	try {
 		URL u = new URL(filemgr.getDocumentBase(),
 				"show.cgi"+filemgr.urlize(file.path)+
-				"?rand="+System.currentTimeMillis());
+				"?rand="+System.currentTimeMillis()+
+				"&trust="+filemgr.trust+"&edit=1");
 		URLConnection uc = u.openConnection();
-		byte buf[] = new byte[uc.getContentLength()];
+		int len = uc.getContentLength();
 		InputStream is = uc.getInputStream();
-		int got = 0;
-		while(got < buf.length)
-			got += is.read(buf, got, buf.length-got);
+		byte buf[];
+		if (len >= 0) {
+			// Length is known
+			buf = new byte[uc.getContentLength()];
+			int got = 0;
+			while(got < buf.length)
+				got += is.read(buf, got, buf.length-got);
+			}
+		else {
+			// Length is unknown .. read till the end
+			buf = new byte[0];
+			while(true) {
+			    byte data[] = new byte[16384];
+			    int got;
+			    try { got = is.read(data); }
+			    catch(EOFException ex) { break; }
+			    if (got <= 0) break;
+			    byte nbuf[] = new byte[buf.length + got];
+			    System.arraycopy(buf, 0, nbuf, 0, buf.length);
+			    System.arraycopy(data, 0, nbuf, buf.length, got);
+			    buf = nbuf;
+			    }
+			}
+		System.out.println("buf size = "+buf.length);
 		edit.setText(new String(buf, 0));
 		is.close();
 		file.size = buf.length;
@@ -902,13 +1032,15 @@ class EditorWindow extends FixedFrame implements CbButtonCallback
 		try {
 			URL u = new URL(filemgr.getDocumentBase(),
 					"save.cgi"+filemgr.urlize(save_path)+
-					"?rand="+System.currentTimeMillis());
+					"?rand="+System.currentTimeMillis()+
+					"&trust="+filemgr.trust);
 			URLConnection uc = u.openConnection();
 			uc.setDoOutput(true);
 			OutputStream os = uc.getOutputStream();
 			byte buf[] = new byte[s.length()];
 			s.getBytes(0, buf.length, buf, 0);
 			os.write(buf);
+			os.close();
 			LineInputStream is = new LineInputStream(
 							uc.getInputStream());
 			String err = is.gets();
@@ -984,8 +1116,10 @@ class PropertiesWindow extends FixedFrame implements CbButtonCallback
 					filemgr.text("cancel"),
 					CbButton.LEFT, this));
 	add("South", bot);
+
 	Panel mid = new Panel();
 	mid.setLayout(new BorderLayout());
+	TabbedPanel tab = null;
 	add("Center", mid);
 
 	// Create file details section
@@ -1175,6 +1309,7 @@ class PropertiesWindow extends FixedFrame implements CbButtonCallback
 	p.add("West", c);
 	r.add(p);
 	}
+
 }
 
 class PermissionsPanel extends Panel
@@ -1454,11 +1589,12 @@ class RenameWindow extends FixedFrame implements CbButtonCallback
 			return;
 			}
 
-		// Update data structures
+		// Update data structure
 		file.name = newstr;
 		file.path = newpath;
 		file.directory.delete(file);
 		file.directory.add(file);
+		file.list = null;
 		FileNode parnode = (FileNode)filemgr.nodemap.get(file.directory);
 		FileNode filenode = (FileNode)filemgr.nodemap.get(file);
 		if (parnode != null && filenode != null) {
@@ -1468,6 +1604,7 @@ class RenameWindow extends FixedFrame implements CbButtonCallback
 			dispose();
 			filemgr.dirs.redraw();
 			}
+
 		filemgr.show_files(filemgr.showing_files);
 		dispose();
 		}
@@ -2041,3 +2178,780 @@ class SharingWindow extends FixedFrame implements CbButtonCallback
 
 }
 
+class SearchWindow extends FixedFrame
+	implements CbButtonCallback,MultiColumnCallback
+{
+	TabbedPanel tab;
+	MultiColumn list;
+	CbButton search_b, cancel_b;
+	FileManager filemgr;
+	TextField dir, match, user, group;
+	Checkbox uany, usel, gany, gsel;
+	Choice type;
+	Checkbox sany, smore, sless;
+	TextField more, less;
+	Checkbox xon, xoff;
+	String types[] = { "", "f", "d", "l", "p" };
+	RemoteFile results[];
+
+	SearchWindow(String d, FileManager p)
+	{
+	filemgr = p;
+	setTitle(filemgr.text("search_title"));
+
+	// setup UI
+	setLayout(new BorderLayout());
+	tab = new TabbedPanel();
+	Panel search = new Panel();
+	search.setLayout(new BorderLayout());
+	tab.addItem(filemgr.text("search_crit"), search);
+	Panel l = new Panel(), r = new Panel();
+	l.setLayout(new GridLayout(0, 1));
+	r.setLayout(new GridLayout(0, 1));
+
+	String cols[] = { "", filemgr.text("right_name"),
+			  filemgr.text("right_size") };
+	float widths[] = { .07f, .78f, .15f };
+	list = new MultiColumn(cols, this);
+	list.setWidths(widths);
+	list.setDrawLines(false);
+	tab.addItem(filemgr.text("search_list"), list);
+
+	add_item(filemgr.text("search_dir"), dir = new TextField(d, 30), l, r);
+
+	add_item(filemgr.text("search_match"), match = new TextField(20), l, r);
+
+	Panel up = new Panel();
+	up.setLayout(new FlowLayout(FlowLayout.LEFT, 1, 1));
+	CheckboxGroup ug = new CheckboxGroup();
+	up.add(uany = new Checkbox(filemgr.text("search_any"), ug, true));
+	up.add(usel = new Checkbox("", ug, false));
+	up.add(user = new TextField(10));
+	add_item(filemgr.text("search_user"), up, l, r);
+
+	Panel gp = new Panel();
+	gp.setLayout(new FlowLayout(FlowLayout.LEFT, 1, 1));
+	CheckboxGroup gg = new CheckboxGroup();
+	gp.add(gany = new Checkbox(filemgr.text("search_any"), gg, true));
+	gp.add(gsel = new Checkbox("", gg, false));
+	gp.add(group = new TextField(10));
+	add_item(filemgr.text("search_group"), gp, l, r);
+
+	type = new Choice();
+	for(int i=0; i<types.length; i++)
+		type.addItem(filemgr.text("search_types_"+types[i]));
+	add_item(filemgr.text("search_type"), type, l, r);
+
+	CheckboxGroup sg = new CheckboxGroup();
+	add_item(filemgr.text("search_size"),
+		 sany = new Checkbox(filemgr.text("search_any"), sg, true),
+		 l, r);
+	Panel mp = new Panel();
+	mp.setLayout(new FlowLayout(FlowLayout.LEFT, 1, 1));
+	mp.add(smore = new Checkbox(filemgr.text("search_more"), sg, false));
+	mp.add(more = new TextField(10));
+	add_item("", mp, l, r);
+	Panel lp = new Panel();
+	lp.setLayout(new FlowLayout(FlowLayout.LEFT, 1, 1));
+	lp.add(sless = new Checkbox(filemgr.text("search_less"), sg, false));
+	lp.add(less = new TextField(10));
+	add_item("", lp, l, r);
+
+	CheckboxGroup xg = new CheckboxGroup();
+	Panel xp = new Panel();
+	xp.setLayout(new FlowLayout(FlowLayout.LEFT, 1, 1));
+	xp.add(xoff = new Checkbox(filemgr.text("yes"), xg, true));
+	xp.add(xon = new Checkbox(filemgr.text("no"), xg, false));
+	add_item(filemgr.text("search_xdev"), xp, l, r);
+
+	search.add("West", l); search.add("East", r);
+	add("Center", tab);
+
+	// Create search and cancel buttons
+	Panel bot = new Panel();
+	bot.setLayout(new FlowLayout(FlowLayout.RIGHT));
+	bot.add(search_b = new CbButton(filemgr.get_image("save.gif"),
+				      filemgr.text("search_ok"),
+				      CbButton.LEFT, this));
+	bot.add(cancel_b = new CbButton(filemgr.get_image("cancel.gif"),
+					filemgr.text("cancel"),
+					CbButton.LEFT, this));
+	add("South", bot);
+	pack();
+	show();
+	}
+
+	void add_item(String t, Component c, Panel l, Panel r)
+	{
+	l.add(new Label(t));
+	Panel p = new Panel();
+	p.setLayout(new BorderLayout());
+	p.add("West", c);
+	r.add(p);
+	}
+
+	public void click(CbButton b)
+	{
+	if (b == cancel_b)
+		dispose();
+	else if (b == search_b) {
+		// validate inputs and build search URL
+		String url = "search.cgi";
+		String d = dir.getText().trim();
+		if (d.length() == 0 || d.charAt(0) != '/') {
+			new ErrorWindow(filemgr.text("search_edir"));
+			return;
+			}
+		url += "?dir="+filemgr.urlize(d);
+		String mt = match.getText().trim();
+		if (mt.length() == 0) {
+			new ErrorWindow(filemgr.text("search_ematch"));
+			return;
+			}
+		url += "&match="+filemgr.urlize(mt);
+		if (type.getSelectedIndex() > 0)
+			url += "&type="+types[type.getSelectedIndex()];
+		if (usel.getState()) {
+			String u = user.getText().trim();
+			if (u.length() == 0) {
+				new ErrorWindow(filemgr.text("search_euser"));
+				return;
+				}
+			url += "&user="+filemgr.urlize(u);
+			}
+		if (gsel.getState()) {
+			String g = group.getText().trim();
+			if (g.length() == 0) {
+				new ErrorWindow(filemgr.text("search_egroup"));
+				return;
+				}
+			url += "&group="+filemgr.urlize(g);
+			}
+		if (smore.getState()) {
+			String m = more.getText().trim();
+			try { Integer.parseInt(m); }
+			catch(Exception e) {
+				new ErrorWindow(filemgr.text("search_esize"));
+				return;
+				}
+			url += "&size=%2B"+m+"c";
+			}
+		else if (sless.getState()) {
+			String l = less.getText().trim();
+			try { Integer.parseInt(l); }
+			catch(Exception e) {
+				new ErrorWindow(filemgr.text("search_esize"));
+				return;
+				}
+			url += "&size=%2D"+l+"c";
+			}
+		if (xon.getState())
+			url += "&xdev=1";
+
+		// send off the search
+		setCursor(WAIT_CURSOR);
+		String f[] = filemgr.get_text(url);
+		if (f[0].length() > 0) {
+			new ErrorWindow(f[0]);
+			return;
+			}
+		Object rows[][] = new Object[f.length-1][];
+		results = new RemoteFile[f.length-1];
+		for(int i=1; i<f.length; i++) {
+			RemoteFile r = new RemoteFile(filemgr, f[i], null);
+			results[i-1] = r;
+			Object row[] = rows[i-1] = new Object[3];
+			row[0] = filemgr.get_image(RemoteFile.tmap[r.type]);
+			row[1] = r.path;
+			if (r.size < 1000)
+				row[2] = filemgr.spad(r.size, 5)+" B";
+			else if (r.size < 1000000)
+				row[2] = filemgr.spad(r.size/1000, 5)+" kB";
+			else
+				row[2] = filemgr.spad(r.size/1000000, 5)+" MB";
+			}
+		list.clear();
+		list.addItems(rows);
+		tab.select(filemgr.text("search_list"));
+		setCursor(DEFAULT_CURSOR);
+		}
+	}
+
+	public void singleClick(MultiColumn list, int num)
+	{
+	}
+
+	// go to the directory of the double-clicked file
+	public void doubleClick(MultiColumn list, int num)
+	{
+	RemoteFile f = results[num];
+	int sl = f.path.lastIndexOf('/');
+	String dir = sl == 0 ? "/" : f.path.substring(0, sl);
+	filemgr.find_directory(dir, true);
+	RemoteFile l[] = filemgr.showing_files.list();
+	for(int i=0; i<l.length; i++) {
+		if (l[i].name.equals(f.name)) {
+			// select the file in the list
+			filemgr.files.select(i+1);
+			filemgr.files.scrollto(i+1);
+			break;
+			}
+		}
+	dispose();
+	}
+}
+
+class FileSystem
+{
+	String mount;
+	String dev;
+	String type;
+	String opts[];
+	boolean acls;
+	boolean attrs;
+
+	FileSystem(String l)
+	{
+	StringSplitter tok = new StringSplitter(l, ' ');
+	mount = tok.nextToken();
+	dev = tok.nextToken();
+	type = tok.nextToken();
+	String optstr = tok.nextToken();
+	acls = tok.nextToken().equals("1");
+	attrs = tok.nextToken().equals("1");
+	StringTokenizer tok2 = new StringTokenizer(optstr, ",");
+	opts = new String[tok2.countTokens()];
+	for(int i=0; i<opts.length; i++)
+		opts[i] = tok2.nextToken();
+	}
+}
+
+class ACLEntry
+{
+	FileManager filemgr;
+	RemoteFile file;
+	boolean def;
+	String type;
+	String owner;
+	boolean read, write, exec;
+
+	ACLEntry(String l, ACLWindow w)
+	{
+	filemgr = w.filemgr;
+	file = w.file;
+	StringSplitter tok = new StringSplitter(l, ':');
+	type = tok.nextToken();
+	if (type.equals("default")) {
+		def = true;
+		type = tok.nextToken();
+		}
+	if (!type.equals("mask") && !type.equals("other")) {
+		owner = tok.nextToken();
+		if (owner.length() == 0)
+			owner = null;
+		}
+	String rwx = tok.nextToken();
+	if (rwx.length() == 0)
+		rwx = tok.nextToken();	// getfacl outputs a blank owner for mask
+					// and other on some systems
+	read = (rwx.charAt(0) == 'r');
+	write = (rwx.charAt(1) == 'w');
+	exec = (rwx.charAt(2) == 'x');
+	}
+
+	ACLEntry(ACLWindow w)
+	{
+	filemgr = w.filemgr;
+	file = w.file;
+	}
+
+	String[] getRow()
+	{
+	String rv[] = new String[3];
+	rv[0] = type;
+	if (def) rv[0] = "default "+rv[0];
+	if (type.equals("mask") || type.equals("other")) rv[1] = "";
+	else if (owner != null) rv[1] = owner;
+	else if (type.equals("user")) rv[1] = filemgr.text("eacl_user", file.user);
+	else rv[1] = filemgr.text("eacl_group", file.group);
+	rv[2] = "";
+	if (read) rv[2] += filemgr.text("info_read")+" ";
+	if (write) rv[2] += filemgr.text("info_write")+" ";
+	if (exec) rv[2] += filemgr.text("info_exec")+" ";
+	return rv;
+	}
+
+	public String toString()
+	{
+	String rv = def ? "default:" : "";
+	rv += type+":";
+	if (!type.equals("mask") && !type.equals("other"))
+		rv += (owner == null ? "" : owner)+":";
+	rv += (read ? 'r' : '-');
+	rv += (write ? 'w' : '-');
+	rv += (exec ? 'x' : '-');
+	return rv;
+	}
+}
+
+class ACLEditor extends FixedFrame implements CbButtonCallback
+{
+	FileManager filemgr;
+	ACLWindow aclwin;
+	ACLEntry acl;
+	boolean creating;
+	CbButton ok, del;
+	Checkbox read, write, exec;
+	TextField owner;
+
+	// Editing an existing ACL entry
+	ACLEditor(ACLWindow w, ACLEntry a)
+	{
+	aclwin = w;
+	filemgr = aclwin.filemgr;
+	acl = a;
+	creating = false;
+	makeUI();
+	}
+
+	// Creating a new ACL entry
+	ACLEditor(ACLWindow w, String type, boolean def)
+	{
+	aclwin = w;
+	filemgr = aclwin.filemgr;
+	acl = new ACLEntry(aclwin);
+	acl.def = def;
+	acl.type = type;
+	creating = true;
+	makeUI();
+	}
+
+	void makeUI()
+	{
+	setTitle(filemgr.text(creating ? "eacl_create" : "eacl_edit"));
+	setLayout(new BorderLayout());
+	Panel left = new Panel();
+	left.setLayout(new GridLayout(0, 1));
+	add("West", left);
+	Panel right = new Panel();
+	right.setLayout(new GridLayout(0, 1));
+	add("East", right);
+
+	left.add(new Label(filemgr.text("eacl_acltype")));
+	TextField type;
+	right.add(type = new TextField((acl.def ? "default " : "")+acl.type, 20));
+	type.setEditable(false);
+
+	if (!acl.type.equals("mask") && !acl.type.equals("other")) {
+		left.add(new Label(filemgr.text("eacl_aclname")));
+		if (creating || acl.owner != null) {
+			// Only allow the adding or editing of ACLs on a user
+			owner = new TextField(acl.owner == null ? "" : acl.owner,
+					      20);
+			right.add(owner);
+			}
+		else {
+			// Just day that this ACL is on the owner
+			String str;
+			if (acl.type.equals("user"))
+				str = filemgr.text("eacl_user", aclwin.file.user);
+			else
+				str = filemgr.text("eacl_group", aclwin.file.group);
+			TextField o = new TextField(str);
+			o.setEditable(false);
+			right.add(o);
+			}
+		}
+
+	left.add(new Label(filemgr.text("eacl_aclperms")));
+	Panel pp = new Panel();
+	pp.setLayout(new FlowLayout(FlowLayout.RIGHT));
+	pp.add(read = new Checkbox(filemgr.text("info_read"), null, acl.read));
+	pp.add(write = new Checkbox(filemgr.text("info_write"), null, acl.write));
+	pp.add(exec = new Checkbox(filemgr.text("info_exec"), null, acl.exec));
+	right.add(pp);
+
+	Panel bot = new Panel();
+	bot.setLayout(new FlowLayout(FlowLayout.RIGHT));
+	bot.add(ok = new CbButton(filemgr.get_image("save.gif"),
+				  filemgr.text("save"),
+				  CbButton.LEFT, this));
+	if (!creating && (acl.owner != null || acl.def))
+		bot.add(del = new CbButton(filemgr.get_image("cancel.gif"),
+					   filemgr.text("delete"),
+					   CbButton.LEFT, this));
+	add("South", bot);
+
+	pack();
+	show();
+	}
+
+	public void click(CbButton b)
+	{
+	if (b == ok) {
+		// Update or add the ACL entry
+		if (owner != null) {
+			String o = owner.getText().trim();
+			if (o.length() == 0) {
+				new ErrorWindow(filemgr.text("eacl_eowner"));
+				return;
+				}
+			acl.owner = owner.getText();
+			}
+		acl.read = read.getState();
+		acl.write = write.getState();
+		acl.exec = exec.getState();
+		if (creating) {
+			// Add to the ACL table
+			aclwin.acllist.addElement(acl);
+			aclwin.acltable.addItem(acl.getRow());
+			}
+		else {
+			// Update the table
+			int idx = aclwin.acllist.indexOf(acl);
+			aclwin.acltable.modifyItem(acl.getRow(), idx);
+			}
+		dispose();
+		}
+	else if (b == del) {
+		// Remove this entry
+		int idx = aclwin.acllist.indexOf(acl);
+		aclwin.acllist.removeElementAt(idx);
+		aclwin.acltable.deleteItem(idx);
+		dispose();
+		}
+	}
+}
+
+class ACLWindow extends FixedFrame implements CbButtonCallback,MultiColumnCallback
+{
+	FileManager filemgr;
+	RemoteFile file;
+	Vector acllist = new Vector();
+
+	CbButton ok, cancel, add;
+	Choice addtype;
+	MultiColumn acltable;
+
+	ACLWindow(FileManager p, RemoteFile f)
+	{
+	super(400, 300);
+	setTitle(p.text("eacl_title", f.path));
+	filemgr = p;
+	file = f;
+
+	// Get the ACLs
+	String a[] = filemgr.get_text(
+			"getfacl.cgi?file="+filemgr.urlize(file.path));
+	if (a[0].length() != 0) {
+		new ErrorWindow(filemgr.text("eacl_eacls", a[0]));
+		return;
+		}
+
+	// Create the UI
+	setLayout(new BorderLayout());
+	String titles[] = { filemgr.text("eacl_acltype"),
+			    filemgr.text("eacl_aclname"),
+			    filemgr.text("eacl_aclperms") };
+	acltable = new MultiColumn(titles, this);
+	for(int i=1; i<a.length; i++) {
+		ACLEntry acl = new ACLEntry(a[i], this);
+		acllist.addElement(acl);
+		acltable.addItem(acl.getRow());
+		}
+	add("Center", acltable);
+	Panel abot = new Panel();
+	abot.setLayout(new FlowLayout(FlowLayout.RIGHT));
+	abot.add(add = new CbButton(filemgr.get_image("add.gif"),
+				   filemgr.text("eacl_add"),
+				   CbButton.LEFT, this));
+	String acltypes[] = { "user", "group",
+			      "default user", "default group", "default other",
+			      "mask" };
+	abot.add(addtype = new Choice());
+	for(int i=0; i<acltypes.length; i++)
+		addtype.addItem(acltypes[i]);
+	abot.add(new Label(" "));
+	abot.add(ok = new CbButton(filemgr.get_image("save.gif"),
+				   filemgr.text("save"),
+				   CbButton.LEFT, this));
+	abot.add(cancel = new CbButton(filemgr.get_image("cancel.gif"),
+				       filemgr.text("cancel"),
+				       CbButton.LEFT, this));
+	add("South", abot);
+
+	pack();
+	show();
+	}
+
+	public void click(CbButton b)
+	{
+	if (b == ok) {
+		// Save the ACLs
+		String aclstr = "";
+		for(int i=0; i<acllist.size(); i++)
+			aclstr += (ACLEntry)acllist.elementAt(i)+"\n";
+		System.out.println(aclstr);
+		String rv[] = filemgr.get_text("setfacl.cgi?file="+
+						filemgr.urlize(file.path)+
+						"&acl="+filemgr.urlize(aclstr));
+		if (rv[0].length() > 0)
+			new ErrorWindow(filemgr.text("eacl_efailed",
+				file.path, rv[0]));
+		else
+			dispose();
+		}
+	else if (b == add) {
+		// Open a window for a new ACL entry
+		String t = addtype.getSelectedItem();
+		String d = "default ";
+		boolean def = t.startsWith(d);
+		if (def)
+			t = t.substring(d.length());
+		if (t.equals("mask")) {
+			// Only allow one mask
+			for(int i=0; i<acllist.size(); i++)
+				if (((ACLEntry)acllist.elementAt(i)).
+				    type.equals(t)) {
+					new ErrorWindow(filemgr.text("eacl_emask"));
+					return;
+					}
+			}
+		new ACLEditor(this, t, def);
+		}
+	else if (b == cancel) {
+		// Don't save
+		dispose();
+		}
+	}
+
+	// Bring up an editor for an ACL
+        public void doubleClick(MultiColumn list, int num)
+	{
+	int idx = list.selected();
+	if (idx >= 0) {
+		ACLEntry e = (ACLEntry)acllist.elementAt(idx);
+		new ACLEditor(this, e);
+		}
+	}
+
+        public void singleClick(MultiColumn list, int num)
+	{
+	}
+
+}
+
+class AttributesWindow extends FixedFrame
+	implements CbButtonCallback,MultiColumnCallback
+{
+	FileManager filemgr;
+	RemoteFile file;
+	Vector attrlist = new Vector();
+
+	CbButton ok, cancel, add;
+	MultiColumn attrtable;
+
+	AttributesWindow(FileManager p, RemoteFile f)
+	{
+	super(400, 300);
+	setTitle(p.text("attr_title", f.path));
+	filemgr = p;
+	file = f;
+
+	// Get the attributes
+	String a[] = filemgr.get_text(
+			"getattrs.cgi?file="+filemgr.urlize(file.path));
+	if (a[0].length() != 0) {
+		new ErrorWindow(filemgr.text("attr_eattrs", a[0]));
+		return;
+		}
+
+	// Create the UI
+	setLayout(new BorderLayout());
+	String titles[] = { filemgr.text("attr_name"),
+			    filemgr.text("attr_value") };
+	attrtable = new MultiColumn(titles, this);
+	for(int i=1; i<a.length; i++) {
+		FileAttribute at = new FileAttribute(a[i], filemgr);
+		attrlist.addElement(at);
+		attrtable.addItem(at.getRow());
+		}
+	add("Center", attrtable);
+	Panel abot = new Panel();
+	abot.setLayout(new FlowLayout(FlowLayout.RIGHT));
+	abot.add(add = new CbButton(filemgr.get_image("add.gif"),
+				   filemgr.text("attr_add"),
+				   CbButton.LEFT, this));
+	abot.add(new Label(" "));
+	abot.add(ok = new CbButton(filemgr.get_image("save.gif"),
+				   filemgr.text("save"),
+				   CbButton.LEFT, this));
+	abot.add(cancel = new CbButton(filemgr.get_image("cancel.gif"),
+				       filemgr.text("cancel"),
+				       CbButton.LEFT, this));
+	add("South", abot);
+
+	pack();
+	show();
+	}
+
+	public void click(CbButton b)
+	{
+	if (b == ok) {
+		// Save the attributes
+		String pstr = "";
+		for(int i=0; i<attrlist.size(); i++) {
+			FileAttribute at = (FileAttribute)attrlist.elementAt(i);
+			pstr += "&name"+i+"="+filemgr.urlize(at.name)+
+			        "&value"+i+"="+filemgr.urlize(at.value);
+			}
+		String rv[] = filemgr.get_text("setattrs.cgi?file="+
+						filemgr.urlize(file.path)+pstr);
+		if (rv[0].length() > 0)
+			new ErrorWindow(filemgr.text("attr_efailed",
+				file.path, rv[0]));
+		else
+			dispose();
+		}
+	else if (b == add) {
+		// Open a window for a new ACL entry
+		new AttributeEditor(this);
+		}
+	else if (b == cancel) {
+		// Don't save
+		dispose();
+		}
+	}
+
+	// Bring up an editor for an ACL
+        public void doubleClick(MultiColumn list, int num)
+	{
+	int idx = list.selected();
+	if (idx >= 0) {
+		FileAttribute at = (FileAttribute)attrlist.elementAt(idx);
+		new AttributeEditor(this, at);
+		}
+	}
+
+        public void singleClick(MultiColumn list, int num)
+	{
+	}
+
+}
+
+class FileAttribute
+{
+	String name;
+	String value;
+
+	FileAttribute(String l, FileManager f)
+	{
+	int eq = l.indexOf('=');
+	name = f.un_urlize(l.substring(0, eq));
+	value = f.un_urlize(l.substring(eq+1));
+	}
+
+	FileAttribute(String n, String v)
+	{
+	name = n;
+	value = v;
+	}
+
+	String[] getRow()
+	{
+	return new String[] { name, value };
+	}
+}
+
+class AttributeEditor extends FixedFrame implements CbButtonCallback
+{
+	FileManager filemgr;
+	AttributesWindow attrwin;
+	FileAttribute attr;
+	boolean creating;
+	CbButton ok, del;
+	TextField name;
+	TextArea value;
+
+	AttributeEditor(AttributesWindow w, FileAttribute a)
+	{
+	attrwin = w;
+	attr = a;
+	filemgr = w.filemgr;
+	creating = false;
+	makeUI();
+	}
+
+	AttributeEditor(AttributesWindow w)
+	{
+	attrwin = w;
+	attr = new FileAttribute("", "");
+	filemgr = w.filemgr;
+	creating = true;
+	makeUI();
+	}
+
+	void makeUI()
+	{
+	setTitle(filemgr.text(creating ? "attr_create" : "attr_edit"));
+	setLayout(new BorderLayout());
+
+	Panel top = new Panel();
+	top.setLayout(new GridLayout(1, 2));
+	top.add(new Label(filemgr.text("attr_name")));
+	top.add(name = new TextField(attr.name, 20));
+	add("North", top);
+
+	Panel mid = new Panel();
+	mid.setLayout(new GridLayout(1, 2));
+	mid.add(new Label(filemgr.text("attr_value")));
+	mid.add(value = new TextArea(attr.value, 5, 20));
+	add("Center", mid);
+
+	Panel bot = new Panel();
+	bot.setLayout(new FlowLayout(FlowLayout.RIGHT));
+	bot.add(ok = new CbButton(filemgr.get_image("save.gif"),
+				  filemgr.text("save"),
+				  CbButton.LEFT, this));
+	if (!creating)
+		bot.add(del = new CbButton(filemgr.get_image("cancel.gif"),
+					   filemgr.text("delete"),
+					   CbButton.LEFT, this));
+	add("South", bot);
+
+	pack();
+	show();
+	}
+
+	public void click(CbButton b)
+	{
+	if (b == ok) {
+		// Update or add the attribute
+		if (name.getText().length() == 0) {
+			new ErrorWindow(filemgr.text("attr_ename"));
+			return;
+			}
+		attr.name = name.getText();
+		attr.value = value.getText();
+		if (creating) {
+			// Add to the attribs table
+			attrwin.attrlist.addElement(attr);
+			attrwin.attrtable.addItem(attr.getRow());
+			}
+		else {
+			// Update the table
+			int idx = attrwin.attrlist.indexOf(attr);
+			attrwin.attrtable.modifyItem(attr.getRow(), idx);
+			}
+		dispose();
+		}
+	else if (b == del) {
+		// Remove this entry
+		int idx = attrwin.attrlist.indexOf(attr);
+		attrwin.attrlist.removeElementAt(idx);
+		attrwin.attrtable.deleteItem(idx);
+		dispose();
+		}
+	}
+}
+
