@@ -77,6 +77,16 @@ public class MultiColumn extends BorderPanel implements CbScrollbarCallback
 	compscroll();
 	}
 
+	// modifyItem
+	// Changes one row of the table
+	void modifyItem(Object item[], int row)
+	{
+	for(int i=0; i<title.length; i++)
+		list[i].setElementAt(item[i], row);
+	repaint();
+	compscroll();
+	}
+
 	// getItem
 	// Returns the contents of a given row
 	Object []getItem(int n)
@@ -102,6 +112,20 @@ public class MultiColumn extends BorderPanel implements CbScrollbarCallback
 	repaint();
 	}
 
+	// scrollto
+	// Scroll to make some row visible
+	void scrollto(int s)
+	{
+	int r = rows();
+	if (s < top || s >= top+r) {
+		top = s-1;
+		if (top > list[0].size() - r)
+			top = list[0].size() - r;
+		sb.setValue(top);
+		repaint();
+		}
+	}
+
 	// deleteItem
 	// Remove one row from the list
 	void deleteItem(int n)
