@@ -2,8 +2,24 @@
 # Common functions for file manager CGIs
 
 do '../web-lib.pl';
+&ReadParse(\%prein, 'GET');
+if ($prein{'trust'}) {
+	dbmopen(%trustdb, "$ENV{'WEBMIN_CONFIG'}/file/trust", 0700);
+	if ($trustdb{$prein{'trust'}}) {
+		$trust_unknown_referers = 1;
+		$trustdb{$prein{'trust'}} = time();
+		}
+	dbmclose(%trustdb);
+	}
 &init_config();
 %access = &get_module_acl();
+if ($access{'home'}) {
+	local @u = getpwnam($remote_user);
+	if (@u) {
+		$access{'root'} .= ' ' if ($access{'root'});
+		$access{'root'} .= $u[7];
+		}
+	}
 
 $icon_map = (	"c", 1,    "txt", 1,
 		"pl", 1,   "cgi", 1,
@@ -20,6 +36,7 @@ local @st = $access{'follow'} ? stat($_[0]) : lstat($_[0]);
 local $ext = $_[0] =~ /\S+\.([^\.\/]+)$/ ? $1 : undef;
 local $user = getpwuid($st[4]); $user = $st[4] if (!$user);
 local $group = getgrgid($st[5]); $group = $st[5] if (!$group);
+local $rl = readlink($_[0]);
 return sprintf ("%s\t%d\t%s\t%s\t%d\t%d\t%d\t%s",
 		$_[0],
 		(-l $_[0] && !$access{'follow'} ? 5 :
@@ -30,16 +47,18 @@ return sprintf ("%s\t%d\t%s\t%s\t%d\t%d\t%d\t%s",
 		 -S $_[0] ? 7 :
 		 defined($icon_map{$ext}) ? $icon_map{$ext} : 4),
 		$user, $group,
-		$st[7], $st[2], $st[9], readlink($_[0]));
+		$st[7], $st[2], $st[9], -l $_[0] && !$rl ? "???" : $rl);
 }
 
 # switch_acl_uid()
 sub switch_acl_uid
 {
 if ($access{'uid'}) {
-	local @u = getpwuid($access{'uid'});
-	$( = $u[3]; $) = "$u[3] $u[3]";
-	$< = $> = $u[2];
+	local @u = $access{'uid'} < 0 ? getpwnam($remote_user)
+				      : getpwuid($access{'uid'});
+	@u || &error($text{'switch_euser'});
+	$( = $u[3]; $) = "$u[3] ".join(" ", $u[3], &other_groups($u[0]));
+	($>, $<) = ($u[2], $u[2]);
 	umask(oct($access{'umask'}));
 	}
 }
