@@ -24,7 +24,8 @@ while(1) {
 	if ($rv == 0) { last; }
 	elsif ($rv == 1) {
 		local $disk = { 'device' => "/dev/dsk/$matches[1]",
-			    	'type' => $matches[2],
+			    	'type' => $matches[2] eq 'DEFAULT' ?
+					  undef : $matches[2],
 			    	'cyl' => $matches[3],
 			    	'alt' => $matches[4],
 			    	'hd' => $matches[5],
@@ -34,6 +35,10 @@ while(1) {
 			$disk->{'desc'} = &text('select_device',
 						"$1", "$2", "$3");
 			}
+		elsif ($matches[1] =~ /c(\d+)d(\d+)$/) {
+			$disk->{'desc'} = &text('select_idedevice',
+					    	chr($1*2 + $2 + 65));
+			}
 		push(@rv, $disk);
 		}
 	}
@@ -77,51 +82,53 @@ while(<VTOC>) {
 				'start' => int($4 / $secs),
 				'end' => int($6 / $secs),
 				'device' => $_[0]."s$n" };
-		if ($_[0] =~ /c(\d+)t(\d+)d(\d+)$/) {
-			$part->{'desc'} = &text('select_part',
-						"$1", "$2", "$3", "$n");
-			}
 		$rv[$n] = $part;
 		}
 	}
 close(VTOC);
-for($i=0; $i<8; $i++) {
+for($i=0; $i<8 || $i<@rv; $i++) {
 	$rv[$i] = { 'tag' => 'unassigned',
 		    'flag' => 'wm',
 		    'device' => $_[0]."s$i" } if (!$rv[$i]);
+	if ($_[0] =~ /c(\d+)t(\d+)d(\d+)$/) {
+		$rv[$i]->{'desc'} = &text('select_part',
+					  "$1", "$2", "$3", $i);
+		}
+	elsif ($_[0] =~ /c(\d+)d(\d+)$/) {
+		$rv[$i]->{'desc'} = &text('select_idepart',
+				    	  chr($1*2 + $2 + 65), $i);
+		}
 	}
 return @rv;
 
-
-&open_format();
-&choose_disk($_[0]);
-if (!&wait_for($fh, 'unformatted', 'formatted')) { return (); }
-&wait_for($fh, 'format>');
-&wprint("partition\n");
-&wait_for($fh, 'partition>');
-&wprint("print\n");
-&wait_for($fh, 'Blocks\r\n');
-while(&wait_for($fh, 'partition>', '\s+\d+\s+(\S+)\s+(\S+)\s+(\d+)(\s+-\s+(\d+))?.*\r\n')) {
-	local $part = { 'tag' => $matches[1],
-			'flag' => $matches[2],
-			'start' => $matches[3],
-			'end' => $matches[5] ? $matches[5] : $matches[3] };
-	if ($matches[1] =~ /c(\d+)t(\d+)d(\d+)s(\d+)$/) {
-		$part->{'desc'} = &text('select_part', "$1", "$2", "$3", "$4");
-		}
-	push(@rv, $part);
-	}
-&wprint("quit\n");
-&wait_for($fh, 'format>');
-return @rv[0..7];
+#&open_format();
+#&choose_disk($_[0]);
+#if (!&wait_for($fh, 'unformatted', 'formatted')) { return (); }
+#&wait_for($fh, 'format>');
+#&wprint("partition\n");
+#&wait_for($fh, 'partition>');
+#&wprint("print\n");
+#&wait_for($fh, 'Blocks\r\n');
+#while(&wait_for($fh, 'partition>', '\s+\d+\s+(\S+)\s+(\S+)\s+(\d+)(\s+-\s+(\d+))?.*\r\n')) {
+#	local $part = { 'tag' => $matches[1],
+#			'flag' => $matches[2],
+#			'start' => $matches[3],
+#			'end' => $matches[5] ? $matches[5] : $matches[3] };
+#	if ($matches[1] =~ /c(\d+)t(\d+)d(\d+)s(\d+)$/) {
+#		$part->{'desc'} = &text('select_part', "$1", "$2", "$3", "$4");
+#		}
+#	push(@rv, $part);
+#	}
+#&wprint("quit\n");
+#&wait_for($fh, 'format>');
+#return @rv[0..7];
 }
 
 # modify_partition(disk, partition, tag, flag, start, end)
 # Changes an existing partition
 sub modify_partition
 {
-`uname -m` =~ /sun/i ||
-	&error("Partitions can only be modified on Sun hardware");
+`uname -m` =~ /sun/i || &error($text{'esun'});
 local(@rv);
 &open_format();
 &choose_disk($_[0]);
@@ -141,11 +148,11 @@ else {
 &wait_for($fh, 'partition>');
 &wprint("label\n");
 if (&wait_for($fh, 'continue', 'Cannot')) {
-	&error("Disk has mounted partitions");
+	&error($text{'emounted'});
 	}
 &wprint("y\n");
 if (&wait_for($fh, 'partition>', 'no backup labels')) {
-	&error("The last partition cannot be removed");
+	&error($text{'elast'});
 	}
 &wprint("quit\n");
 &wait_for($fh, 'format>');
@@ -187,7 +194,8 @@ else {
 # Returns a human-readable filesystem name
 sub fstype_name
 {
-return $_[0] eq "ufs" ? "Solaris Unix Filesystem" : "???";
+return $text{"fstype_$_[0]"} ? $text{"fstype_$_[0]"}
+			     : $text{'fstype_unknown'};
 }
 
 # filesystem_type(device)
@@ -204,16 +212,7 @@ return undef;
 # Translate an error code from fsck
 sub fsck_error
 {
-%errmap = (
-	0, "no errors detected",
-	1, "incorrect parameters",
-	32, "filesystem needs checking",
-	34, "cannot open device",
-	36, "uncorrectable  errors  detected",
-	37, "process killed during checking",
-	39, "uncorrectable  errors  detected!",
-	40, "no errors detected" );
-return $errmap{$_[0]} ? $errmap{$_[0]} : "unknown error";
+return $text{"fsck_$_[0]"} ? $text{"fsck_$_[0]"} : $text{'fsck_unknown'};
 }
 
 
@@ -229,7 +228,7 @@ return if ($format_already_open);
 while(1) {
 	local $rv = &wait_for($fh, 'Specify.*:', 'no disks found', 'space for more');
 	if ($rv == 0) { last; }
-	elsif ($rv == 1) { &error("<tt>format</tt> didn't find any disks"); }
+	elsif ($rv == 1) { &error($text{'eformat'}); }
 	else { &wprint(" "); }
 	}
 &wprint("0\n");
@@ -244,21 +243,21 @@ syswrite($fh, $_[0], length($_[0]));
 
 sub opt_input
 {
-print $_[3] ? "<tr>" : "";
-print "<td align=right><b>$_[0]</b></td>\n";
-print "<td nowrap><input type=radio name=$_[1]_def value=1 checked>Default\n";
-print "&nbsp; <input type=radio name=$_[1]_def value=0>\n";
-print "<input name=$_[1] size=6> $_[2]</td>";
-print $_[3] ? "\n" : "</tr>\n";
+print $_[2] ? "<tr>" : "";
+print "<td align=right><b>$text{$_[0]}</b></td> <td nowrap>\n";
+print "<input type=radio name=$_[0]_def value=1 checked> $text{'default'}\n";
+print "&nbsp; <input type=radio name=$_[0]_def value=0>\n";
+print "<input name=$_[0] size=6> $_[1]</td>";
+print $_[2] ? "\n" : "</tr>\n";
 }
 
 sub opt_check
 {
 if ($in{"$_[0]_def"}) { return ""; }
-elsif ($in{$_[0]} !~ /^$_[2]$/) {
-	&error("'$in{$_[0]}' is not a valid $_[1]");
+elsif ($in{$_[0]} !~ /^$_[1]$/) {
+	&error(&text('opt_error', $in{$_[0]}, $text{$_[0]}));
 	}
-else { return " $_[3] $in{$_[0]}"; }
+else { return " $_[2] $in{$_[0]}"; }
 }
 
 # metamap_init()
@@ -342,3 +341,13 @@ $rv .= "</select>\n";
 return $rv;
 }
 
+# disk_space(device)
+# Returns the amount of total and free space for some filesystem, or an
+# empty array if not appropriate.
+sub disk_space
+{
+local $out = `df -k $_[0] 2>&1`;
+$out =~ /(\/dev\/\S+)\s+(\d+)\s+\S+\s+(\d+)/ || return ();
+return ($2, $3);
+}
+
