@@ -259,9 +259,20 @@ if ($config{"inittab_rl_$rv[0]"}) {
 return @rv;
 }
 
-# init_description(file)
+# init_description(file, \%hasargs)
 sub init_description
 {
+open(FILE, $_[0]);
+local @lines = <FILE>;
+close(FILE);
+if ($_[1]) {
+	foreach (@lines) {
+		if (/^\s*(['"]?)([a-z]+)\1\)/i) {
+			$_[1]->{$2}++;
+			}
+		}
+	}
+
 local $desc;
 if ($config{'daemons_dir'}) {
 	# First try the daemons file
@@ -272,10 +283,9 @@ if ($config{'daemons_dir'}) {
 		return $daemon{'DESCRIPTIVE'};
 		}
 	}
-open(FILE, $_[0]);
 if ($config{'chkconfig'}) {
 	# Find the redhat-style description: section
-	while(<FILE>) {
+	foreach (@lines) {
 		s/\r|\n//g;
 		s/\\$//;
 		if (/^#+\s*description:(.*)/) {
@@ -289,9 +299,18 @@ if ($config{'chkconfig'}) {
 			}
 		}
 	}
+elsif ($config{'init_info'}) {
+	# Find the suse-style Description: line
+	foreach (@lines) {
+		s/\r|\n//g;
+		if (/^#\s*Description:\s*(.*)/) {
+			$desc = $1;
+			}
+		}
+	}
 else {
 	# Use the first comments
-	while(<FILE>) {
+	foreach (@lines) {
 		s/\r|\n//g;
 		next if (/^#!\s*\/(bin|sbin|usr)\// || /\$id/i || /^#+\s+@/ ||
 			 /source function library/i || /^#+\s*copyright/i);
@@ -304,7 +323,6 @@ else {
 	$_[0] =~ /\/([^\/]+)$/;
 	$desc =~ s/^\s*$1\s+//;
 	}
-close(FILE);
 return $desc;
 }
 
