@@ -1,7 +1,7 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-require './init-lib.pl';
+do './init-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params)
 # Converts logged information from this module into human-readable form
@@ -38,6 +38,9 @@ elsif ($action eq 'shutdown') {
 elsif ($action eq 'local') {
 	return $text{'log_local'};
 	}
+elsif ($action eq 'bootup') {
+	return &text('log_bootup', join(" , ", map { "<tt>$_</tt>" } keys %$p));
+	}
 else {
 	return undef;
 	}
