@@ -15,7 +15,8 @@ sub p_link
 sub p_radio
 {
     my ( $name, $checked, @list ) = @_;
-    local $out, $size = @list, $i = 0;
+    local ($out, $size, $i);
+    $size = @list; $i = 0;
 
     do
     {
@@ -53,7 +54,7 @@ sub p_select_wdl
 sub p_select
 {
     my ( $name, $selected, @list ) = @_;
-    local @newlist, $item;
+    local (@newlist, $item);
 
     foreach $item ( @list )
     {
