@@ -1,7 +1,7 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-require './lilo-lib.pl';
+do './lilo-lib.pl';
 &foreign_require("mount", "mount-lib.pl");
 
 # parse_webmin_log(user, script, action, type, object, &params)
@@ -12,7 +12,7 @@ local ($user, $script, $action, $type, $object, $p, $long) = @_;
 local $ll = $long ? "_l" : "";
 if ($type eq 'image') {
 	return &text("log_${action}_image${ll}", "<tt>$object</tt>",
-		     "<tt>$p->{'image'}</tt>");
+		     "<tt>".&html_escape($p->{'image'})."</tt>");
 	}
 elsif ($type eq 'other') {
 	local $other = &foreign_call("mount", "device_name", $p->{'other'})
