@@ -29,7 +29,7 @@ foreach $l (&list_printcap()) {
 		$prn{'dev'} = $l->{'lp'};
 		$prn{'rhost'} = $l->{'rm'};
 		$prn{'rqueue'} = $l->{'rp'};
-		$prn{'msize'} = $l->{'mx#'};
+		$prn{'msize'} = $l->{'mx'};
 		$prn{'comment'} = $l->{'comment'};
 
 		if (!$_[1]) {
@@ -91,7 +91,7 @@ return @jobs;
 
 sub printer_support
 {
-return $_[0] !~ /^(why|allow|default|ctype|sysv|direct)$/;
+return $_[0] !~ /^(why|allow|default|ctype|sysv|direct|ipp)$/;
 }
 
 
@@ -187,7 +187,7 @@ $cap{'sh'} = $prn{'banner'} ? undef : "";
 $cap{'lp'} = $prn{'dev'} ? $prn{'dev'} : undef;
 $cap{'rm'} = $prn{'rhost'} ? $prn{'rhost'} : undef;
 $cap{'rp'} = $prn{'rqueue'} ? $prn{'rqueue'} : undef;
-$cap{'mx#'} = $prn{'msize'} ? $prn{'msize'} : undef;
+$cap{'mx'} = defined($prn{'msize'}) ? $prn{'msize'} : undef;
 $rv = $prn{'comment'}."\n" if ($prn{'comment'});
 $rv .= $prn{'name'};
 foreach $a (@{$prn{'alias'}}) { $rv .= "|$a"; }
@@ -195,7 +195,7 @@ $rv .= "|$prn{'desc'}" if ($prn{'desc'});
 foreach $c (keys %cap) {
 	if ($c =~ /^(\S\S)(#?)$/ && defined($cap{$c})) {
 		if ($cap{$c} eq "") { $rv .= ":$c"; }
-		elsif ($2 eq "#") { $rv .= ":$c$cap{$c}"; }
+		elsif ($cap{$c} =~ /^\d+$/) { $rv .= ":$c#$cap{$c}"; }
 		else { $rv .= ":$c=$cap{$c}"; }
 		}
 	}
@@ -247,8 +247,7 @@ for($i=0; $i<@line; $i++) {
 	$cap{'eline'} = $eline[$i];
 	$cap{'comment'} = $comment[$i];
 	foreach $w (@w[1..$#w]) {
-		if ($w =~ /^([A-z0-9]+)=(.*)$/) { $cap{$1} = $2; }
-		elsif ($w =~ /^([A-z0-9]+)#(.*)$/) { $cap{$1."#"} = $2; }
+		if ($w =~ /^([A-z0-9]+)[=#](.*)$/) { $cap{$1} = $2; }
 		elsif ($w =~ /^([A-z0-9]+)$/) { $cap{$w} = ""; }
 		}
 	push(@rv, \%cap);
@@ -294,7 +293,15 @@ if ($?) { &error("failed to start lpd : <tt>$out</tt>"); }
 # Stop the running lpsched process
 sub stop_sched
 {
-&kill_logged('TERM', $_[0]) || &error("Failed to stop lpd : $!");
+local @pid = &find_byname("lpd");
+&kill_logged('TERM', @pid) || &error("Failed to stop lpd : $!");
+}
+
+# print_command(printer, file)
+# Returns the command to print some file on some printer
+sub print_command
+{
+return "lpr -P$_[0] $_[1]";
 }
 
 if (-r "/dev/lp0") {
@@ -305,8 +312,13 @@ else {
 	@device_files = ("/dev/lp1", "/dev/lp2",
 			 "/dev/ttyS0", "/dev/ttyS1", "/dev/null");
 	}
-push(@device_files, "/dev/usblp0") if (-r "/dev/usblp0");
-@device_names = ("Parallel port 1", "Parallel port 2",
-		 "Serial Port 1 (COM1)", "Serial Port 2 (COM2)",
-		 "Null Device", "USB printer");
+if (-r "/dev/usblp0") {
+	push(@device_files, "/dev/usblp0", "/dev/usblp1",
+			    "/dev/usblp2", "/dev/usblp3");
+	}
+@device_names = (&text('linux_paralel', "1"), &text('linux_paralel', "2"),
+		 &text('linux_serial', "1"), &text('linux_serial', "2"),
+		 $text{'linux_null'}, &text('linux_usb', 1),
+		 &text('linux_usb', 2), &text('linux_usb', 3),
+		 &text('linux_usb', 4));
 
