@@ -4,7 +4,12 @@
 do '../web-lib.pl';
 &init_config();
 do "./$config{'print_style'}-lib.pl";
-do "./$config{'driver_style'}-driver.pl";
+if ($config{'driver_style'}) {
+	do "./$config{'driver_style'}-driver.pl";
+	}
+else {
+	do "./webmin-driver.pl";
+	}
 %access = &get_module_acl();
 
 $drivers_directory = "$module_config_directory/drivers";
@@ -384,7 +389,7 @@ if (&has_ghostscript()) {
 		      "</td> </tr>\n";
 		}
 	}
-else {
+elsif ($config{'gs_path'}) {
 	print "<tr> <td colspan=2>",
 	      &text('webmin_egs', "<tt>$config{'gs_path'}</tt>"),
 	      "</td> </tr>\n";
@@ -439,13 +444,31 @@ foreach $p (split(/\s+/, $access{'printers'})) {
 return 0;
 }
 
-# can_edit_jobs(printer)
+# can_edit_jobs(printer, user)
 sub can_edit_jobs
 {
-return 1 if ($access{'cancel'} == 1);
-return 0 if ($access{'cancel'} == 0);
-foreach $p (split(/\s+/, $access{'jobs'})) {
-	return 1 if ($p eq $_[0]);
+local $rv = 0;
+if ($access{'cancel'} == 1) {
+	$rv = 1;
+	}
+elsif ($access{'cancel'} == 0) {
+	$rv = 0;
+	}
+else {
+	foreach $p (split(/\s+/, $access{'jobs'})) {
+		$rv = 1 if ($p eq $_[0]);
+		}
+	}
+if ($rv) {
+	if ($access{'user'} eq '*') {
+		return 1;
+		}
+	elsif ($access{'user'} eq $_[1]) {
+		return 1;
+		}
+	elsif (!$access{'user'} && $remote_user eq $_[1]) {
+		return 1;
+		}
 	}
 return 0;
 }
