@@ -38,21 +38,15 @@ foreach $l (&list_printcap()) {
 		else {
 			$prn{'dev'} = $l->{'lp'};
 			}
-		$prn{'msize'} = $l->{'mx#'};
+		$prn{'msize'} = $l->{'mx'};
+		$prn{'comment'} = $l->{'comment'};
 
 		if (!$_[1]) {
-			if (!$l->{'rm'}) {
-				# call lpc to get status
-				$out = `lpc -P$prn{'name'} status 2>&1`;
-				if ($out =~ /\n(\S+)\s+(\S+)\s+(\S+)/) {
-					$prn{'accepting'} = $3 eq 'enabled';
-					$prn{'enabled'} = $2 eq 'enabled';
-					}
-				}
-			else {
-				# remote printers are always accepting
-				$prn{'accepting'} = 1;
-				$prn{'enabled'} = 1;
+			# call lpc to get status
+			$out = `lpc -P$prn{'name'} status 2>&1`;
+			if ($out =~ /\n(\S+)\s+(\S+)\s+(\S+)/) {
+				$prn{'accepting'} = $3 eq 'enabled';
+				$prn{'enabled'} = $2 eq 'enabled';
 				}
 			}
 
@@ -108,7 +102,7 @@ return @jobs;
 
 sub printer_support
 {
-return $_[0] !~ /^(why|allow|default|ctype|riface|sysv)$/;
+return $_[0] !~ /^(why|allow|default|ctype|sysv|ipp)$/;
 }
 
 
@@ -117,8 +111,13 @@ return $_[0] !~ /^(why|allow|default|ctype|riface|sysv)$/;
 sub create_printer
 {
 local(%cap);
+if (!-d $config{'spool_dir'}) {
+	&lock_file($config{'spool_dir'});
+	mkdir($config{'spool_dir'}, 0755);
+	&unlock_file($config{'spool_dir'});
+	&system_logged("chown $config{'iface_owner'} $config{'spool_dir'}");
+	}
 $cap{'sd'} = "$config{'spool_dir'}/$_[0]->{'name'}";
-local @st = stat($config{'spool_dir'});
 &lock_file($cap{'sd'});
 mkdir($cap{'sd'}, 0755);
 &unlock_file($cap{'sd'});
@@ -134,7 +133,13 @@ close(TOUCH);
 open(TOUCH, ">$cap{'sd'}/unspooler.$_[0]->{'name'}");
 close(TOUCH);
 &unlock_file("$cap{'sd'}/unspooler.$_[0]->{'name'}");
-&system_logged("chown -R $st[4]:$st[5] $cap{'sd'}");
+if ($config{'iface_owner'}) {
+	&system_logged("chown -R $config{'iface_owner'} $cap{'sd'}");
+	}
+else {
+	local @st = stat($config{'spool_dir'});
+	&system_logged("chown -R $st[4]:$st[5] $cap{'sd'}");
+	}
 &system_logged("chmod 2700 $cap{'sd'}");
 &system_logged("chmod 600 $cap{'sd'}/*");
 &system_logged("chmod 700 $cap{'sd'}/printfilter")
@@ -227,15 +232,16 @@ $cap{'lp'} = $prn{'dhost'} ? "$prn{'dhost'}%$prn{'dport'}" :
 	     $prn{'dev'} ? $prn{'dev'} : undef;
 $cap{'rm'} = $prn{'rhost'} ? $prn{'rhost'} : undef;
 $cap{'rp'} = $prn{'rqueue'} ? $prn{'rqueue'} : undef;
-$cap{'mx#'} = $prn{'msize'} ? $prn{'msize'} : undef;
+$cap{'mx'} = defined($prn{'msize'}) ? $prn{'msize'} : undef;
 $cap{'cm'} = $prn{'desc'} ? $prn{'desc'} : undef;
-$rv = $prn{'name'};
+$rv = $prn{'comment'}."\n" if ($prn{'comment'});
+$rv .= $prn{'name'};
 foreach $a (@{$prn{'alias'}}) { $rv .= "|$a"; }
 $rv .= "|$prn{'desc'}";
 foreach $c (keys %cap) {
 	if ($c =~ /^(\S\S)(#?)$/ && defined($cap{$c})) {
 		if ($cap{$c} eq "") { $rv .= ":$c"; }
-		elsif ($2 eq "#") { $rv .= ":$c$cap{$c}"; }
+		elsif ($cap{$c} =~ /^\d+$/) { $rv .= ":$c#$cap{$c}"; }
 		else { $rv .= ":$c=$cap{$c}"; }
 		}
 	}
@@ -247,36 +253,49 @@ return $rv;
 # Returns an array of associative arrays containing printcap fields
 sub list_printcap
 {
-local(@rv, @line, $line, $cont, $lnum, $i);
+local(@rv, @line, @comment, $line, $cont, $lnum, $i);
 open(CAP, $config{'printcap_file'});
 $lnum = 0;
 while($line = <CAP>) {
-	$line =~ s/^#.*$//g;	# remove comments
 	$line =~ s/\s+$//g;	# remove trailing spaces
-	if ($line =~ /\S/) {
-		$ncont = ($line =~ s/\\$//g);
-		if ($cont || $line =~ /^\s+/) {
-			$line[$#line] .= $line;
-			$eline[@line - 1] = $lnum;
+	if ($line =~ /^(##.*)/) {
+		# special commented line .. keep it
+		$comment[@line] = $line;
+		}
+	else {
+		local $oline = $line;
+		$line =~ s/^#.*$//g;    # remove comments
+		$line =~ s/^\s+//g;     # remove leading spaces
+		if ($line =~ /\S/) {
+			$ncont = ($line =~ s/\\$//g);
+			if ($cont || $oline =~ /^\s+:/) {
+				$line[$#line] .= $line;
+				$eline[@line - 1] = $lnum;
+				}
+			else {
+				push(@line, $line);
+				$eline[@line - 1] = $sline[@line - 1] = $lnum;
+				}
+			$cont = $ncont;
 			}
 		else {
-			push(@line, $line);
-			$eline[@line - 1] = $sline[@line - 1] = $lnum;
+			# only keep comments immediately before an entry
+			$comment[@line] = undef;
 			}
-		$cont = $ncont;
 		}
 	$lnum++;
 	}
 close(CAP);
 for($i=0; $i<@line; $i++) {
 	local(%cap);
+	next if ($line[$i] =~ /^include\s+(\S+)/);
 	@w = split(/:+/, $line[$i]);
 	$cap{'name'} = $w[0];
-	$cap{'line'} = $sline[$i];
+	$cap{'line'} = $sline[$i] - ($comment[$i] ? 1 : 0);
 	$cap{'eline'} = $eline[$i];
+	$cap{'comment'} = $comment[$i];
 	foreach $w (@w[1..$#w]) {
-		if ($w =~ /^([A-z0-9]+)=(.*)$/) { $cap{$1} = $2; }
-		elsif ($w =~ /^([A-z0-9]+)#(.*)$/) { $cap{$1."#"} = $2; }
+		if ($w =~ /^([A-z0-9]+)[=#](.*)$/) { $cap{$1} = $2; }
 		elsif ($w =~ /^([A-z0-9]+)$/) { $cap{$w} = ""; }
 		}
 	push(@rv, \%cap);
@@ -327,6 +346,13 @@ sub stop_sched
 &kill_logged('INT', $_[0]) || &error(&text("estop", $!));
 }
 
+# print_command(printer, file)
+# Returns the command to print some file on some printer
+sub print_command
+{
+return "lpr -P$_[0] $_[1]";
+}
+
 if (-r "/dev/lp0") {
 	@device_files = ("/dev/lp0", "/dev/lp1",
 			 "/dev/ttyS0", "/dev/ttyS1", "/dev/null");
@@ -335,7 +361,14 @@ else {
 	@device_files = ("/dev/lp1", "/dev/lp2",
 			 "/dev/ttyS0", "/dev/ttyS1", "/dev/null");
 	}
-@device_names = ("Parallel port 1", "Parallel port 2",
-		 "Serial Port 1 (COM1)", "Serial Port 2 (COM2)",
-		 "Null Device");
+if (-r "/dev/usblp0") {
+	push(@device_files, "/dev/usblp0", "/dev/usblp1",
+			    "/dev/usblp2", "/dev/usblp3");
+	}
+@device_names = (&text('lprng_paralel', "1"), &text('lprng_paralel', "2"),
+		 &text('lprng_serial', "1"), &text('lprng_serial', "2"),
+		 $text{'lprng_null'}, &text('linux_usb', 1),
+		 &text('linux_usb', 2), &text('linux_usb', 3),
+		 &text('linux_usb', 4));
+
 
