@@ -11,13 +11,17 @@ foreach $l (@lists) {
 	if ($config{"sync_$l"}) {
 		local $dom = $config{"shost_$l"};
 		$dom = &get_system_hostname() if (!$dom);
+		local $list = &get_list($l, $conf);
 		local $pass = &find_value("admin_passwd",
-			   &get_list_config(&get_list($l, $conf)->{'config'}));
+			   &get_list_config($list->{'config'}));
+		&lock_file($list->{'members'});
 		open(WRAPPER, "|$config{'program_dir'}/wrapper majordomo");
-		print WRAPPER "From: root\n\n";
+		print WRAPPER "From: $_[0]->{'user'}\@$dom\n\n";
 		print WRAPPER "approve $pass subscribe $l ",
 			      "$_[0]->{'user'}\@$dom\n\n";
 		close(WRAPPER);
+		sleep(1);
+		&unlock_file($list->{'members'});
 		}
 	}
 }
@@ -30,13 +34,17 @@ foreach $l (@lists) {
 	if ($config{"sync_$l"}) {
 		local $dom = $config{"shost_$l"};
 		$dom = &get_system_hostname() if (!$dom);
+		local $list = &get_list($l, $conf);
 		local $pass = &find_value("admin_passwd",
-			   &get_list_config(&get_list($l, $conf)->{'config'}));
+			   &get_list_config($list->{'config'}));
+		&lock_file($list->{'members'});
 		open(WRAPPER, "|$config{'program_dir'}/wrapper majordomo");
-		print WRAPPER "From: root\n\n";
+		print WRAPPER "From: $_[0]->{'user'}\@$dom\n\n";
 		print WRAPPER "approve $pass unsubscribe $l ",
 			      "$_[0]->{'user'}\@$dom\n\n";
 		close(WRAPPER);
+		sleep(1);
+		&unlock_file($list->{'members'});
 		}
 	}
 }
