@@ -1166,7 +1166,7 @@ $out = `netstat -i 2>&1 | grep $hostname`;
 if ($out =~ /\s+(\S*)\s+(\S*)\s+(.*)/) {
 	$broadcast = "$2.255.255.255";
 	@tmp = split(/\./,$broadcast);
-	$broadcast = "@tmp[0].@tmp[1].@tmp[2].@tmp[3]";
+	$broadcast = "$tmp[0].$tmp[1].$tmp[2].$tmp[3]";
 	return $broadcast;
 	}
 return "255.255.255.255";
