@@ -386,7 +386,7 @@ if ($_[0] eq "ffs") {
 	printf "<input type=radio name=ffs_groupquota value=1 %s> Enabled\n",
 		defined($options{'groupquota'}) && $options{'groupquota'} eq ""
 			? "checked" : "";
-	printf "<input type=radio name=ffs_groupquota value=2 %s>\n";
+	printf "<input type=radio name=ffs_groupquota value=2 %s>\n",
 		$options{'groupquota'} ? "checked" : "";
 	print "Enabled, use file\n";
 	printf "<input name=ffs_groupquota_file size=30 value=\"%s\">\n",
