@@ -11,6 +11,7 @@ $smbmount = &has_command("rumba") ? "rumba" :
 # and 'swap' in the directory field
 sub list_mounts
 {
+return @list_mounts_cache if (@list_mounts_cache);
 local(@rv, @p, $_, $i); $i = 0;
 
 # List normal filesystem mounts
@@ -36,6 +37,7 @@ while(<AUTOTAB>) {
 	}
 close(AUTOTAB);
 
+@list_mounts_cache = @rv;
 return @rv;
 }
 
@@ -65,6 +67,7 @@ else {
 	print FSTAB "$_[1]  $fsck  $dir  $_[2]  $_[4]  $_[5]  $_[3]\n";
 	close(FSTAB);
 	}
+undef(@list_mounts_cache);
 return $len;
 }
 
@@ -101,6 +104,7 @@ foreach (@autotab) {
 	else { print AUTOTAB $_,"\n"; }
 	}
 close(AUTOTAB);
+undef(@list_mounts_cache);
 }
 
 
@@ -145,6 +149,7 @@ foreach (@autotab) {
 	else { print AUTOTAB $_,"\n"; }
 	}
 close(AUTOTAB);
+undef(@list_mounts_cache);
 }
 
 
@@ -155,6 +160,7 @@ close(AUTOTAB);
 # For swap files, the directory will be 'swap'
 sub list_mounted
 {
+return @list_mounted_cache if (@list_mounted_cache);
 local(@rv, @p, $_, $i, $r);
 foreach (split(/\n/, `swap -l`)) {
 	if (/^(\/\S+)\s+/) { push(@rv, [ "swap", $1, "swap", "-" ]); }
@@ -204,6 +210,7 @@ foreach $r (@rv) {
 			}
 		}
 	}
+@list_mounted_cache = @rv;
 return @rv;
 }
 
@@ -261,8 +268,9 @@ else {
 		 (defined($options{'readwrite'}) ? "-S " : "").
 		 (defined($options{'readonly'}) ? "-w " : "").
 		 (defined($options{'attr'}) ? "-e " : "");
-		$rv = &system_logged("$smbmount \"$shar\" $_[0] $opts >/tmp/$$.rumba 2>&1 </dev/null");
-		$out = `cat /tmp/$$.rumba`; unlink("/tmp/$$.rumba");
+		local $rtemp = &tempname();
+		$rv = system("rumba \"$shar\" $_[0] $opts >$rtemp 2>&1 </dev/null");
+		$out = `cat $rtemp`; unlink($rtemp);
 		if ($rv) { return "<pre>$out</pre> : rumba \"$shar\" $_[0] $opts"; }
 		}
 	else {
@@ -271,6 +279,7 @@ else {
 		if ($?) { return $out; }
 		}
 	}
+undef(@list_mounted_cache);
 return 0;
 }
 
@@ -297,6 +306,7 @@ elsif ($_[2] eq "rumba") {
 else {
 	$out = &backquote_logged("umount $_[0] 2>&1");
 	}
+undef(@list_mounted_cache);
 if ($?) { return $out; }
 return 0;
 }
@@ -342,8 +352,8 @@ local(%fsmap);
 	  "lofs","Loopback Filesystem",
 	  "cachefs","Caching Filesystem",
 	  "swap","Virtual Memory",
-	  "tmpfs","Ram Disk",
-	  "xmemfs","Large Ram Disk",
+	  "tmpfs","RAM Disk",
+	  "xmemfs","Large RAM Disk",
 	  "autofs","Automounter Filesystem",
 	  "proc","Process Image Filesystem",
 	  "fd","File Descriptor Filesystem",
@@ -861,9 +871,9 @@ if ($_[0] eq "tmpfs") {
 	($tmpsz = $options{size}) =~ s/[A-z]+$//g;
 	print "<input name=tmpfs_size size=6 value=\"$tmpsz\">\n";
 	print "<select name=tmpfs_unit>\n";
-	printf "<option value=m %s> Mb\n",
+	printf "<option value=m %s> MB\n",
 		$options{"size"} =~ /m$/ ? "selected" : "";
-	printf "<option value=k %s> Kb\n",
+	printf "<option value=k %s> kB\n",
 		$options{"size"} =~ /k$/ ? "selected" : "";
 	printf "<option value=b %s> bytes\n",
 		$options{"size"} !~ /(k|m)$/ ? "selected" : "";
@@ -1044,9 +1054,9 @@ if ($_[0] eq "xmemfs") {
 	($tmpsz = $options{size}) =~ s/[A-z]+$//g;
 	print "<input name=xmemfs_size size=6 value=\"$tmpsz\">\n";
 	print "<select name=xmemfs_unit>\n";
-	printf "<option value=m %s> Mb\n",
+	printf "<option value=m %s> MB\n",
 		$options{"size"} =~ /m$/ ? "selected" : "";
-	printf "<option value=k %s> Kb\n",
+	printf "<option value=k %s> kB\n",
 		$options{"size"} =~ /k$/ ? "selected" : "";
 	printf "<option value=b %s> bytes\n",
 		$options{"size"} !~ /(k|m)$/ ? "selected" : "";
