@@ -218,7 +218,7 @@ return	$_[0] =~ /^tun/ ? "Loopback tunnel" :
 	$_[0] =~ /^lo/ ? "Loopback" :
 	$_[0] =~ /^ar/ ? "Arnet" :
 	$_[0] =~ /^(ax|mx|pn|rl|tx|wb)/ ? "Fast ethernet" :
-	$_[0] =~ /^(cs|de|ed|el|ex|fe|fxp|ie|le|lnc|tl|vr|vx|xl|ze|zp)/ ? "Ethernet" : "Unknown";
+	$_[0] =~ /^(cs|dc|de|ed|el|ex|fe|fxp|ie|le|lnc|tl|vr|vx|xl|ze|zp)/ ? "Ethernet" : $text{'ifcs_unknown'};
 }
 
 # iface_hardware(name)
@@ -325,7 +325,7 @@ for($i=1; $i<@srcs; $i++) {
 		$rv .= sprintf "<option value=\"%s\" %s>%s\n",
 				$srcs[$j],
 				$o[$ii] eq $srcs[$j] ? "selected" : "",
-				$srcn[$j];
+				$srcn[$j] ? $srcn[$j] : "&nbsp;";
 		}
 	$rv .= "</select>\n";
 	}
@@ -438,5 +438,11 @@ foreach $file ("/etc/defaults/rc.conf", "/etc/rc.conf") {
 return %rv;
 }
 
+sub os_feedback_files
+{
+return ( "/etc/rc.conf", "/etc/resolv.conf", "/etc/host.conf",
+	 "/etc/resolv.conf" );
+}
+
 1;
 
