@@ -1,7 +1,7 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-require './net-lib.pl';
+do './net-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params)
 # Converts logged information from this module into human-readable form
@@ -19,6 +19,7 @@ elsif ($action eq 'routes') {
 	}
 elsif ($type eq 'aifc' || $type eq 'bifc') {
 	return &text("log_${action}_${type}", "<tt>$object</tt>",
+		     $p->{'dhcp'} || $p->{'bootp'} ? $text{'log_dyn'} :
 		     "<tt>$p->{'address'}</tt>");
 	}
 else {
