@@ -3,7 +3,7 @@
 
 $net_scripts_dir = "/etc/sysconfig/network-scripts";
 $network_config = "/etc/sysconfig/network";
-$static_route_config = "/etc/sysconfig/static-routes";
+$static_route_config = "/etc/sysconfig/network-scripts/ifcfg-routes";
 $nis_conf = "/etc/nis.conf";
 
 # active_interfaces()
@@ -76,9 +76,10 @@ sub boot_interfaces
 local(@rv, $f);
 opendir(CONF, $net_scripts_dir);
 while($f = readdir(CONF)) {
-	next if ($f !~ /^ifcfg-(\S+)/);
+	next if ($f !~ /^ifcfg-(\S+)/ || $f eq 'ifcfg-routes' ||
+		 $f =~ /\.sample$/);
 	local (%conf, $b);
-	&read_file("$net_scripts_dir/$f", \%conf);
+	&read_env_file("$net_scripts_dir/$f", \%conf);
 	$b->{'fullname'} = $conf{'DEVICE'} ? $conf{'DEVICE'} : $1;
 	if ($b->{'fullname'} =~ /(\S+):(\d+)/) {
 		$b->{'name'} = $1;
@@ -107,7 +108,7 @@ local(%conf);
 local $name = $_[0]->{'virtual'} ne "" ? $_[0]->{'name'}.":".$_[0]->{'virtual'}
 				       : $_[0]->{'name'};
 &lock_file("$net_scripts_dir/ifcfg-$name");
-&read_file("$net_scripts_dir/ifcfg-$name", \%conf);
+&read_env_file("$net_scripts_dir/ifcfg-$name", \%conf);
 $conf{'DEVICE'} = $name;
 if ($_[0]->{'dhcp'}) {
 	$conf{'DYNAMIC'} = 'dhcp';
@@ -126,7 +127,7 @@ $conf{'NETWORK'} = sprintf "%d.%d.%d.%d",
 			($ip4 & int($nm4))&0xff;
 $conf{'BROADCAST'} = $_[0]->{'broadcast'};
 $conf{'ONBOOT'} = $_[0]->{'up'} ? "yes" : "no";
-&write_file("$net_scripts_dir/ifcfg-$name", \%conf);
+&write_env_file("$net_scripts_dir/ifcfg-$name", \%conf);
 &unlock_file("$net_scripts_dir/ifcfg-$name");
 }
 
@@ -153,7 +154,7 @@ return "Arcnet" if ($_[0] =~ /^arc/);
 return "Token Ring" if ($_[0] =~ /^tr/);
 return "Pocket/ATP" if ($_[0] =~ /^atp/);
 return "Loopback" if ($_[0] =~ /^lo/);
-return "Unknown";
+return $text{'ifcs_unknown'};
 }
 
 # iface_hardware(name)
@@ -273,7 +274,7 @@ else {
 			$rv .= sprintf "<option value=\"%s\" %s>%s\n",
 					$srcs[$j],
 					$o[$ii] eq $srcs[$j] ? "selected" : "",
-					$srcn[$j];
+					$srcn[$j] ? $srcn[$j] : "&nbsp;";
 			}
 		$rv .= "</select>\n";
 		}
@@ -370,12 +371,18 @@ printf "<input name=gateway size=15 value=\"%s\"> %s\n",
 printf "<input name=gatewaydev size=6 value=\"%s\"></td> </tr>\n",
 	$gatewaydev;
 
-# show routing
 print "<tr> <td><b>$text{'routes_forward'}</b></td> <td>\n";
 printf "<input type=radio name=forward value=1 %s> $text{'yes'}\n",
 	$conf{'IPFORWARDING'} =~ /yes|true/i ? "checked" : "";
 printf "<input type=radio name=forward value=0 %s> $text{'no'}</td> </tr>\n",
 	$conf{'IPFORWARDING'} =~ /yes|true/i ? "" : "checked";
+
+print "<tr> <td valign=top><b>$text{'routes_script'}</b></td> <td>\n";
+print "<textarea name=script rows=4 cols=60>\n";
+open(SCRIPT, $static_route_config);
+while(<SCRIPT>) { print; }
+close(SCRIPT);
+print "</textarea></td> </tr>\n";
 }
 
 sub parse_routing
@@ -411,6 +418,24 @@ closedir(CONF);
 
 &write_file($network_config, \%conf);
 &unlock_file($network_config);
+
+&lock_file($static_route_config);
+open(SCRIPT, ">$static_route_config");
+$in{'script'} =~ s/\r//g;
+print SCRIPT $in{'script'};
+close(SCRIPT);
+&unlock_file($static_route_config);
+&system_logged("chmod +x $static_route_config");
+}
+
+sub os_feedback_files
+{
+opendir(DIR, $net_scripts_dir);
+local @f = readdir(DIR);
+closedir(DIR);
+return ( (map { "$net_scripts_dir/$_" } grep { /^ifcfg-/ } @f),
+	 $network_config, $static_route_config, $nis_conf, "/etc/resolv.conf",
+	 "/etc/nsswitch.conf", "/etc/HOSTNAME" );
 }
 
 1;
