@@ -27,7 +27,7 @@ foreach $l (@lines) {
 	$ifc{'edit'} = ($ifc{'name'} !~ /ipdptp|ppp/);
 	$ifc{'index'} = scalar(@rv);
 	if ($ifc{'ether'}) {
-		$ifc{'ether'} = join(":", map { sprintf "%2.2d", $_ }
+		$ifc{'ether'} = join(":", map { sprintf "%2.2X", hex($_) }
 					      split(/:/, $ifc{'ether'}));
 		}
 	push(@rv, \%ifc);
@@ -44,6 +44,9 @@ if ($a->{'virtual'} eq "") {
 	local $out = &backquote_logged("ifconfig $a->{'name'} plumb 2>&1");
 	if ($out) { &error(&text('aifc_eexist', $a->{'name'})); }
 	}
+elsif ($gconfig{'os_version'} >= 8) {
+	&system_logged("ifconfig $a->{'name'}:$a->{'virtual'} plumb >/dev/null 2>&1");
+	}
 local $cmd = "ifconfig $a->{'name'}";
 if ($a->{'virtual'} ne "") { $cmd .= ":$a->{'virtual'}"; }
 $cmd .= " $a->{'address'}";
@@ -55,7 +58,7 @@ if ($a->{'mtu'}) { $cmd .= " mtu $a->{'mtu'}"; }
 if ($a->{'up'}) { $cmd .= " up"; }
 else { $cmd .= " down"; }
 local $out = &backquote_logged("$cmd 2>&1");
-if ($?) { &error($out); }
+if ($?) { &error("$cmd : $out"); }
 if ($a->{'ether'}) {
 	$out = &backquote_logged(
 		"ifconfig $a->{'name'} ether $a->{'ether'} 2>&1");
@@ -71,6 +74,9 @@ local $cmd;
 if ($a->{'virtual'} eq "") {
 	$cmd = "ifconfig $a->{'name'} unplumb";
 	}
+elsif ($gconfig{'os_version'} >= 8) {
+	$cmd = "ifconfig $a->{'name'}:$a->{'virtual'} unplumb";
+	}
 else {
 	$cmd = "ifconfig $a->{'name'}:$a->{'virtual'} 0.0.0.0 down";
 	}
@@ -285,8 +291,8 @@ else {
 	# Can select by menus
 	local @o = split(/\s+/, $_[0]->{'order'});
 	local ($rv, $i, $j);
-	local @srcs = ( "", "files", "dns", "nis", "nisplus" );
-	local @srcn = ( "", "Hosts", "DNS", "NIS", "NIS+" );
+	local @srcs = ( "", "files", "dns", "nis", "nisplus", "ldap" );
+	local @srcn = ( "", "Hosts", "DNS", "NIS", "NIS+", "LDAP" );
 	for($i=1; $i<@srcs; $i++) {
 		local $ii = $i-1;
 		$rv .= "<select name=order_$ii>\n";
@@ -294,7 +300,7 @@ else {
 			$rv .= sprintf "<option value=\"%s\" %s>%s\n",
 					$srcs[$j],
 					$o[$ii] eq $srcs[$j] ? "selected" : "",
-					$srcn[$j];
+				        $srcn[$j] ? $srcn[$j] : "&nbsp;";
 			}
 		$rv .= "</select>\n";
 		}
@@ -420,5 +426,14 @@ if (!-r $_[0]) {
 	}
 }
 
+sub os_feedback_files
+{
+opendir(DIR, "/etc");
+local @f = map { "/etc/$_" } grep { /^hostname\./ } readdir(DIR);
+closedir(DIR);
+return ( @f, "/etc/netmasks", "/etc/resolv.conf", "/etc/nsswitch.conf",
+	 "/etc/defaultrouter", "/etc/notrouter", "/etc/gateways" );
+}
+
 1;
 
