@@ -105,7 +105,7 @@ foreach $nc (@nc) {
 			if ($conf =~ /netmask\s+(\S+)/) {
 				$b->{'netmask'} = $1;
 				}
-			if ($conf =~ /\s+up/) {
+			if ($conf =~ /\s+up/ || $gconfig{'os_version'} >= 7.1) {
 				$b->{'up'} = 1;
 				}
 			}
@@ -115,6 +115,12 @@ foreach $nc (@nc) {
 			$b->{'broadcast'} = 'Automatic';
 			$b->{'up'}++;
 			}
+		elsif ($conf =~ /dhcpclient/) {
+			$b->{'dhcp'} = 1;
+			$b->{'netmask'} = 'Automatic';
+			$b->{'broadcast'} = 'Automatic';
+			$b->{'up'}++;
+			}
 		$b->{'edit'} = 1;
 		$b->{'index'} = scalar(@rv);
 		$b->{'nc'} = $nc;
@@ -148,6 +154,9 @@ local $ifconfig;
 if ($_[0]->{'bootp'}) {
 	$ifconfig = 'bootp';
 	}
+elsif ($_[0]->{'dhcp'}) {
+	$ifconfig = 'dhcpclient';
+	}
 else {
 	$ifconfig = $_[0]->{'address'};
 	$ifconfig .= " broadcast $_[0]->{'broadcast'}"
@@ -212,7 +221,7 @@ return "Arcnet" if ($_[0] =~ /^arc/);
 return "Token Ring" if ($_[0] =~ /^tr/);
 return "Pocket/ATP" if ($_[0] =~ /^atp/);
 return "Loopback" if ($_[0] =~ /^lo/);
-return "Unknown";
+return $text{'ifcs_unknown'};
 }
 
 # iface_hardware(name)
@@ -226,11 +235,21 @@ return $_[0] =~ /^eth/;
 # Can some boot-time interface parameter be edited?
 sub can_edit
 {
-if ($_[1] && $_[1]->{'bootp'}) {
-	return $_[0] ne "mtu" && $_[0] ne "dhcp" &&
-	       $_[0] ne "netmask" && $_[0] ne "broadcast" && $_[0] ne "up";
+if ($gconfig{'os_version'} >= 7.1) {
+	if ($_[1] && ($_[1]->{'bootp'} || $_[1]->{'dhcp'})) {
+		return $_[0] ne "mtu" && $_[0] ne "netmask" &&
+		       $_[0] ne "broadcast" && $_[0] ne "up";
+		}
+	return $_[0] ne "mtu" && $_[0] ne "up";
+	}
+else {
+	if ($_[1] && $_[1]->{'bootp'}) {
+		return $_[0] ne "mtu" && $_[0] ne "dhcp" &&
+		       $_[0] ne "netmask" && $_[0] ne "broadcast" &&
+		       $_[0] ne "up";
+		}
+	return $_[0] ne "mtu" && $_[0] ne "dhcp";
 	}
-return $_[0] ne "mtu" && $_[0] ne "dhcp";
 }
 
 # valid_boot_address(address)
@@ -246,10 +265,31 @@ sub get_dns_config
 {
 local $dns;
 local $rc = &parse_rc_config();
-local @ns = split(/\s+/, $rc->{'NAMESERVER'}->{'value'});
-$dns->{'nameserver'} = [ grep { $_ ne "YAST_ASK" } @ns ];
-local $src = $rc->{'SEARCHLIST'};
-$dns->{'domain'} = [ split(/\s+/, $src->{'value'}) ] if ($src);
+if ($rc->{'NAMESERVER'}) {
+	# Use the settings from yast
+	local @ns = split(/\s+/, $rc->{'NAMESERVER'}->{'value'});
+	$dns->{'nameserver'} = [ grep { $_ ne "YAST_ASK" } @ns ];
+	local $src = $rc->{'SEARCHLIST'};
+	$dns->{'domain'} = [ split(/\s+/, $src->{'value'}) ] if ($src);
+	}
+else {
+	# Read the resolv.conf file directly
+	open(RESOLV, "/etc/resolv.conf");
+	while(<RESOLV>) {
+		s/\r|\n//g;
+		s/#.*$//;
+		if (/nameserver\s+(.*)/) {
+			push(@{$dns->{'nameserver'}}, split(/\s+/, $1));
+			}
+		elsif (/domain\s+(\S+)/) {
+			$dns->{'domain'} = [ $1 ];
+			}
+		elsif (/search\s+(.*)/) {
+			$dns->{'domain'} = [ split(/\s+/, $1) ];
+			}
+		}
+	close(RESOLV);
+	}
 open(SWITCH, "/etc/nsswitch.conf");
 while(<SWITCH>) {
 	s/\r|\n//g;
@@ -267,8 +307,34 @@ sub save_dns_config
 {
 &lock_file($rc_config);
 local $rc = &parse_rc_config();
-&save_rc_config($rc, "NAMESERVER", join(" ", @{$_[0]->{'nameserver'}}));
-&save_rc_config($rc, "SEARCHLIST", join(" ", @{$_[0]->{'domain'}}));
+if ($rc->{'NAMESERVER'}) {
+	&save_rc_config($rc, "NAMESERVER", join(" ", @{$_[0]->{'nameserver'}}));
+	&save_rc_config($rc, "SEARCHLIST", join(" ", @{$_[0]->{'domain'}}));
+	}
+else {
+	&lock_file("/etc/resolv.conf");
+	open(RESOLV, "/etc/resolv.conf");
+	local @resolv = <RESOLV>;
+	close(RESOLV);
+	open(RESOLV, ">/etc/resolv.conf");
+	foreach (@{$_[0]->{'nameserver'}}) {
+		print RESOLV "nameserver $_\n";
+		}
+	if ($_[0]->{'domain'}) {
+		if ($_[0]->{'domain'}->[1]) {
+			print RESOLV "search ",
+				     join(" ", @{$_[0]->{'domain'}}),"\n";
+			}
+		else {
+			print RESOLV "domain $_[0]->{'domain'}->[0]\n";
+			}
+		}
+	foreach (@resolv) {
+		print RESOLV $_ if (!/^\s*(nameserver|domain|search)\s+/);
+		}
+	close(RESOLV);
+	&unlock_file("/etc/resolv.conf");
+	}
 
 &lock_file("/etc/nsswitch.conf");
 open(SWITCH, "/etc/nsswitch.conf");
@@ -283,11 +349,13 @@ foreach (@switch) {
 	}
 close(SWITCH);
 &unlock_file("/etc/nsswitch.conf");
-if ($_[0]->{'order'} =~ /nis/) {
-	&save_rc_config($rc, "USE_NIS_FOR_RESOLVING", "yes");
-	}
-else {
-	&save_rc_config($rc, "USE_NIS_FOR_RESOLVING", "no");
+if ($rc->{'USE_NIS_FOR_RESOLVING'}) {
+	if ($_[0]->{'order'} =~ /nis/) {
+		&save_rc_config($rc, "USE_NIS_FOR_RESOLVING", "yes");
+		}
+	else {
+		&save_rc_config($rc, "USE_NIS_FOR_RESOLVING", "no");
+		}
 	}
 &unlock_file($rc_config);
 }
@@ -316,7 +384,7 @@ else {
 			$rv .= sprintf "<option value=\"%s\" %s>%s\n",
 					$srcs[$j],
 					$o[$ii] eq $srcs[$j] ? "selected" : "",
-					$srcn[$j];
+				        $srcn[$j] ? $srcn[$j] : "&nbsp;";
 			}
 		$rv .= "</select>\n";
 		}
@@ -356,20 +424,6 @@ sub save_hostname
 local $rc = &parse_rc_config();
 &save_rc_config($rc, "FQHOSTNAME", $_[0]);
 &unlock_file($rc_config);
-}
-
-# get_domainname()
-sub get_domainname
-{
-local $rc = &parse_rc_config();
-return $rc->{'YP_DOMAINNAME'}->{'value'};
-}
-
-# save_domainname(domain)
-sub save_domainname
-{
-local $rc = &parse_rc_config();
-&save_rc_config($rc, "YP_DOMAINNAME", $_[0]);
 
 # run SuSEconfig, as this function is called last
 system("SuSEconfig -quick >/dev/null 2>&1");
@@ -519,5 +573,10 @@ else {
 	}
 }
 
+sub os_feedback_files
+{
+return ( $rc_config, $route_conf, "/etc/resolv.conf", "/etc/nsswitch.conf" );
+}
+
 1;
 
