@@ -296,7 +296,7 @@ else {
 			$rv .= sprintf "<option value=\"%s\" %s>%s\n",
 					$srcs[$j],
 					$o[$ii] eq $srcs[$j] ? "selected" : "",
-					$srcn[$j];
+					$srcn[$j] ? $srcn[$j] : "&nbsp;";
 			}
 		$rv .= "</select>\n";
 		}
@@ -421,5 +421,14 @@ if (!-r $_[0]) {
 	}
 }
 
+sub os_feedback_files
+{
+opendir(DIR, "/etc");
+local @f = map { "/etc/$_" } grep { /^hostname\./ } readdir(DIR);
+closedir(DIR);
+return ( @f, "/etc/netmasks", "/etc/resolv.conf", "/etc/nsswitch.conf",
+	 "/etc/defaultrouter", "/etc/notrouter", "/etc/gateways" );
+}
+
 1;
 
