@@ -1,7 +1,7 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-require './pap-lib.pl';
+do './pap-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params)
 # Converts logged information from this module into human-readable form
@@ -9,13 +9,13 @@ sub parse_webmin_log
 {
 local ($user, $script, $action, $type, $object, $p) = @_;
 if ($action eq 'modify') {
-	return &text('log_modify', "<tt>$object</tt>");
+	return &text('log_modify', "<tt>".&html_escape($object)."</tt>");
 	}
 elsif ($action eq 'create') {
-	return &text('log_create', "<tt>$object</tt>");
+	return &text('log_create', "<tt>".&html_escape($object)."</tt>");
 	}
 elsif ($action eq 'delete') {
-	return &text('log_delete', "<tt>$object</tt>");
+	return &text('log_delete', "<tt>".&html_escape($object)."</tt>");
 	}
 elsif ($action eq 'sync') {
 	return $text{'log_sync'};
