@@ -9,12 +9,12 @@ open(PS, "ps -axwwww -o pid,ppid,user,vsz,%cpu,time,nice,tty,ruser,rgid,pgid,lst
 for($i=0; $line=<PS>; $i++) {
 	chop($line);
 	if ($line =~ /ps -axwwww/ || $line =~ /^\s*PID/) { $i--; next; }
-	$line =~ /^\s*(\d+)\s+(\d+)\s+(\S+)\s+(\d+)\s+([\d\.]+)\s+(\S+)\s+(-?\d+)\s+(\S+)\s+(\S+)\s+(\d+)\s+(\d+)\s+(-|\S+\s+\S+\s+\d+\s+\S+\s+\d+)\s+(\S+)\s+(.*)$/;
+	$line =~ /^\s*(\d+)\s+(\d+)\s+(\S+)\s+(\d+)\s+([\d\.]+)\s+(\S+)\s+(-?\d+)\s+(\S+)\s+(\S+)\s+(\d+)\s+(\d+)\s+(-|\S+\s+\S+\s+\d+\s+\S+\s+\d+|\d+)\s+(\S+)\s+(.*)$/;
 
 	$plist[$i]->{"pid"} = $1;
 	$plist[$i]->{"ppid"} = $2;
 	$plist[$i]->{"user"} = $3;
-	$plist[$i]->{"size"} = "$4 Kb";
+	$plist[$i]->{"size"} = "$4 kB";
 	$plist[$i]->{"cpu"} = $5;
 	$plist[$i]->{"time"} = $6;
 	$plist[$i]->{"nice"} = $7;
