@@ -13,18 +13,18 @@ open(PS, "ps -fl $pcmd |");
 for($i=0; $line=<PS>; $i++) {
         chop($line);
         if ($line =~ /ps -fl/ || $line =~ /COMD/) { $i--; next; }
-        $line =~ /^\s*(\d+)\s+(\S*)\s+(\S*)\s+(\d+)\s+(\d+)\s+(\d+)\s+([\-\d]+)\s+([\-\d]+)\s+(\S*)\s+(\d+)\s+(\S*)\s+(\S*)\s+(\S*)\s+([0-9:]+)\s+(.*)$/;
+        $line =~ /^\s*(\d+)\s+(\S*)\s+(\S*)\s+(\d+)\s+(\d+)\s+(\d+)\s+([\-\d]+)\s+([\-\d]+)\s+(\S*)\s+(\d+)\s+(\S*)\s+(.*)$/;
         $plist[$i]->{"pid"} = $4;
         $plist[$i]->{"ppid"} = $5;
         $plist[$i]->{"user"} = $3;
         $plist[$i]->{"size"} = "$10 Pg";
         $plist[$i]->{"cpu"} = "0.$6 %";
-        $plist[$i]->{"time"} = substr($line,77,6);
+        $plist[$i]->{"time"} = substr($12,17,6);
         $plist[$i]->{"nice"} = $8;
-        $plist[$i]->{"args"} = substr($line,84,60);
+        $plist[$i]->{"args"} = substr($12,23,60);
         $plist[$i]->{"_pri"} = $7;
-        $tty = substr($line,69,7);
-        $plist[$i]->{"_tty"} =  $tty eq "?      " ? $text{'edit_none'} : "/dev/$tty";
+        $tty = substr($12,8,8);
+        $plist[$i]->{"_tty"} =  $tty eq "?       " ? $text{'edit_none'} : "/dev/$tty";
         $plist[$i]->{"_status"} = $stat_map{$2};
         $plist[$i]->{"_wchan"} = $11;
         }
