@@ -7,12 +7,12 @@ local($pcmd, $line, $i, %pidmap, @plist, $dummy, @w, $_);
 local $out = `ps V`;
 if ($out =~ /version\s+2\./ || $out =~ /version\s+\./) {
 	# New version of ps, as found in redhat 6
-	open(PS, "ps -eo user,ruser,group,rgroup,pid,ppid,pgid,pcpu,vsz,nice,etime,time,tty,args  |");
+	open(PS, "ps -eo user,ruser,group,rgroup,pid,ppid,pgid,pcpu,vsz,nice,etime,time,tty,args 2>/dev/null |");
 	$dummy = <PS>;
 	for($i=0; $line=<PS>; $i++) {
 		chop($line);
 		$line =~ s/^\s+//g;
-		@w = split(/\s+/, $line);
+		@w = split(/\s+/, $line, -1);
 		if ($line =~ /ps -o user,ruser/) {
 			# Skip process ID 0 or ps command
 			$i--; next;
@@ -25,7 +25,7 @@ if ($out =~ /version\s+2\./ || $out =~ /version\s+\./) {
 		$plist[$i]->{"ppid"} = $w[5];
 		$plist[$i]->{"user"} = $w[0];
 		$plist[$i]->{"cpu"} = "$w[7] %";
-		$plist[$i]->{"size"} = "$w[8] Kb";
+		$plist[$i]->{"size"} = "$w[8] kB";
 		$plist[$i]->{"time"} = $w[11];
 		$plist[$i]->{"nice"} = $w[9];
 		$plist[$i]->{"args"} = @w<14 ? "defunct" : join(' ', @w[13..$#w]);
@@ -52,7 +52,7 @@ else {
 		$plist[$i]->{"pid"} = $3;
 		$plist[$i]->{"ppid"} = $4;
 		$plist[$i]->{"user"} = getpwuid($2);
-		$plist[$i]->{"size"} = "$7 Kb";
+		$plist[$i]->{"size"} = "$7 kB";
 		$plist[$i]->{"cpu"} = "Unknown";
 		$plist[$i]->{"time"} = $12;
 		$plist[$i]->{"nice"} = $6;
@@ -114,13 +114,30 @@ return split(/\s+/, $out);
 # Returns the filehandles and names for a pty and tty
 sub get_new_pty
 {
-opendir(DEV, "/dev");
-local @ptys = map { "/dev/$_" } (grep { /^pty/ } readdir(DEV));
-closedir(DEV);
+local @ptys;
+local $devstyle;
+if (-d "/dev/pty") {
+	opendir(DEV, "/dev/pty");
+	@ptys = map { "/dev/pty/$_" } readdir(DEV);
+	closedir(DEV);
+	$devstyle = 1;
+	}
+else {
+	opendir(DEV, "/dev");
+	@ptys = map { "/dev/$_" } (grep { /^pty/ } readdir(DEV));
+	closedir(DEV);
+	$devstyle = 0;
+	}
 local ($pty, $tty);
 foreach $pty (@ptys) {
 	open(PTY, "+>$pty") || next;
-	local $tty = $pty; $tty =~ s/pty/tty/;
+	local $tty = $pty;
+	if ($devstyle == 0) {
+		$tty =~ s/pty/tty/;
+		}
+	else {
+		$tty =~ s/m(\d+)$/s$1/;
+		}
 	open(TTY, "+>$tty") || next;
 	local $old = select(PTY); $| = 1;
 	select(TTY); $| = 1; select($old);
@@ -129,6 +146,32 @@ foreach $pty (@ptys) {
 return ();
 }
 
+# get_memory_info()
+# Returns a list containing the real mem, free real mem, swap and free swap
+sub get_memory_info
+{
+local %m;
+open(MEMINFO, "/proc/meminfo") || return ();
+while(<MEMINFO>) {
+	if (/^(\S+):\s+(\d+)/) {
+		$m{lc($1)} = $2;
+		}
+	}
+close(MEMINFO);
+return ( $m{'memtotal'}, $m{'memfree'}+$m{'buffers'}+$m{'cached'},
+	 $m{'swaptotal'}, $m{'swapfree'} );
+}
+
+# get_cpu_info()
+# Returns a list containing the 5, 10 and 15 minute load averages
+sub get_cpu_info
+{
+open(LOAD, "/proc/loadavg") || return ();
+local @load = split(/\s+/, <LOAD>);
+close(LOAD);
+return ( $load[0], $load[1], $load[2] );
+}
+
 foreach $ia (keys %text) {
 	if ($ia =~ /^linux(_\S+)/) {
 		$info_arg_map{$1} = $text{$ia};
