@@ -1,7 +1,7 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-require './proc-lib.pl';
+do './proc-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params)
 # Converts logged information from this module into human-readable form
@@ -9,7 +9,7 @@ sub parse_webmin_log
 {
 local ($user, $script, $action, $type, $object, $p, $long) = @_;
 if ($action eq 'run') {
-	return &text('log_run', "<tt>$p->{'cmd'}</tt>");
+	return &text('log_run', "<tt>".&html_escape($p->{'cmd'})."</tt>");
 	}
 elsif ($action eq 'kill') {
 	local ($desc, $i);
