@@ -5,6 +5,8 @@ do '../web-lib.pl';
 &init_config();
 do "./$config{ps_style}-lib.pl";
 use POSIX;
+%access = &get_module_acl();
+map { $hide{$_}++ } split(/\s+/, $access{'hide'});
 
 sub process_info
 {
@@ -18,6 +20,7 @@ sub index_links
 local(%linkname, $l);
 print "<b>$text{'index_display'} : </b>&nbsp;&nbsp;\n";
 foreach $l ("tree", "user", "size", "cpu", "search", "run") {
+	next if ($l eq "run" && !$access{'run'});
 	if ($l ne $_[0]) { print "<a href=index_$l.cgi>"; }
 	else { print "<b>"; }
 	print $text{"index_$l"};
@@ -43,15 +46,19 @@ return $_[0];
 # switch_acl_uid()
 sub switch_acl_uid
 {
-local %access = &get_module_acl();
-if ($access{'uid'}) {
+if ($access{'uid'} < 0) {
+	local @u = getpwnam($remote_user);
+	$( = $u[3]; $) = "$u[3] $u[3]";
+	($>, $<) = ($u[2], $u[2]);
+	}
+elsif ($access{'uid'}) {
 	local @u = getpwuid($access{'uid'});
 	$( = $u[3]; $) = "$u[3] $u[3]";
-	$< = $> = $u[2];
+	($>, $<) = ($u[2], $u[2]);
 	}
 }
 
-# safe_process_exec(command, uid, gid, handle, input, fixtags)
+# safe_process_exec(command, uid, gid, handle, input, fixtags, bsmode)
 # Executes the given command as the given user/group and writes all output
 # to the given file handle. Finishes when there is no more output or the
 # process stops running. Returns the number of bytes read.
@@ -64,7 +71,7 @@ pipe(OUTr, OUTw);
 pipe(INr, INw);
 local $pid = fork();
 if (!$pid) {
-	setsid();
+	#setsid();
 	untie(*STDIN);
 	untie(*STDOUT);
 	untie(*STDERR);
@@ -75,12 +82,23 @@ if (!$pid) {
 	close(OUTr); close(INw);
 
 	if ($_[1]) {
-		# switch to correct UID and GID
-		$( = $_[2]; $) = "$_[2] $_[2]";
-		$< = $> = $_[1];
+		if (defined($_[2])) {
+			# switch to given UID and GID
+			$( = $_[2]; $) = "$_[2] $_[2]";
+			($>, $<) = ($_[1], $_[1]);
+			}
+		else {
+			# switch to UID and all GIDs
+			local @u = getpwuid($_[1]);
+			$( = $u[3];
+			$) = "$u[3] ".join(" ", $u[3], &other_groups($u[0]));
+			($>, $<) = ($u[2], $u[2]);
+			}
 		}
 
 	# run the command
+	delete($ENV{'FOREIGN_MODULE_NAME'});
+	delete($ENV{'SCRIPT_NAME'});
 	exec("/bin/sh", "-c", $_[0]);
 	print "Exec failed : $!\n";
 	exit 1;
@@ -95,6 +113,7 @@ close(INw);
 local $fn = fileno(OUTr);
 local $got = 0;
 local $out = $_[3];
+local $line;
 while(1) {
 	local ($rmask, $buf);
 	vec($rmask, $fn, 1) = 1;
@@ -108,7 +127,18 @@ while(1) {
 			$buf =~ s/</&lt;/g;
 			$buf =~ s/>/&gt;/g;
 			}
-		print $out $buf;
+		if ($_[6]) {
+			# Convert backspaces and returns
+			$line .= $buf;
+			while($line =~ s/^([^\n]*\n)//) {
+				local $one = $1;
+				while($one =~ s/.\010//) { }
+				print $out $one;
+				}
+			}
+		else {
+			print $out $buf;
+			}
 		}
 	elsif ($sel == 0) {
 		# nothing to read. maybe the process is done, and a subprocess
@@ -117,8 +147,9 @@ while(1) {
 		}
 	}
 close(OUTr);
-return $got;
+print $out $line;
 $SIG{'CHLD'} = $chld;
+return $got;
 }
 
 # safe_process_exec_logged(..)
@@ -178,5 +209,23 @@ local @rv = grep { $_->{'args'} =~ /$name/ } &list_processes();
 return wantarray ? @rv : $rv[0];
 }
 
+$has_lsof_command = &has_command("lsof");
+
+# find_socket_processes(protocol, port)
+# Returns all processes using some port and protocol
+sub find_socket_processes
+{
+local @rv;
+open(LSOF, "lsof -i '$_[0]:$_[1]' |");
+while(<LSOF>) {
+	if (/^(\S+)\s+(\d+)/) {
+		push(@rv, $2);
+		}
+	}
+close(LSOF);
+return @rv;
+}
+
+
 1;
 
