@@ -21,7 +21,7 @@ for($i=0; $line=<PS>; $i++) {
 	$plist[$i]->{"ppid"} = $w[5];
 	$plist[$i]->{"user"} = $w[0];
 	$plist[$i]->{"cpu"} = "$w[7] %";
-	$plist[$i]->{"size"} = "$w[8] Kb";
+	$plist[$i]->{"size"} = "$w[8] kB";
 	$plist[$i]->{"time"} = $w[11];
 	$plist[$i]->{"nice"} = $w[9] =~ /\d+/ ? $w[9]-20 : $w[9];
 	$plist[$i]->{"args"} = @w<14 ? "defunct" : join(' ', @w[13..$#w]);
