@@ -145,16 +145,20 @@ $n=0; while(<QUOTA>) {
 	if (/^(\S+)$/) {
 		# Bogus wrapped line!
 		$filesys{$n,'filesys'} = $1;
-		<QUOTA> =~ /^.{13}(.{7})(.{7})(.{7}).{12}(.{7})(.{7})(.{7})/;
-		$filesys{$n,'ublocks'} = int($1);
-		$filesys{$n,'sblocks'} = int($2);
-		$filesys{$n,'hblocks'} = int($3);
-		$filesys{$n,'ufiles'} = int($4);
-		$filesys{$n,'sfiles'} = int($5);
-		$filesys{$n,'hfiles'} = int($6);
-		$n++;
+		local $nl = <QUOTA>;
+		if ($nl =~ /^\s+(\d+)\s+(\d+)\s+(\d+)\s.{0,15}\s(\d+)\s+(\d+)\s+(\d+)/ ||
+		    $nl =~ /^.{13}(.{7})(.{7})(.{7}).{12}(.{7})(.{7})(.{7})/) {
+			$filesys{$n,'ublocks'} = int($1);
+			$filesys{$n,'sblocks'} = int($2);
+			$filesys{$n,'hblocks'} = int($3);
+			$filesys{$n,'ufiles'} = int($4);
+			$filesys{$n,'sfiles'} = int($5);
+			$filesys{$n,'hfiles'} = int($6);
+			$n++;
+			}
 		}
-	elsif (/^(.{13})(.{7})(.{7})(.{7}).{12}(.{7})(.{7})(.{7})/) {
+	elsif (/^(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\s.{0,15}\s(\d+)\s+(\d+)\s+(\d+)/ ||
+	       /^(.{13})(.{7})(.{7})(.{7}).{12}(.{7})(.{7})(.{7})/) {
 		$filesys{$n,'filesys'} = $1;
 		$filesys{$n,'ublocks'} = int($2);
 		$filesys{$n,'sblocks'} = int($3);
