@@ -1,13 +1,14 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-require './quota-lib.pl';
+do './quota-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params)
 # Converts logged information from this module into human-readable form
 sub parse_webmin_log
 {
 local ($user, $script, $action, $type, $object, $p) = @_;
+$object = &html_escape($object);
 if ($action eq 'activate') {
 	return &text($p->{'mode'} == 1 ? 'log_activate_u' :
 		     $p->{'mode'} == 2 ? 'log_activate_g' :
