@@ -58,7 +58,7 @@ setpwent();
 while(@uinfo = getpwent()) {
 	$hasu{$uinfo[0]}++;
 	}
-endpwent();
+endpwent() if ($gconfig{'os_type'} ne 'hpux');
 @rep = split(/\n/, $rep); @rep = @rep[3..$#rep];
 for($n=0; $n<@rep; $n++) {
 	if ($rep[$n] =~ /(\S+)\s+[\-\+]{2}\s+(\d+)\s+(\d+)\s+(\d+)\s+.{0,15}\s+(\d+)\s+(\d+)\s+(\d+)/ || $rep[$n] =~ /(\S+)\s+[\-\+]{2}(.{7})(.{7})(.{7}).{13}(.{7})(.{7})(.{7})/) {
