@@ -87,7 +87,7 @@ open(CONF, $config{smb_conf});
 close(CONF);
 open(CONF, "> $config{smb_conf}");
 for($i=0; $i<@conf; $i++) {
-	chop($_ = $conf[$i]); s/;.*$//g;
+	chop($_ = $conf[$i]); s/;.*$//g; s/#.*$//g;
 	if (/^\s*\[([^\]]+)\]/) {
 		$first = 1;
 		if ($replacing) { $replacing = 0; }
@@ -147,7 +147,7 @@ close(CONF);
 sub list_connections
 {
 local($l, $started, @rv);
-foreach $l (split(/\n/ , `$config{samba_status_program}`)) {
+foreach $l (split(/\n/ , `$config{samba_status_program} -S`)) {
 	if ($l =~ /^----/) { $started = 1; }
 	if ($started && $l =~ /^(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)\s+\(\S+\)\s+(.*)$/ && (!$_[0] || $1 eq $_[0] || $1 eq $2 && $_[0] eq "homes")) {
 		push(@rv, [ $1, $2, $3, $4, $5, $6 ]);
@@ -156,6 +156,21 @@ foreach $l (split(/\n/ , `$config{samba_status_program}`)) {
 return @rv;
 }
 
+# list_locks()
+# Returns a list of locked files as an array, in the form:
+#  pid, mode, rw, oplock, file, date
+sub list_locks
+{
+local($l, $started, @rv);
+foreach $l (split(/\n/ , `$config{samba_status_program} -L`)) {
+	if ($l =~ /^----/) { $started = 1; }
+	if ($started && $l =~ /^(\d+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)\s+(\S+\s+\S+\s+\d+\s+\d+:\d+:\d+\s+\d+)/) {
+		push(@rv, [ $1, $2, $3, $4, $5, $6 ]);
+		}
+	}
+return @rv;
+}
+
 
 # istrue(key)
 # Checks if the value of this key (or it's synonyms) in %share is true
@@ -381,8 +396,12 @@ sub user_list
 {
 local($u, @rv);
 foreach $u (split(/[ \t,]+/ , $_[0])) {
-	if ($u =~ /^\@(.*)$/) { push(@rv, "group <tt>$1</tt>"); }
-	else { push(@rv, "<tt>$u</tt>"); }
+	if ($u =~ /^\@(.*)$/) {
+		push(@rv, "group <tt>".&html_escape($1)."</tt>");
+		}
+	else {
+		push(@rv, "<tt>".&html_escape($u)."</tt>");
+		}
 	}
 return join("," , @rv);
 }
@@ -393,8 +412,8 @@ return join("," , @rv);
 sub yesno_input
 {
 ($n = $_[0]) =~ s/ /_/g;
-return sprintf "<input type=radio name=$n value=yes %s> Yes\n".
-	       "<input type=radio name=$n value=no %s> No\n",
+return sprintf "<input type=radio name=$n value=yes %s> $text{'yes'}\n".
+	       "<input type=radio name=$n value=no %s> $text{'no'}\n",
 		&istrue($_[0]) ? "checked" : "",
 		&isfalse($_[0]) ? "checked" : "";
 }
@@ -503,9 +522,91 @@ return join(":", @u),"\n";
 sub set_password
 {
 local($out);
-$out = `$config{samba_password_program} $_[0] $_[1] 2>&1 </dev/null`;
+$out = `$config{'samba_password_program'} "$_[0]" "$_[1]" 2>&1 </dev/null`;
 return $out =~ /changed/;
 }
 
+# is_samba_running()
+# Returns 0 if not, 1 if it is, or 2 if run from (x)inetd
+sub is_samba_running
+{
+local ($found_inet, @smbpids, @nmbpids);
+if (&foreign_check("inetd")) {
+	&foreign_require("inetd", "inetd-lib.pl");
+	foreach $inet (&foreign_call("inetd", "list_inets")) {
+		$found_inet++ if (($inet->[8] =~ /smbd/ ||
+				   $inet->[9] =~ /smbd/) && $inet->[1]);
+		}
+	}
+elsif (&foreign_check("xinetd")) {
+	&foreign_require("xinetd", "xinetd-lib.pl");
+	foreach $xi (&foreign_call("xinetd", "get_xinetd_config")) {
+		local $q = $xi->{'quick'};
+		$found_inet++ if ($q->{'disable'}->[0] ne 'yes' &&
+				  $q->{'server'}->[0] =~ /smbd/);
+		}
+	}
+@smbpids = &find_byname("smbd");
+@nmbpids = &find_byname("nmbd");
+return !$found_inet && !@smbpids && !@nmbpids ? 0 :
+       !$found_inet ? 1 : 2;
+}
+
+# can($permissions_string, \%access, [$sname])
+# check global and per-share permissions:
+#
+# $permissions_string = any exists permissions except 'c' (creation).
+# \%access = ref on what get_module_acl() returns.
+sub can
+{
+local ($acl, $stype, @perm);
+local ($perm, $acc, $sname) = @_;
+@perm  = split(//, $perm);
+$sname = $in{'old_name'} || $in{'share'} unless $sname;
+
+{	local %share;
+	&get_share($sname); # use local %share
+	$stype = &istrue('printable') ? 'ps' : 'fs';
+	}
+
+# check global acl (r,w)
+foreach (@perm) {
+	next if ($_ ne 'r') && ($_ ne 'w');
+	return 0 unless $acc->{$_ . '_' . $stype};
+	}
+
+# check per-share acl
+if ($acc->{'per_' . $stype . '_acls'}) {
+    $acl = $acc->{'ACL' . $stype . '_' . $sname};
+    foreach (@perm) {
+#        next if $_ eq 'c'; # skip creation perms for per-share acls
+		return 0 if index($acl, $_) == -1;
+		}
+	}
+return 1;	
+}
+
+# save_samba_acl($permissions_string, \%access, $share_name)
+sub save_samba_acl
+{
+local ($p, $a, $s)=@_;
+defined(%share) || &get_share($s); # use global %share
+local $t=&istrue('printable') ? 'ps' : 'fs';
+$a->{'ACL'. $t .'_'. $s} = $p;
+#undef($can_cache);
+return &save_module_acl($a);
+}
+
+# drop_samba_acl(\%access, $share_name)
+sub drop_samba_acl
+{
+local ($a, $s)=@_;
+defined(%share) || &get_share($s); # use global %share
+local $t=&istrue('printable') ? 'ps' : 'fs';
+delete($a->{'ACL'. $t .'_' . $s});
+#undef($can_cache);
+return &save_module_acl($a);
+}
+
 1;
 
