@@ -114,7 +114,7 @@ printf "<input name=aliases size=40 value='%s'></td> </tr>\n",
 	$_[0]->{'aliases'};
 
 print "<tr> <td><b>$text{'acl_atypes'}</b></td> <td colspan=3>\n";
-for($n=1; $n<6; $n++) {
+for($n=1; $n<=6; $n++) {
 	printf "<input type=checkbox name=aedit_%s value=1 %s> %s\n",
 		$n, $_[0]->{"aedit_$n"} ? "checked" : "",
 		$text{"acl_atype$n"};
@@ -150,6 +150,8 @@ print "<tr> <td colspan=4><hr></td> </tr>\n";
 print "<tr> <td valign=top><b>$text{'acl_read'}</b></td> <td colspan=3>\n";
 printf "<input type=radio name=mmode value=0 %s> $text{'acl_none'}\n",
 	$_[0]->{'mmode'} == 0 ? "checked" : "";
+printf "<input type=radio name=mmode value=4 %s> $text{'acl_same'}\n",
+	$_[0]->{'mmode'} == 4 ? "checked" : "";
 printf "<input type=radio name=mmode value=1 %s> $text{'acl_all'}<br>\n",
 	$_[0]->{'mmode'} == 1 ? "checked" : "";
 printf "<input type=radio name=mmode value=2 %s> $text{'acl_users'}\n",
@@ -159,9 +161,18 @@ printf "<input name=musers size=40 value='%s'> %s<br>\n",
 	&user_chooser_button("musers", 1);
 printf "<input type=radio name=mmode value=3 %s> $text{'acl_userse'}\n",
 	$_[0]->{'mmode'} == 3 ? "checked" : "";
-printf "<input name=muserse size=40 value='%s'> %s</td> </tr>\n",
+printf "<input name=muserse size=40 value='%s'> %s<br>\n",
 	$_[0]->{'mmode'} == 3 ? $_[0]->{'musers'} : "",
 	&user_chooser_button("muserse", 1);
+printf "<input type=radio name=mmode value=5 %s> $text{'acl_usersg'}\n",
+	$_[0]->{'mmode'} == 5 ? "checked" : "";
+printf "<input name=musersg size=8 value='%s'> %s<br>\n",
+	$_[0]->{'mmode'} == 5 ? scalar(getgrgid($_[0]->{'musers'})) : "",
+	&group_chooser_button("musersg");
+printf "<input type=radio name=mmode value=6 %s> $text{'acl_usersm'}\n",
+	$_[0]->{'mmode'} == 6 ? "checked" : "";
+printf "<input name=musersm size=15 value='%s'></td> </tr>\n",
+	$_[0]->{'mmode'} == 6 ? $_[0]->{'musers'} : "";
 
 print "<tr> <td valign=top><b>$text{'acl_from'}</b></td> <td colspan=3>\n";
 printf "<input type=radio name=fmode value=0 %s> $text{'acl_any'}<br>\n",
@@ -180,14 +191,40 @@ printf "<input name=fdom size=20 value='%s'><br>\n",
 	$_[0]->{'fmode'} == 3 ? $_[0]->{'from'} : '';
 print "</td> </tr>\n";
 
-print "<tr> <td valign=top><b>$text{'acl_attach'}</b></td> <td colspan=3>\n";
+print "<tr> <td><b>$text{'acl_fromname'}</b></td>\n";
+print "<td colspan=3><input name=fromname size=40 ",
+      "value='$_[0]->{'fromname'}'></td> </tr>\n";
+
+print "<tr> <td><b>$text{'acl_attach'}</b></td> <td>\n";
 printf "<input type=radio name=attach_def value=1 %s> %s\n",
 	$_[0]->{'attach'}<0 ? 'checked' : '', $text{'acl_unlimited'};
 printf "<input type=radio name=attach_def value=0 %s>\n",
 	$_[0]->{'attach'}<0 ? '' : 'checked';
-printf "<input name=attach size=5 value='%s'> Kb\n",
+printf "<input name=attach size=5 value='%s'> kB\n",
 	$_[0]->{'attach'}<0 ? '' : $_[0]->{'attach'};
-print "</td> </tr>\n";
+print "</td>\n";
+
+print "<td><b>$text{'acl_canattach'}</b></td>\n";
+printf "<td><input type=radio name=canattach value=1 %s> %s\n",
+	$_[0]->{'canattach'} ? 'checked' : '', $text{'yes'};
+printf "<input type=radio name=canattach value=0 %s> %s</td> </tr>\n",
+	$_[0]->{'canattach'} ? '' : 'checked', $text{'no'};
+
+print "<tr> <td><b>$text{'acl_sent'}</b></td> <td colspan=3>\n";
+printf "<input type=radio name=sent_def value=1 %s> %s\n",
+	$_[0]->{'sent'} ? '' : 'checked', $text{'acl_none'};
+printf "<input type=radio name=sent_def value=0 %s>\n",
+	$_[0]->{'sent'} ? 'checked' : '';
+printf "<input name=sent size=13 value='%s'> %s</td> </tr>\n",
+	$_[0]->{'sent'}, &user_chooser_button("sent");
+
+print "<tr> <td><b>$text{'acl_boxname'}</b></td> <td colspan=3>\n";
+printf "<input type=radio name=boxname value=0 %s> %s\n",
+	$_[0]->{'boxname'} == 0 ? 'checked' : '', $text{'acl_boxname0'};
+printf "<input type=radio name=boxname value=1 %s> %s\n",
+	$_[0]->{'boxname'} == 1 ? 'checked' : '', $text{'acl_boxname1'};
+printf "<input type=radio name=boxname value=2 %s> %s</td> </tr>\n",
+	$_[0]->{'boxname'} == 2 ? 'checked' : '', $text{'acl_boxname2'};
 }
 
 # acl_security_save(&options)
@@ -215,18 +252,24 @@ $_[0]->{'amode'} = $in{'amode'};
 $_[0]->{'aliases'} = $in{'amode'} == 2 ? $in{'aliases'} : "";
 $_[0]->{'amax'} = $in{'amax_def'} ? undef : $in{'amax'};
 $_[0]->{'apath'} = $in{'apath'};
-foreach $i (1..5) {
+foreach $i (1..6) {
 	$_[0]->{"aedit_$i"} = $in{"aedit_$i"};
 	}
 $_[0]->{'omode'} = $in{'omode'};
 $_[0]->{'oaddrs'} = $in{'omode'} == 2 ? $in{'oaddrs'} : "";
 $_[0]->{'mmode'} = $in{'mmode'};
 $_[0]->{'musers'} = $in{'mmode'} == 2 ? $in{'musers'} :
-		    $in{'mmode'} == 3 ? $in{'muserse'} : "";
+		    $in{'mmode'} == 3 ? $in{'muserse'} :
+		    $in{'mmode'} == 5 ? scalar(getgrnam($in{'musersg'})) :
+		    $in{'mmode'} == 6 ? $in{'musersm'} : "";
 $_[0]->{'fmode'} = $in{'fmode'};
 $_[0]->{'from'} = $in{'fmode'} == 0 ? undef :
 		  $in{'fmode'} == 1 ? $in{'fdoms'} :
 		  $in{'fmode'} == 2 ? $in{'faddrs'} : $in{'fdom'};
+$_[0]->{'fromname'} = $in{'fromname'};
 $_[0]->{'attach'} = $in{'attach_def'} ? -1 : $in{'attach'};
+$_[0]->{'sent'} = $in{'sent_def'} ? '' : $in{'sent'};
+$_[0]->{'canattach'} = $in{'canattach'};
+$_[0]->{'boxname'} = $in{'boxname'};
 }
 
