@@ -1,6 +1,8 @@
 # boxes-lib.pl
 # Functions to parsing user mail files
 
+use POSIX;
+
 # list_mails(user, [start], [end])
 sub list_mails
 {
@@ -19,7 +21,7 @@ else {
 	$end = @index-1 if ($end >= @index);
 	}
 @rv = map { undef } @index;
-open(MAIL, "$config{'mail_dir'}/$_[0]");
+open(MAIL, &user_mail_file($_[0]));
 for($i=$start; $i<=$end; $i++) {
 	local ($mail, $line, @headers);
 	seek(MAIL, $index[$i]->[0], 0);
@@ -29,8 +31,9 @@ for($i=$start; $i<=$end; $i++) {
 	local $lnum = 0;
 	while(1) {
 		$lnum++;
-		($line = <MAIL>) =~ s/\r|\n//g;
+		$line = <MAIL>;
 		$mail->{'size'} += length($line);
+		$line =~ s/\r|\n//g;
 		last if ($line =~ /^$/);
 		if ($line =~ /^(\S+):\s*(.*)/) {
 			push(@headers, [ $1, $2 ]);
@@ -38,6 +41,9 @@ for($i=$start; $i<=$end; $i++) {
 		elsif ($line =~ /^(\s+.*)/) {
 			$headers[$#headers]->[1] .= $1 unless($#headers < 0);
 			}
+		elsif ($line =~ /^From\s+(\S+).*\d+/ && $1 ne '-') {
+			$mail->{'fromline'} = $line;
+			}
 		}
 	$mail->{'headers'} = \@headers;
 	foreach $h (@headers) {
@@ -47,7 +53,7 @@ for($i=$start; $i<=$end; $i++) {
 	# read the mail body
 	while(1) {
 		$line = <MAIL>;
-		last if (!$line || $line =~ /^From\s+.*\d+\n/);
+		last if (!$line || $line =~ /^From\s+(\S+).*\d+\n/ && $1 ne '-');
 		$lnum++;
 		$mail->{'size'} += length($line);
 		$mail->{'body'} .= $line;
@@ -65,7 +71,7 @@ sub search_mail
 {
 local @index = &build_index($_[0]);
 local (@rv, $i);
-open(MAIL, "$config{'mail_dir'}/$_[0]");
+open(MAIL, &user_mail_file($_[0]));
 for($i=@index-1; $i>=0; $i--) {
 	local ($mail, $line, @headers);
 	seek(MAIL, $index[$i]->[0], 0);
@@ -84,6 +90,9 @@ for($i=@index-1; $i>=0; $i--) {
 		elsif ($line =~ /^(\s+.*)/) {
 			$headers[$#headers]->[1] .= $1;
 			}
+		elsif ($line =~ /^From\s+(\S+).*\d+/ && $1 ne '-') {
+			$mail->{'fromline'} = $line;
+			}
 		}
 	$mail->{'headers'} = \@headers;
 	foreach $h (@headers) {
@@ -93,16 +102,21 @@ for($i=@index-1; $i>=0; $i--) {
 	# read mail body
 	while(1) {
 		$line = <MAIL>;
-		last if (!$line || $line =~ /^From\s+.*\d+\n/);
+		last if (!$line || $line =~ /^From\s+(\S+).*\d+\n/ && $1 ne '-');
 		$lnum++;
 		$mail->{'size'} += length($line);
 		$mail->{'body'} .= $line;
 		}
 	$mail->{'eline'} = $mail->{'line'} + $lnum - 1;
 	$mail->{'idx'} = $i;
-	if ($_[1] eq 'body' && $mail->{'body'} =~ /$_[2]/i ||
-	    $_[1] ne 'body' && $mail->{'header'}->{$_[1]} =~ /$_[2]/i) {
-		push(@rv, $mail);
+	if ($_[1] eq 'body') {
+		push(@rv, $mail) if ($mail->{'body'} =~ /$_[2]/i);
+		}
+	elsif ($_[1] eq 'size') {
+		push(@rv, $mail) if ($mail->{'size'} > $_[2]);
+		}
+	else {
+		push(@rv, $mail) if ($mail->{'header'}->{$_[1]} =~ /$_[2]/i);
 		}
 	}
 return @rv;
@@ -113,35 +127,62 @@ return @rv;
 sub build_index
 {
 local @index;
-local @ist = stat("$module_config_directory/$_[0].index");
-local @st = stat("$config{'mail_dir'}/$_[0]");
-if (!@ist || $ist[9] < $st[9]) {
-	# Need to build index
-	local $pos = 0;
-	local $lnum = 0;
-	open(MAIL, "$config{'mail_dir'}/$_[0]");
+local $ifile = $user_module_config_directory ?
+		"$user_module_config_directory/$_[0].index" :
+		"$module_config_directory/$_[0].index";
+local @ist = stat($ifile);
+local @st = stat(&user_mail_file($_[0]));
+
+if (open(INDEX, $ifile)) {
+	@index = map { /(\d+)\s+(\d+)/; [ $1, $2 ] } <INDEX>;
+	close(INDEX);
+	}
+
+if (!@ist || !@st || $ist[9] < $st[9]) {
+	# The mail file is newer than the index
+	local $fromok = 1;
+	local ($l, $ll);
+	if (open(MAIL, &user_mail_file($_[0]))) {
+		local $il = $#index;
+		local $i;
+		for($i=($il>100 ? 100 : $il); $i>=0; $i--) {
+			$l = $index[$il-$i];
+			seek(MAIL, $index[$il-$i]->[0], 0);
+			$ll = <MAIL>;
+			$fromok = 0 if ($ll !~ /^From\s+(\S+).*\d+\n/ || $1 eq '-');
+			}
+		}
+	else { $fromok = 0; }
+	local ($pos, $lnum);
+	if (scalar(@index) && $fromok && $st[7] > $l->[0]) {
+		# Mail file seems to have gotten bigger, most likely
+		# because new mail has arrived ... only reindex the new mails
+		$pos = $l->[0] + length($ll);
+		$lnum = $l->[1] + 1;
+		}
+	else {
+		# Mail file has changed in some other way ... do a rebuild
+		$pos = 0;
+		$lnum = 0;
+		undef(@index);
+		seek(MAIL, 0, 0);
+		}
 	while(<MAIL>) {
-		if (/^From\s+.*\d+\n/) {
+		if (/^From\s+(\S+).*\d+\n/ && $1 ne '-') {
 			push(@index, [ $pos, $lnum ]);
 			}
 		$pos += length($_);
 		$lnum++;
 		}
 	close(MAIL);
-	open(INDEX, ">$module_config_directory/$_[0].index");
+	open(INDEX, ">$ifile");
 	print INDEX map { $_->[0]." ".$_->[1]."\n" } @index;
 	close(INDEX);
 	}
-else {
-	# Read the index file
-	open(INDEX, "$module_config_directory/$_[0].index");
-	@index = map { /(\d+)\s+(\d+)/; [ $1, $2 ] } <INDEX>;
-	close(INDEX);
-	}
 return @index;
 }
 
-# parse_mail(&mail)
+# parse_mail(&mail, [&parent])
 # Extracts the attachments from the mail body
 sub parse_mail
 {
@@ -161,6 +202,7 @@ if ($ct =~ /multipart\/(\S+)/i && ($ct =~ /boundary="([^"]+)"/i ||
 		# read attachment headers
 		local (@headers, $attach);
 		while($lines[$l]) {
+			$attach->{'raw'} .= $lines[$l]."\n";
 			if ($lines[$l] =~ /^(\S+):\s*(.*)/) {
 				push(@headers, [ $1, $2 ]);
 				}
@@ -169,6 +211,7 @@ if ($ct =~ /multipart\/(\S+)/i && ($ct =~ /boundary="([^"]+)"/i ||
 				}
 			$l++;
 			}
+		$attach->{'raw'} .= $lines[$l]."\n";
 		$l++;
 		$attach->{'headers'} = \@headers;
 		foreach $h (@headers) {
@@ -181,23 +224,26 @@ if ($ct =~ /multipart\/(\S+)/i && ($ct =~ /boundary="([^"]+)"/i ||
 			$attach->{'type'} = 'text/plain';
 			}
 		if ($attach->{'header'}->{'content-disposition'} =~
-		    /filename="([^"]+)"/i) {
+		    /filename\s*=\s*"([^"]+)"/i) {
 			$attach->{'filename'} = $1;
 			}
 		elsif ($attach->{'header'}->{'content-disposition'} =~
-		       /filename=([^;\s]+)/i) {
+		       /filename\s*=\s*([^;\s]+)/i) {
 			$attach->{'filename'} = $1;
 			}
 		elsif ($attach->{'header'}->{'content-type'} =~
-		    /name="([^"]+)"/i) {
+		    /name\s*=\s*"([^"]+)"/i) {
 			$attach->{'filename'} = $1;
 			}
 
 		# read the attachment body
-		while($l < $max && $lines[$l] !~ /^$bound(--)?$/) {
+		while($l < $max && $lines[$l] ne $bound && $lines[$l] ne "$bound--") {
 			$attach->{'data'} .= $lines[$l]."\n";
+			$attach->{'raw'} .= $lines[$l]."\n";
 			$l++;
 			}
+		$attach->{'data'} =~ s/\n\n$/\n/;	# Lose trailing blank line
+		$attach->{'raw'} =~ s/\n\n$/\n/;
 
 		# decode if necessary
 		if (lc($attach->{'header'}->{'content-transfer-encoding'}) eq
@@ -214,37 +260,14 @@ if ($ct =~ /multipart\/(\S+)/i && ($ct =~ /boundary="([^"]+)"/i ||
 			}
 
 		$attach->{'idx'} = scalar(@attach);
-		push(@attach, $attach);
-		if ($attach->{'type'} eq 'message/rfc822') {
-			# Decode this included email as well
-			local ($amail, @aheaders, $i);
-			local @alines = split(/\n/, $attach->{'data'});
-			while($i < @alines && $alines[$i]) {
-				if ($alines[$i] =~ /^(\S+):\s*(.*)/) {
-					push(@aheaders, [ $1, $2 ]);
-					}
-				elsif ($alines[$i] =~ /^(\s+.*)/) {
-					$aheaders[$#aheaders]->[1] .= $1;
-					}
-				$i++;
-				}
-			$amail->{'headers'} = \@aheaders;
-			foreach $h (@aheaders) {
-				$amail->{'header'}->{lc($h->[0])} = $h->[1];
-				}
-			splice(@alines, 0, $i);
-			$amail->{'body'} = join("\n", @alines)."\n";
-			&parse_mail($amail);
-			map { $_->{'idx'} += scalar(@attach) }
-			    @{$amail->{'attach'}};
-			push(@attach, @{$amail->{'attach'}});
-			}
-		elsif ($attach->{'type'} =~ /multipart\/(\S+)/i) {
+		$attach->{'parent'} = $_[1] ? $_[1] : $_[0];
+		push(@attach, $attach) if (@headers || $attach->{'data'});
+		if ($attach->{'type'} =~ /multipart\/(\S+)/i) {
 			# This attachment contains more attachments
 			local $amail = { 'header' => $attach->{'header'},
 					 'body' => $attach->{'data'} };
-			&parse_mail($amail);
-			pop(@attach);
+			&parse_mail($amail, $attach);
+			$attach->{'attach'} = [ @{$amail->{'attach'}} ];
 			map { $_->{'idx'} += scalar(@attach) }
 			    @{$amail->{'attach'}};
 			push(@attach, @{$amail->{'attach'}});
@@ -254,19 +277,43 @@ if ($ct =~ /multipart\/(\S+)/i && ($ct =~ /boundary="([^"]+)"/i ||
 		}
 	$_[0]->{'attach'} = \@attach;
 	}
-elsif ($_[0]->{'body'} =~ /begin\s+([0-7]+)\s+(\S+)/i) {
-	# Looks like a uuencoded message!
-	local $filename = $2;
-	$_[0]->{'attach'} = [ { 'type' => &guess_type($filename),
-				'idx' => 0,
-				'filename' => $filename,
-				'data' => &uudecode($_[0]->{'body'}) } ];
+elsif ($_[0]->{'body'} =~ /begin\s+([0-7]+)\s+(.*)/i) {
+	# Message contains uuencoded file(s)
+	local @lines = split(/\n/, $_[0]->{'body'});
+	local ($attach, $rest);
+	foreach $l (@lines) {
+		if ($l =~ /^begin\s+([0-7]+)\s+(.*)/i) {
+			$attach = { 'type' => &guess_type($2),
+				    'idx' => scalar(@{$_[0]->{'attach'}}),
+				    'parent' => $_[1],
+				    'filename' => $2 };
+			push(@{$_[0]->{'attach'}}, $attach);
+			}
+		elsif ($l =~ /^end/ && $attach) {
+			$attach = undef;
+			}
+		elsif ($attach) {
+			$attach->{'data'} .= unpack("u", $l);
+			}
+		else {
+			$rest .= $l."\n";
+			}
+		}
+	if ($rest =~ /\S/) {
+		# Some leftover text
+		push(@{$_[0]->{'attach'}},
+			{ 'type' => "text/plain",
+			  'idx' => scalar(@{$_[0]->{'attach'}}),
+			  'parent' => $_[1],
+			  'data' => $rest });
+		}
 	}
 elsif ($_[0]->{'header'}->{'content-transfer-encoding'} eq 'base64') {
 	# Signed body section
 	$ct =~ s/;.*$//;
 	$_[0]->{'attach'} = [ { 'type' => lc($ct),
 				'idx' => 0,
+				'parent' => $_[1],
 				'data' => &b64decode($_[0]->{'body'}) } ];
 	}
 else {
@@ -282,35 +329,164 @@ else {
 		}
 	$_[0]->{'attach'} = [ { 'type' => lc($type),
 				'idx' => 0,
+				'parent' => $_[1],
 				'data' => $body } ];
 	}
 }
 
 # delete_mail(user, &mail, ...)
+# Delete mail messages from a user by copying the file and rebuilding the index
 sub delete_mail
 {
-local $lref = &read_file_lines("$config{'mail_dir'}/$_[0]");
-local @m = sort { $b->{'line'} <=> $a->{'line'} } @_[1..@_-1];
-local $m;
-foreach $m (@m) {
-	splice(@$lref, $m->{'line'}, $m->{'eline'} - $m->{'line'} + 1);
+local @m = sort { $a->{'line'} <=> $b->{'line'} } @_[1..@_-1];
+local $i = 0;
+local $f = &user_mail_file($_[0]);
+local $ifile = $user_module_config_directory ?
+		"$user_module_config_directory/$_[0].index" :
+		"$module_config_directory/$_[0].index";
+local $lnum = 0;
+local %dline;
+local ($dpos = 0, $dlnum = 0);
+local @index;
+
+local $tmpf = $< == 0 ? "$f.del" : "$user_module_config_directory/$_[0].del";
+open(SOURCE, $f);
+open(DEST, ">$tmpf");
+while(<SOURCE>) {
+	if ($i >= @m || $lnum < $m[$i]->{'line'}) {
+		if (/^From\s+(\S+).*\d+\n/ && $1 ne '-') {
+			push(@index, [ $dpos, $dlnum ]);
+			}
+		$dpos += length($_);
+		$dlnum++;
+		local $w = (print DEST $_);
+		if (!$w) {
+			local $e = $?;
+			close(DEST);
+			close(SOURCE);
+			unlink($tmpf);
+			&error("Write to $tmpf failed : $e");
+			}
+		}
+	elsif ($lnum == $m[$i]->{'eline'}) {
+		$dline{$m[$i]->{'line'}}++;
+		$i++;
+		}
+	$lnum++;
 	}
-&flush_file_lines();
+close(SOURCE);
+close(DEST);
+local @st = stat($f);
+unlink($f);
+open(INDEX, ">$ifile");
+print INDEX map { $_->[0]." ".$_->[1]."\n" } @index;
+close(INDEX);
+if ($< == 0) {
+	rename($tmpf, $f);
+	}
+else {
+	system("cat $tmpf >$f && rm -f $tmpf");
+	}
+chown($st[4], $st[5], $f);
+chmod($st[2], $f);
 }
 
-# send_mail(&mail)
+# modify_mail(user, old, new)
+# Modify one email message in a mailbox by copying the file and rebuilding
+# the index.
+sub modify_mail
+{
+local $f = &user_mail_file($_[0]);
+local $ifile = $user_module_config_directory ?
+		"$user_module_config_directory/$_[0].index" :
+		"$module_config_directory/$_[0].index";
+local $lnum = 0;
+local ($sizediff, $linesdiff);
+local @index = &build_index($_[0]);
+
+# Replace the email that gets modified
+local $tmpf = $< == 0 ? "$f.del" : "$user_module_config_directory/$_[0].del";
+open(SOURCE, $f);
+open(DEST, ">$tmpf");
+while(<SOURCE>) {
+	if ($lnum < $_[1]->{'line'} || $lnum > $_[1]->{'eline'}) {
+		# before or after the message to change
+		local $w = (print DEST $_);
+		if (!$w) {
+			local $e = $?;
+			close(DEST);
+			close(SOURCE);
+			unlink($tmpf);
+			&error("Write failed : $e");
+			}
+		}
+	elsif ($lnum == $_[1]->{'line'}) {
+		# found start of message to change .. put in the new one
+		close(DEST);
+		local @ost = stat($tmpf);
+		local $nlines = &send_mail($_[2], $tmpf);
+		local @nst = stat($tmpf);
+		local $newsize = $nst[7] - $ost[7];
+		$sizediff = $newsize - $_[1]->{'size'};
+		$linesdiff = $nlines - ($_[1]->{'eline'} - $_[1]->{'line'} + 1);
+		open(DEST, ">>$tmpf");
+		}
+	$lnum++;
+	}
+close(SOURCE);
+close(DEST);
+
+# Now update the index and delete the temp file
+foreach $i (@index) {
+	if ($i->[1] > $_[1]->{'line'}) {
+		# Shift mails after the modified
+		$i->[0] += $sizediff;
+		$i->[1] += $linesdiff;
+		}
+	}
+local @st = stat($f);
+unlink($f);
+open(INDEX, ">$ifile");
+print INDEX map { $_->[0]." ".$_->[1]."\n" } @index;
+close(INDEX);
+if ($< == 0) {
+	rename($tmpf, $f);
+	}
+else {
+	system("cat $tmpf >$f && rm -f $tmpf");
+	}
+chown($st[4], $st[5], $f);
+chmod($st[2], $f);
+
+}
+
+# send_mail(&mail, [file])
+# Send out some email message or append it to a file.
+# Returns the number of lines written.
 sub send_mail
 {
 local (%header, $h);
-if ($config{'send_mode'}) {
+local $lnum = 0;
+foreach $h (@{$_[0]->{'headers'}}) {
+	$header{lc($h->[0])} = $h->[1];
+	}
+local @from = &address_parts($header{'from'});
+if ($_[1]) {
+	# Just append the email to a file using mbox format
+	local @tm = localtime(time());
+	open(MAIL, ">>$_[1]");
+	$lnum++;
+	print MAIL $_[0]->{'fromline'} ? $_[0]->{'fromline'}."\n" :
+		   strftime("From $from[0] %a %b %e %H:%M:%S %Y\n", @tm);
+	push(@{$_[0]->{'headers'}},
+	     [ 'Date', strftime("%a, %d %b %Y %H:%M:%S %Z", @tm) ])
+		if (!$header{'date'});
+	}
+elsif ($config{'send_mode'}) {
 	# Connect to SMTP server
-	foreach $h (@{$_[0]->{'headers'}}) {
-		$header{lc($h->[0])} = $h->[1];
-		}
 	&open_socket($config{'send_mode'}, 25, MAIL);
 	&smtp_command(MAIL);
 	&smtp_command(MAIL, "helo ".&from_hostname()."\n");
-	local @from = &address_parts($header{'from'});
 	&smtp_command(MAIL, "mail from: $from[0]\n");
 	foreach $u (&address_parts($header{'to'}.",".$header{'cc'}.
 						 ",".$header{'bcc'})) {
@@ -320,40 +496,62 @@ if ($config{'send_mode'}) {
 	}
 else {
 	# Start sendmail
-	open(MAIL, "| $config{'sendmail_path'} -t >/dev/null 2>&1");
+	open(MAIL, "| $config{'sendmail_path'} -t -f$from[0] >/dev/null 2>&1");
 	}
+local $ctype = "multipart/mixed";
 foreach $h (@{$_[0]->{'headers'}}) {
-	print MAIL $h->[0],": ",$h->[1],"\n";
+	if ($h->[0] !~ /^(MIME-Version|Content-Type)$/i) {
+		print MAIL $h->[0],": ",$h->[1],"\n";
+		$lnum++;
+		}
+	elsif ($h->[0] eq 'Content-Type') {
+		$ctype = $h->[1];
+		}
 	}
 print MAIL "MIME-Version: 1.0\n";
-local $bound = "--------".time();
-print MAIL "Content-Type: multipart/mixed; boundary=\"$bound\"\n";
+local $bound = "bound".time();
+print MAIL "Content-Type: $ctype; boundary=\"$bound\"\n";
 print MAIL "\n";
+$lnum += 3;
 
 # Send attachments
 print MAIL "This is a multi-part message in MIME format.\n";
+$lnum++;
 foreach $a (@{$_[0]->{'attach'}}) {
+	print MAIL "\n";
 	print MAIL "--",$bound,"\n";
+	$lnum += 2;
 	local $enc;
 	foreach $h (@{$a->{'headers'}}) {
 		print MAIL $h->[0],": ",$h->[1],"\n";
 		$enc = $h->[1] if (lc($h->[0]) eq 'content-transfer-encoding');
+		$lnum++;
 		}
 	print MAIL "\n";
+	$lnum++;
 	if (lc($enc) eq 'base64') {
-		print MAIL &encode_base64($a->{'data'});
+		local $enc = &encode_base64($a->{'data'});
+		print MAIL $enc;
+		$lnum += ($enc =~ tr/\n/\n/);
 		}
 	else {
 		print MAIL $a->{'data'};
-		print MAIL "\n" if ($a->{'data'} !~ /\n$/);
+		$lnum += ($a->{'data'} =~ tr/\n/\n/);
+		if ($a->{'data'} !~ /\n$/) {
+			print MAIL "\n";
+			$lnum++;
+			}
 		}
 	}
+print MAIL "\n";
 print MAIL "--",$bound,"--\n";
-if ($config{'send_mode'}) {
+$lnum += 2;
+if ($config{'send_mode'} && !$_[1]) {
 	&smtp_command(MAIL, ".\n");
 	&smtp_command(MAIL, "quit\n");
 	}
 close(MAIL);
+return $lnum;
 }
 
 # b64decode(string)
@@ -396,8 +594,10 @@ return "application/octet-stream";
 # can_read_mail(user)
 sub can_read_mail
 {
+return 0 if ($_[0] =~ /\.\./);
 return 0 if ($access{'mmode'} == 0);
 return 1 if ($access{'mmode'} == 1);
+return 1 if ($_[0] && $access{'sent'} eq $_[0]);
 local $u;
 if ($access{'mmode'} == 2) {
 	foreach $u (split(/\s+/, $access{'musers'})) {
@@ -405,12 +605,23 @@ if ($access{'mmode'} == 2) {
 		}
 	return 0;
 	}
-else {
+elsif ($access{'mmode'} == 4) {
+	return 1 if ($_[0] eq $remote_user);
+	}
+elsif ($access{'mmode'} == 5) {
+	local @u = getpwnam($_[0]);
+	return $u[3] == $access{'musers'};
+	}
+elsif ($access{'mmode'} == 3) {
 	foreach $u (split(/\s+/, $access{'musers'})) {
 		return 0 if ($u eq $_[0]);
 		}
 	return 1;
 	}
+elsif ($access{'mmode'} == 6) {
+	return ($_[0] =~ /^$access{'musers'}$/);
+	}
+return 0;	# can't happen!
 }
 
 # from_hostname()
@@ -502,7 +713,7 @@ return @rv;
 sub link_urls
 {
 local $r = $_[0];
-$r =~ s/((http|ftp|https|mailto):[^><"'\s&\?]+)/<a href="$1">$1<\/a>/g;
+$r =~ s/((http|ftp|https|mailto):[^><"'\s]+[^><"'\s\.])/<a href="$1">$1<\/a>/g;
 return $r;
 }
 
@@ -531,14 +742,22 @@ return $_[0];
 sub simplify_from
 {
 local $rv = &decode_mimewords($_[0]);
-$rv =~ s/<[^>]+>//g;
-return $rv ? &html_escape($rv) : $text{'mail_none'};
+if ($rv !~ /\S/) {
+	return $text{'mail_nonefrom'};
+	}
+elsif ($rv =~ /(.*\S.*)<[^>]+>/) {
+	return &html_escape("$1");
+	}
+else {
+	return &html_escape($rv);
+	}
 }
 
 # simplify_subject(subject)
 sub simplify_subject
 {
 local $rv = &decode_mimewords($_[0]);
+$rv = substr($rv, 0, 80)." .." if (length($rv) > 80);
 return $rv =~ /\S/ ? &html_escape($rv) : "<br>";
 }
 
@@ -626,4 +845,40 @@ sub _decode_B {
     &decode_base64($str);
 }
 
+# user_mail_file(user)
+sub user_mail_file
+{
+if ($config{'mail_dir'}) {
+	return "$config{'mail_dir'}/$_[0]";
+	}
+else {
+	local @u = getpwnam($_[0]);
+	return "$u[7]/$config{'mail_file'}";
+	}
+}
+
+# extract_mail(data)
+# Converts the text of a message into mail object.
+sub extract_mail
+{
+local ($amail, @aheaders, $i);
+local @alines = split(/\n/, $_[0]);
+while($i < @alines && $alines[$i]) {
+	if ($alines[$i] =~ /^(\S+):\s*(.*)/) {
+		push(@aheaders, [ $1, $2 ]);
+		}
+	elsif ($alines[$i] =~ /^(\s+.*)/) {
+		$aheaders[$#aheaders]->[1] .= $1;
+		}
+	$i++;
+	}
+$amail->{'headers'} = \@aheaders;
+foreach $h (@aheaders) {
+	$amail->{'header'}->{lc($h->[0])} = $h->[1];
+	}
+splice(@alines, 0, $i);
+$amail->{'body'} = join("\n", @alines)."\n";
+return $amail;
+}
+
 1;
