@@ -1,7 +1,7 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-require './sendmail-lib.pl';
+do './sendmail-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params)
 # Converts logged information from this module into human-readable form
@@ -10,17 +10,24 @@ sub parse_webmin_log
 local ($user, $script, $action, $type, $object, $p, $long) = @_;
 if ($type eq 'alias' || $type eq 'virtuser' || $type eq 'mailer' ||
     $type eq 'generic' || $type eq 'domain' || $type eq 'access') {
-	return &text("log_${type}_${action}", "<tt>$object</tt>");
+	return &text("log_${type}_${action}",
+		     "<tt>".&html_escape($object)."</tt>");
 	}
 elsif ($action eq 'delmailq') {
-	return &text("log_delmailq", &extract_email($p->{'from'}));
+	if ($p->{'from'}) {
+		return &text("log_delmailq",
+			     &html_escape(extract_email($p->{'from'})));
+		}
+	else {
+		return &text("log_delmailqs", $p->{'count'});
+		}
 	}
 elsif ($action eq 'delmail') {
 	local @d = split(/\0/, $p->{'d'});
 	return &text("log_delmail", scalar(@d), "<tt>$p->{'user'}</tt>");
 	}
 elsif ($action eq 'send') {
-	return &text('log_send', &extract_email($p->{'to'}));
+	return &text('log_send', &html_escape(&extract_email($p->{'to'})));
 	}
 elsif ($text{"log_$action"}) {
 	return $text{"log_$action"};
