@@ -128,28 +128,38 @@ printf "<td><input name=from_dom size=15 value=\"%s\"></td> </tr>\n",
 $mode = $v->{'to'} =~ /^error:(\S+)\s*(.*)$/ ? 0 :
 	$v->{'to'} =~ /^\%1\@(\S+)$/ ? 1 :
 	$v->{'to'} =~ /^(.*)$/ ? 2 : 2;
-print "<tr> <td><b>$text{'vform_to'}</b></td>\n";
-printf "<td><input type=radio name=to_type value=2 %s> %s</td>\n",
-	$mode == 2 ? "checked" : "", $text{'vform_address'};
-printf "<td><input name=to_addr size=20 value=\"%s\"></td> </tr>\n",
-	$mode == 2 ? $1 : "";
-print "<tr> <td></td>\n";
-printf "<td><input type=radio name=to_type value=1 %s> %s</td>\n",
-	$mode == 1 ? "checked" : "", $text{'vform_domain'};
-printf "<td><input name=to_dom size=15 value=\"%s\"></td> </tr>\n",
-	$mode == 1 ? $1 : "";
-print "<tr> <td></td>\n";
-printf "<td><input type=radio name=to_type value=0 %s> %s</td>\n",
-	$mode == 0 ? "checked" : "", $text{'vform_error'};
-print "<td><select name=to_code>\n";
-foreach $ecode ("$text{'vform_err_nouser'}", "$text{'vform_err_nohost'}",
-	"$text{'vform_err_unav'}", "$text{'vform_err_temp'}",
-	"$text{'vform_err_proto'}") {
-	printf "<option %s>$ecode\n", $1 eq $ecode ? "selected" : "";
+local $tt = "<b>$text{'vform_to'}</b>";
+if ($access{'vedit_2'}) {
+	print "<tr> <td>$tt</td>\n";
+	printf "<td><input type=radio name=to_type value=2 %s> %s</td>\n",
+		$mode == 2 ? "checked" : "", $text{'vform_address'};
+	printf "<td><input name=to_addr size=20 value=\"%s\"></td> </tr>\n",
+		$mode == 2 ? $1 : "";
+	$tt = undef;
+	}
+if ($access{'vedit_1'}) {
+	print "<tr> <td>$tt</td>\n";
+	printf "<td><input type=radio name=to_type value=1 %s> %s</td>\n",
+		$mode == 1 ? "checked" : "", $text{'vform_domain'};
+	printf "<td><input name=to_dom size=15 value=\"%s\"></td> </tr>\n",
+		$mode == 1 ? $1 : "";
+	$tt = undef;
+	}
+if ($access{'vedit_0'}) {
+	print "<tr> <td>$tt</td>\n";
+	printf "<td><input type=radio name=to_type value=0 %s> %s</td>\n",
+		$mode == 0 ? "checked" : "", $text{'vform_error'};
+	print "<td><select name=to_code>\n";
+	foreach $ecode ($text{'vform_err_nouser'}, $text{'vform_err_nohost'},
+			$text{'vform_err_unav'}, $text{'vform_err_temp'},
+			$text{'vform_err_proto'}) {
+		printf "<option %s>$ecode\n", $1 eq $ecode ? "selected" : "";
+		}
+	print "</select> $text{'vform_msg'}\n";
+	printf "<input name=to_error size=15 value=\"%s\"></td> </tr>\n",
+		$mode == 0 ? $2 : "";
+	$tt = undef;
 	}
-print "</select> $text{'vform_msg'}\n";
-printf "<input name=to_error size=15 value=\"%s\"></td> </tr>\n",
-	$mode == 0 ? $2 : "";
 
 print "<tr> <td colspan=3 align=right>\n";
 if ($_[0]) {
