@@ -3,6 +3,7 @@
 
 do '../web-lib.pl';
 &init_config();
+%access = &get_module_acl();
 
 sub list_servers
 {
@@ -31,6 +32,7 @@ sub save_server
 {
 &lock_file("$module_config_directory/$_[0]->{'id'}.serv");
 &write_file("$module_config_directory/$_[0]->{'id'}.serv", $_[0]);
+chmod(0600, "$module_config_directory/$_[0]->{'id'}.serv");
 &unlock_file("$module_config_directory/$_[0]->{'id'}.serv");
 }
 
@@ -42,11 +44,22 @@ unlink("$module_config_directory/$_[0].serv");
 &unlock_file("$module_config_directory/$_[0].serv");
 }
 
+# can_use_server(&server)
+sub can_use_server
+{
+return 1 if ($access{'servers'} eq '*');
+foreach $s (split(/\s+/, $access{'servers'})) {
+	return 1 if ($_[0]->{'host'} eq $s);
+	}
+return 0;
+}
+
 @server_types = ( [ 'caldera', 'OpenLinux' ],
 		  [ 'redhat', 'Redhat Linux' ],
 		  [ 'suse', 'SuSE Linux' ],
 		  [ 'debian', 'Debian Linux' ],
 		  [ 'mandrake', 'Mandrake Linux' ],
+		  [ 'msc', 'MSC.Linux' ],
 		  [ 'linux', 'Linux' ],
 		  [ 'freebsd', 'FreeBSD' ],
 		  [ 'solaris', 'Solaris' ],
@@ -54,7 +67,7 @@ unlink("$module_config_directory/$_[0].serv");
 		  [ 'sco', 'SCO' ],
 		  [ 'mac', 'Macintosh' ],
 		  [ 'irix', 'IRIX' ],
-		  [ 'unknown', 'Other' ] );
+		  [ 'unknown', $text{'lib_other'} ] );
 
 
 1;
