@@ -5,7 +5,11 @@
 
 # Find install directory
 cd `dirname $0`
-wadir=`pwd`;
+if [ -x /bin/pwd ]; then
+	wadir=`/bin/pwd`
+else
+	wadir=`pwd`;
+fi
 ver=`cat $wadir/version`
 
 echo "***********************************************************************"
@@ -18,14 +22,22 @@ echo ""
 # Only root can run this
 id | grep "uid=0(" >/dev/null
 if [ $? != "0" ]; then
-	echo "ERROR: The Webmin install script must be run as root";
-	echo "";
-	exit 1;
+	uname -a | grep -i CYGWIN >/dev/null
+	if [ $? != "0" ]; then
+		echo "ERROR: The Webmin install script must be run as root";
+		echo "";
+		exit 1;
+	fi
 fi
 
 # Display install directory
 echo "Installing Webmin in $wadir ..."
 allmods=`cd $wadir; ls */module.info | sed -e 's/\/module.info//g' | xargs echo`
+if [ "$allmods" = "" ]; then
+	echo "ERROR: Failed to get module list"
+	echo ""
+	exit 1
+fi
 echo ""
 
 # Ask for webmin config directory
@@ -111,7 +123,7 @@ if [ "$upgrading" = 1 ]; then
 else
 	# Config directory exists .. make sure it is not in use
 	ls $config_dir | grep -v rpmsave >/dev/null 2>&1
-	if [ "$?" = "0" ]; then
+	if [ "$?" = "0" -a "$config_dir" != "/etc/webmin" ]; then
 		echo "ERROR: Config directory $config_dir is not empty"
 		echo ""
 		exit 2
@@ -131,6 +143,11 @@ else
 		echo ""
 		exit 3
 	fi
+	if [ "$var_dir" = "/" ]; then
+		echo "Log directory cannot be /"
+		exit ""
+		exit 3
+	fi
 	if [ ! -d $var_dir ]; then
 		mkdir $var_dir
 		if [ $? != 0 ]; then
@@ -139,7 +156,6 @@ else
 			exit 3
 		fi
 	fi
-	echo $var_dir > $config_dir/var-path
 	echo ""
 
 	# Ask where perl is installed
@@ -208,7 +224,6 @@ else
 		echo ""
 		exit 8
 	fi
-	echo $perl > $config_dir/perl-path
 	echo "Perl seems to be installed ok"
 	echo ""
 
@@ -269,6 +284,12 @@ else
 			login="admin"
 		fi
 	fi
+	echo $login | grep : >/dev/null
+	if [ "$?" = "0" ]; then
+		echo "ERROR: Username contains a : character"
+		echo ""
+		exit 14
+	fi
 	printf "Login password: "
 	if [ "$password" = "" -a "$crypt" = "" ]; then
 		stty -echo
@@ -285,9 +306,15 @@ else
 			echo ""
 			exit 14
 		fi
+		echo $password | grep : >/dev/null
+		if [ "$?" = "0" ]; then
+			echo "ERROR: Password contains a : character"
+			echo ""
+			exit 14
+		fi
 	fi
 	defhost=`hostname`
-	printf "Web server hostname (default $defhost): "
+	printf "Web server host name (default $defhost): "
 	if [ "$host" = "" ]; then
 		read host
 		if [ "$host" = "" ]; then
@@ -326,6 +353,8 @@ else
 	makeboot=$atboot
 
 	# Create webserver config file
+	echo $perl > $config_dir/perl-path
+	echo $var_dir > $config_dir/var-path
 	echo "***********************************************************************"
 	echo "Creating web server config files.."
 	cfile=$config_dir/miniserv.conf
@@ -349,6 +378,11 @@ else
 	if [ "$allow" != "" ]; then
 		echo "allow=$allow" >> $cfile
 	fi
+	if [ "$session" != "" ]; then
+		echo "session=$session" >> $cfile
+	else
+		echo "session=1" >> $cfile
+	fi
 
 	ufile=$config_dir/miniserv.users
 	if [ "$crypt" != "" ]; then
@@ -390,7 +424,12 @@ rm -f $config_dir/stop $config_dir/start
 echo "#!/bin/sh" >>$config_dir/start
 echo "echo Starting Webmin server in $wadir" >>$config_dir/start
 echo "trap '' 1" >>$config_dir/start
-echo "exec $wadir/miniserv.pl $config_dir/miniserv.conf &" >>$config_dir/start
+uname -a | grep -i 'HP/*UX' >/dev/null
+if [ $? = "0" ]; then
+	echo "exec $wadir/miniserv.pl $config_dir/miniserv.conf &" >>$config_dir/start
+else
+	echo "exec $wadir/miniserv.pl $config_dir/miniserv.conf" >>$config_dir/start
+fi
 
 echo "#!/bin/sh" >>$config_dir/stop
 echo "echo Stopping Webmin server in $wadir" >>$config_dir/stop
@@ -406,6 +445,7 @@ else
 fi
 $perl $wadir/copyconfig.pl $os_type $os_version $wadir $config_dir "" $allmods
 if [ "$upgrading" != 1 ]; then
+	# Store the OS and version
 	echo "os_type=$os_type" >> $config_dir/config
 	echo "os_version=$os_version" >> $config_dir/config
 	echo "real_os_type=$real_os_type" >> $config_dir/config
@@ -436,6 +476,17 @@ echo $ver > $config_dir/version
 echo "..done"
 echo ""
 
+if [ "$theme" = "" ]; then
+	theme=`cat $wadir/defaulttheme 2>/dev/null`
+fi
+oldthemeline=`grep "^theme=" $config_dir/config`
+oldtheme=`echo $oldthemeline | sed -e 's/theme=//g'`
+if [ "$theme" != "" ] && [ "$oldthemeline" = "" ] && [ -d $wadir/$theme ]; then
+	# Set a special theme if none was set before
+	echo "theme=$theme" >> $config_dir/config
+	echo "preroot=$theme" >> $config_dir/miniserv.conf
+fi
+
 if [ "$makeboot" = "1" ]; then
 	echo "Configuring Webmin to start at boot time.."
 	(cd $wadir/init ; WEBMIN_CONFIG=$config_dir WEBMIN_VAR=$var_dir $wadir/init/atboot.pl)
@@ -470,15 +521,15 @@ fi
 echo "Changing ownership and permissions .."
 chown -R root:bin $config_dir
 chmod -R og-rw $config_dir
-chmod 755 /etc/webmin/sendmail/config >/dev/null 2>&1
-chmod 755 /etc/webmin/sendmail/autoreply.pl >/dev/null 2>&1
+chmod 755 $config_dir/sendmail/config >/dev/null 2>&1
+chmod 755 $config_dir/sendmail/autoreply.pl >/dev/null 2>&1
 if [ "$nochown" = "" ]; then
 	chown -R root:bin $wadir
 	#chmod -R og-rwx $wadir
-	if [ $var_dir != "/var" ]; then
-		chown -R root:bin $var_dir
-		chmod -R og-rwx $var_dir
-	fi
+fi
+if [ $var_dir != "/var" ]; then
+	chown -R root:bin $var_dir
+	chmod -R og-rwx $var_dir
 fi
 echo "..done"
 echo ""
