@@ -12,7 +12,7 @@ while(<PKGINFO>) {
 	next if (/^\|/ || /^\+/);
 	if (/^ii..(\S+)\s+(\S+)\s+(.*)/) {
 		$packages{$i,'name'} = $1;
-		$packages{$i,'class'} = "";
+		$packages{$i,'class'} = &alphabet_name($1);
 		$packages{$i,'desc'} = $3;
 		$i++;
 		}
@@ -21,6 +21,15 @@ close(PKGINFO);
 return $i;
 }
 
+sub alphabet_name
+{
+return lc($_[0]) =~ /^[a-e]/ ? "A-E" :
+       lc($_[0]) =~ /^[f-j]/ ? "F-J" :
+       lc($_[0]) =~ /^[k-o]/ ? "K-O" :
+       lc($_[0]) =~ /^[p-t]/ ? "P-T" :
+       lc($_[0]) =~ /^[u-z]/ ? "U-Z" : "Other";
+}
+
 # package_info(package)
 # Returns an array of package information in the order
 #  name, class, description, arch, version, vendor, installtime
@@ -28,7 +37,7 @@ sub package_info
 {
 local $out = `dpkg --print-avail $_[0] 2>&1`;
 return () if ($? || $out =~ /Package .* is not available/i);
-local @rv = ( $_[0], "" );
+local @rv = ( $_[0], &alphabet_name($_[0]) );
 push(@rv, $out =~ /Description:\s+([\0-\177]*\S)/i ? $1
 						   : $text{'debian_unknown'});
 push(@rv, $out =~ /Architecture:\s+(\S+)/i ? $1 : $text{'debian_unknown'});
@@ -133,10 +142,11 @@ print "<input type=radio name=downgrade value=0 checked> $text{'no'}</td> </tr>\
 # Installs the package in the given file, with options from %in
 sub install_package
 {
-local $args = ($in{'depends'} ? " --force-depends" : "").
-	      ($in{'conflicts'} ? " --force-conflicts" : "").
-	      ($in{'overwrite'} ? " --force-overwrite" : "").
-	      ($in{'downgrade'} ? " --force-downgrade" : "");
+local $in = $_[2] ? $_[2] : \%in;
+local $args = ($in->{'depends'} ? " --force-depends" : "").
+	      ($in->{'conflicts'} ? " --force-conflicts" : "").
+	      ($in->{'overwrite'} ? " --force-overwrite" : "").
+	      ($in->{'downgrade'} ? " --force-downgrade" : "");
 local $out = &backquote_logged("dpkg --install $args $_[0] 2>&1 </dev/null");
 if ($?) {
 	return "<pre>$out</pre>";
@@ -157,7 +167,12 @@ return undef;
 
 sub package_system
 {
-return "Debian DPKG";
+return $text{'debian_manager'};
+}
+
+sub package_help
+{
+return "dpkg";
 }
 
 1;
