@@ -156,8 +156,9 @@ print "<input type=radio name=force value=0 checked> $text{'no'}</td> </tr>\n";
 # Installs the package in the given file, with options from %in
 sub install_package
 {
-local $args = ($in{"scripts"} ? " -I" : "").
-	      ($in{"force"} ? " -f" : "");
+local $in = $_[2] ? $_[2] : \%in;
+local $args = ($in->{"scripts"} ? " -I" : "").
+	      ($in->{"force"} ? " -f" : "");
 local $out = &backquote_logged("pkg_add $args $_[0] 2>&1");
 if ($?) {
 	return "<pre>$out</pre>";
@@ -176,9 +177,12 @@ return undef;
 
 sub package_system
 {
-return "FreeBSD Package Manager";
+return &text('bsd_manager', "FreeBSD");
 }
 
-
+sub package_help
+{
+return "pkg_add pkg_info pkg_delete";
+}
 
 1;
