@@ -84,9 +84,10 @@ print &file_chooser_button("root", 1); print "</td> </tr>\n";
 sub install_package
 {
 local(@opts, %seen, $wf, $rv, $old_input);
-if ($in{'root'} =~ /^\/.+/) {
-	if (!(-d $in{'root'})) { &error(&text('pkgadd_eroot', $in{'root'})); }
-	@opts = ("-R", $in{'root'});
+local $in = $_[2] ? $_[2] : \%in;
+if ($in->{'root'} =~ /^\/.+/) {
+	if (!(-d $in->{'root'})) { &error(&text('pkgadd_eroot', $in->{'root'})); }
+	@opts = ("-R", $in->{'root'});
 	}
 local ($ph, $ppid) = &foreign_call("proc", "pty_process_exec_logged",
 				   "pkgadd", "-d", $_[0], @opts, $_[1]);
@@ -240,7 +241,12 @@ if ($_[0] =~ /following packages:\n(((\s+.*\n)|\n)+)/)
 
 sub package_system
 {
-return "System V Package Manager";
+return $text{'pkgadd_manager'};
+}
+
+sub package_help
+{
+return "pkgadd pkginfo pkgchk pkgrm";
 }
 
 1;
