@@ -7,9 +7,10 @@ sub list_packages
 {
 local($i, $list); $i = 0;
 $list = @_ ? join(' ', @_) : "-a";
-open(RPM, "rpm -q $list --queryformat \"%{NAME}-%{VERSION}\\n%{GROUP}\\n%{SUMMARY}\\n\\n\" |");
+open(RPM, "rpm -q $list --queryformat \"%{NAME}\\n%{VERSION}-%{RELEASE}\\n%{GROUP}\\n%{SUMMARY}\\n\\n\" |");
 while($packages{$i,'name'} = <RPM>) {
 	chop($packages{$i,'name'});
+	chop($packages{$i,'version'} = <RPM>);
 	chop($packages{$i,'class'} = <RPM>);
 	while(<RPM>) {
 		s/\r|\n/ /g;
@@ -28,7 +29,7 @@ return $i;
 sub package_info
 {
 local(@rv, @tmp, $d);
-open(RPM, "rpm -q $_[0] --queryformat \"%{NAME}-%{VERSION}\\n%{GROUP}\\n%{ARCH}\\n%{VERSION}\\n%{VENDOR}\\n%{INSTALLTIME}\\n\" 2>/dev/null |");
+open(RPM, "rpm -q $_[0] --queryformat \"%{NAME}\\n%{GROUP}\\n%{ARCH}\\n%{VERSION}-%{RELEASE}\\n%{VENDOR}\\n%{INSTALLTIME}\\n\" 2>/dev/null |");
 @tmp = <RPM>;
 chop(@tmp);
 if (!@tmp) { return (); }
@@ -52,9 +53,17 @@ if (-d $_[0]) {
 	return @list ? 1 : 0;
 	}
 else {
-	# just a normal file
+	# just a normal file .. check if it is an RPM and not an SRPM
 	$out = `rpm -q -p $_[0] 2>&1`;
-	return $out !~ /does not appear|No such file/;
+	if ($out =~ /does not appear|No such file|with major numbers/i) {
+		return 0;
+		}
+	open(OUT, "rpm -q -p -l $_[0] 2>&1 |");
+	while(<OUT>) {
+		return 0 if (/^([^\/\s]+)\.spec$/);
+		}
+	close(OUT);
+	return 1;
 	}
 }
 
@@ -65,7 +74,7 @@ sub file_packages
 {
 if (-d $_[0]) {
 	local @rv;
-	open(RPM, "cd $_[0] ; rpm -q -p *.rpm --queryformat \"%{NAME}-%{VERSION} %{SUMMARY}\\n\" 2>&1 |");
+	open(RPM, "cd $_[0] ; rpm -q -p *.rpm --queryformat \"%{NAME} %{SUMMARY}\\n\" 2>&1 |");
 	while(<RPM>) {
 		chop;
 		push(@rv, $_) if (!/does not appear|query of.*failed/);
@@ -75,7 +84,7 @@ if (-d $_[0]) {
 	}
 else {
 	local($out);
-	$out = `rpm -q -p $_[0] --queryformat "%{NAME}-%{VERSION} %{SUMMARY}" 2>&1`;
+	$out = `rpm -q -p $_[0] --queryformat "%{NAME} %{SUMMARY}" 2>&1`;
 	return ($out);
 	}
 }
@@ -87,7 +96,7 @@ $wide_install_options = 1;
 sub install_options
 {
 print "<tr>\n";
-print &yesno_input($text{'rpm_upgrade'}, "upgrade", 1, 0);
+print &yesno_input($text{'rpm_upgrade'}, "upgrade", 1, 0, 1);
 print &yesno_input($text{'rpm_replacepkgs'}, "replacepkgs", 1, 0);
 print "</tr>\n";
 
@@ -101,6 +110,11 @@ print &yesno_input($text{'rpm_noscripts'}, "noscripts", 0, 1);
 print &yesno_input($text{'rpm_excludedocs'}, "excludedocs", 0, 1);
 print "</tr>\n";
 
+print "<tr>\n";
+print &yesno_input($text{'rpm_notriggers'}, "notriggers", 0, 1);
+print &yesno_input($text{'rpm_ignoresize'}, "ignoresize", 0, 1);
+print "</tr>\n";
+
 print "<tr>\n";
 print &yesno_input($text{'rpm_replacefiles'}, "replacefiles", 1, 0);
 print "<td align=right>",&hlink("<b>$text{'rpm_root'}</b>","root"),"</td>\n";
@@ -110,27 +124,28 @@ print &file_chooser_button("root", 1),"</td> </tr>\n";
 
 sub yesno_input
 {
-local $cy = $_[2] ? "" : "checked";
-local $cn = $_[3] ? "" : "checked";
+local $cy = $_[2] ^ $_[4] ? "" : "checked";
+local $cn = $_[3] ^ $_[4] ? "" : "checked";
 return  "<td align=right>".&hlink("<b>$_[0]</b>", $_[1])."</td> <td>\n".
 	"<input type=radio name=$_[1] value=$_[2] $cy> $text{'yes'}\n".
 	"<input type=radio name=$_[1] value=$_[3] $cn> $text{'no'}</td>\n";
 }
 
-# install_package(file, package)
+# install_package(file, package, [&inputs])
 # Install the given package from the given file, using options from %in
 sub install_package
 {
 local $file = $_[0];
+local $in = $_[2] ? $_[2] : \%in;
 foreach $o ('oldpackage', 'replacefiles', 'replacepkgs', 'noscripts',
-	    'excludedocs', 'nodeps', 'upgrade') {
-	if ($in{$o}) { $opts .= " --$o"; }
+	    'excludedocs', 'nodeps', 'upgrade', 'notriggers', 'ignoresize') {
+	if ($in->{$o}) { $opts .= " --$o"; }
 	}
-if ($in{'root'} =~ /^\/.+/) {
+if ($in->{'root'} =~ /^\/.+/) {
 	if (!(-d $in{'root'})) {
-		return &text('rpm_eroot', $in{'root'});
+		return &text('rpm_eroot', $in->{'root'});
 		}
-	$opts .= " --root $in{'root'}";
+	$opts .= " --root $in->{'root'}";
 	}
 if (-d $file) {
 	# Find the package in the directory
@@ -138,14 +153,14 @@ if (-d $file) {
 	opendir(DIR, $file);
 	while($f = readdir(DIR)) {
 		next if ($f !~ /\.rpm$/);
-		($out = `rpm -q -p $file/$f --queryformat '%{NAME}-%{VERSION}' 2>&1`) =~ s/\r|\n//g;
+		($out = `rpm -q -p $file/$f --queryformat '%{NAME}' 2>&1`) =~ s/\r|\n//g;
 		if ($out eq $_[1]) {
 			$file = "$file/$f";
 			last;
 			}
 		}
 	closedir(DIR);
-	&error("RPM $_[1] not found : $out") if ($file eq $_[0]);
+	&error(&text('rpm_erpm', $_[1], $out)) if ($file eq $_[0]);
 	}
 local $temp = &tempname();
 local $rv = &system_logged("rpm -i $opts $file >$temp 2>&1");
@@ -167,13 +182,13 @@ open(RPM, "rpm -V $_[0] |");
 while(<RPM>) {
 	/^(.{8}) (.) (.*)$/;
 	if ($1 eq "missing ") {
-		$errs{$3} = "Missing";
+		$errs{$3} = $text{'rpm_missing'};
 		}
 	else {
 		$epath = $3;
 		@w = grep { $_ ne "." } split(//, $1);
 		$errs{$epath} =
-			join("\n", map { "Failed $etype{$_} check" } @w);
+			join("\n", map { &text('rpm_checkfail', $etype{$_}) } @w);
 		}
 	}
 close(RPM);
@@ -204,7 +219,7 @@ sub installed_file
 {
 local($pkg, @w, $_);
 undef(%file);
-$pkg = `rpm -q -f $_[0] --queryformat "%{NAME}-%{VERSION}\\n" 2>&1`;
+$pkg = `rpm -q -f $_[0] --queryformat "%{NAME}\\n" 2>&1`;
 if ($pkg =~ /not owned/ || $?) { return 0; }
 @pkgs = split(/\n/, $pkg);
 open(RPM, "rpm -q $pkgs[0] -l --dump |");
@@ -229,11 +244,21 @@ close(RPM);
 return 1;
 }
 
-# delete_package(package)
+# delete_options(package)
+# Outputs HTML for package uninstall options
+sub delete_options
+{
+print "<b>$text{'delete_nodeps'}</b>\n";
+print "<input type=radio name=nodeps value=1> $text{'yes'}\n";
+print "<input type=radio name=nodeps value=0 checked> $text{'no'}<br>\n";
+}
+
+# delete_package(package, [&options])
 # Attempt to remove some package
 sub delete_package
 {
-$out = &backquote_logged("rpm -e $_[0] 2>&1");
+local $opts = $_[1]->{'nodeps'} ? " --nodeps" : "";
+$out = &backquote_logged("rpm -e $opts $_[0] 2>&1");
 if ($out) { return "<pre>$out</pre>"; }
 return undef;
 }
@@ -243,10 +268,15 @@ sub package_system
 return "RPM";
 }
 
-%etype = (	"5", "MD5",	"S", "file size",
-		"L", "symlink",	"T", "modification time",
-		"D", "device",	"U", "user ownership",
-		"M", "permissions", "G", "group ownership" );
+sub package_help
+{
+return "rpm";
+}
+
+%etype = (	"5", $text{'rpm_md5'},	"S", $text{'rpm_fsize'},
+		"L", $text{'rpm_sym'},	"T", $text{'rpm_mtime'},
+		"D", $text{'rpm_dev'},	"U", $text{'rpm_user'},
+		"M", $text{'rpm_perm'},	"G", $text{'rpm_group'} );
 
 1;
 
