@@ -223,8 +223,9 @@ print "<td colspan=3><input name=root size=30 value='/'> ",
 # Installs the package in the given file, with options from %in
 sub install_package
 {
-return $text{'slack_eroot'} if (!-d $in{'root'});
-$ENV{'ROOT'} = $in{'root'};
+local $in = $_[2] ? $_[2] : \%in;
+return $text{'slack_eroot'} if (!-d $in->{'root'});
+$ENV{'ROOT'} = $in->{'root'};
 local $out = &backquote_logged("installpkg $_[0] 2>&1");
 if ($?) {
 	return "<pre>$out</pre>";
@@ -241,11 +242,14 @@ if ($?) { return "<pre>$out</pre>"; }
 return undef;
 }
 
-
-
 sub package_system
 {
-return "Slackware Package Manager";
+return $text{'slack_manager'};
+}
+
+sub package_help
+{
+return "installpkg removepkg";
 }
 
 1;
