@@ -44,6 +44,21 @@ for($i=4; $i<@_; $i+=2) {
 return $rv."</td>\n";
 }
 
+# select_input(text, name, &config, default, [display, option]+)
+# Like choice_input, but uses a drop-down select field
+sub select_input
+{
+local($v, $vv, $rv, $i);
+$v = &find_config($_[1], $_[2]);
+$vv = $v ? $v->{'value'} : $_[3];
+$rv = "<td><b>$_[0]</b></td> <td valign=top><select name=$_[1]>";
+for($i=4; $i<@_; $i+=2) {
+	$rv .= "<option value=\"".$_[$i+1]."\" ".
+		($vv eq $_[$i+1] ? "selected" : "")."> $_[$i]\n";
+	}
+return $rv."</select></td>\n";
+}
+
 # save_choice(name, default, &config)
 # Save a selection from choice_input()
 sub save_choice
@@ -278,7 +293,8 @@ else {
 	      "proto", $text{'lib_aclup1'},
 	      "method", $text{'lib_aclrm'},
 	      "browser", $text{'lib_aclbr'},
-	      "user", $text{'lib_aclpl'} );
+	      "user", $text{'lib_aclpl'},
+	      "arp", $text{'lib_aclarp'} );
 if ($squid_version >= 2.0) {
 	$acl_types{'src_as'} = $text{'lib_aclsan'};
 	$acl_types{'dst_as'} = $text{'lib_acldan'};
@@ -289,12 +305,19 @@ if ($squid_version >= 2.0) {
 if ($squid_version >= 2.2) {
 	$acl_types{'ident'} = $text{'lib_aclru'};
 	$acl_types{'myip'} = $text{'lib_aclpia'};
+	delete($acl_types{'user'});
 	}
 if ($squid_version >= 2.3) {
 	$acl_types{'maxconn'} = $text{'lib_aclmc'};
 	$acl_types{'myport'} = $text{'lib_aclpp'};
 	$acl_types{'snmp_community'} = $text{'lib_aclsc'};
-	delete($acl_types{'user'});
+	}
+if ($squid_version >= 2.4) {
+	$acl_types{'req_mime_type'} = $text{'lib_aclrmt'};
+	$acl_types{'proxy_auth_regex'} = $text{'lib_aclear'};
+	}
+if ($squid_version >= 2.5) {
+	$acl_types{'ident_regex'} = $text{'lib_aclrur'};
 	}
 
 # restart_button()
@@ -314,10 +337,10 @@ if (open(PID, $pidfile)) {
 $args = "redir=".&urlize(&this_url())."&pid=$pid";
 if ($pid && kill(0, $pid)) {
 	return "<a href=\"restart.cgi?$args\">$text{'lib_buttac'}</a><br>\n".
-	       "<a href=\"stop.cgi?$args\">$text{'lib_buttss'}</a>\n";
+	       ($access{'start'} ? "<a href=\"stop.cgi?$args\">$text{'lib_buttss'}</a>\n" : "");
 	}
 else {
-	return "<a href=\"start.cgi?$args\">$text{'lib_buttss1'}</a><br>\n";
+	return $access{'start'} ? "<a href=\"start.cgi?$args\">$text{'lib_buttss1'}</a><br>\n" : "";
 	}
 close(PID);
 }
@@ -338,8 +361,9 @@ sub list_auth_users
 local(@rv, $lnum); $lnum = 0;
 open(USERS, $_[0]);
 while(<USERS>) {
-	if (/^([^:]+):(\S+)/) {
-		push(@rv, { 'user' => $1, 'pass' => $2, 'line' => $lnum });
+	if (/^(#*)([^:]+):(\S+)/) {
+		push(@rv, { 'user' => $2, 'pass' => $3,
+			    'enabled' => !$1, 'line' => $lnum });
 		}
 	$lnum++;
 	}
@@ -400,10 +424,10 @@ system("chown -Rf $_[0]:$_[1] ".join(" ",@list)." >/dev/null 2>&1");
 sub can_access
 {
 local @f = grep { $_ ne '' } split(/\//, $_[0]);
-return 1 if ($access{'root'} eq '/' || $_[0] eq '/');
+return 1 if ($access{'root'} eq '/');
 local @a = grep { $_ ne '' } split(/\//, $access{'root'});
 local $i;
-for($i=0; $i<@a && $i<@f; $i++) {
+for($i=0; $i<@a; $i++) {
 	return 0 if ($a[$i] ne $f[$i]);
 	}
 return 1;
