@@ -37,13 +37,14 @@ while($line = <CONF>) {
 			       'eline' => $lnum };
 		if ($act =~ /^\-(\/\S+)$/) {
 			$log->{'file'} = $1;
-			$log->{'sync'} = 1;
+			$log->{'sync'} = 0;
 			}
 		elsif ($act =~ /^\|(.*)$/) {
 			$log->{'pipe'} = $1;
 			}
 		elsif ($act =~ /^(\/\S+)$/) {
 			$log->{'file'} = $1;
+			$log->{'sync'} = 1;
 			}
 		elsif ($act =~ /^\@(\S+)$/) {
 			$log->{'host'} = $1;
@@ -122,7 +123,7 @@ sub log_line
 {
 local $d;
 if ($_[0]->{'file'}) {
-	$d = ($_[0]->{'sync'} ? "-" : "").$_[0]->{'file'};
+	$d = ($_[0]->{'sync'} ? "" : "-").$_[0]->{'file'};
 	}
 elsif ($_[0]->{'pipe'}) {
 	$d = '|'.$_[0]->{'pipe'};
@@ -139,5 +140,14 @@ else {
 return ($_[0]->{'active'} ? "" : "#").join(";", @{$_[0]->{'sel'}})."\t".$d;
 }
 
+# list_priorities()
+# Returns a list of all priorities
+sub list_priorities
+{
+return ( 'debug', 'info', 'notice', 'warning',
+	 'err', 'crit', 'alert', 'emerg' );
+}
+
+
 1;
 
