@@ -1,7 +1,7 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-require './time-lib.pl';
+do './time-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params)
 # Converts logged information from this module into human-readable form
@@ -11,7 +11,7 @@ local ($user, $script, $action, $type, $object, $p, $long) = @_;
 if ($action eq 'remote') {
 	local $tm = localtime($object);
 	return &text($long ? "log_remote_${type}_l" : "log_remote_${type}",
-		     "<i>$tm</i>", "<tt>$p->{'timeserver'}</tt>");
+	     "<i>$tm</i>", "<tt>".&html_escape($p->{'timeserver'})."</tt>");
 	}
 elsif ($action eq 'set') {
 	local $tm = localtime($object);
