@@ -43,6 +43,22 @@ while(1) {
 	}
 }
 
+# logged_in_users()
+# Returns a list of hashes containing details of logged-in users
+sub logged_in_users
+{
+local @rv;
+open(WHO, "who |");
+while(<WHO>) {
+	if (/^(\S+)\s+(\S+)\s+(\S+\s+\d+\s+\d+:\d+)\s+(\((\S+)\))?/) {
+		push(@rv, { 'user' => $1, 'tty' => $2,
+			    'when' => $3, 'from' => $5 });
+		}
+	}
+close(WHO);
+return @rv;
+}
+
 # encrypt_password(password)
 sub encrypt_password
 {
