@@ -3,10 +3,10 @@
 
 # passfiles_type()
 # Returns 0 for old-style passwords (/etc/passwd only), 1 for FreeBSD-style
-# (/etc/master.passwd) and 2 for SysV (/etc/passwd & /etc/shadow)
+# (/etc/master.passwd) and 2 for SysV (/etc/passwd & /etc/shadow),3 for Unixware
 sub passfiles_type
 {
-return 2;
+return 3;
 }
 
 # groupfiles_type()
@@ -46,8 +46,10 @@ while(1) {
 # encrypt_password(password)
 sub encrypt_password
 {
-local $salt = chr(int(rand(26))+65) . chr(int(rand(26))+65);
-return crypt($_[0], $salt);
+# We let the passwd programa to do encryption on Unixware.
+# local $salt = chr(int(rand(26))+65) . chr(int(rand(26))+65);
+# return crypt($_[0], $salt);
+return $_[0];
 }
 
 1;
