@@ -1,7 +1,7 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-require './webmin-lib.pl';
+do './webmin-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params)
 # Converts logged information from this module into human-readable form
@@ -11,6 +11,9 @@ local ($user, $script, $action, $type, $object, $p) = @_;
 if ($action eq 'install') {
 	return &text('log_install', "<i>$p->{'desc'}</i>");
 	}
+elsif ($action eq 'tinstall') {
+	return &text('log_tinstall', "<i>$p->{'desc'}</i>");
+	}
 elsif ($action eq 'clone') {
 	return &text('log_clone', "<i>$p->{'desc'}</i>",
 				  "<i>$p->{'dstdesc'}</i>");
@@ -21,6 +24,10 @@ elsif ($action eq 'delete') {
 elsif ($action eq 'upgrade') {
 	return &text('log_upgrade', $p->{'version'});
 	}
+elsif ($action eq 'theme') {
+	return $p->{'theme'} ? &text('log_theme', "<tt>$p->{'theme'}</tt>")
+			     : $text{'log_theme_def'};
+	}
 elsif ($text{"log_$action"}) {
 	return $text{"log_$action"};
 	}
